/* Data types and global declarations 
   for the Cheriyan-Hagerup Max-flow algorithm */

/* Copyright:
	This program was written by 

	Tamas Badics, 1991,
	Rutgers University, RUTCOR
	P.O.Box 5062
	New Brunswick, NJ, 08903
	e-mail: badics@rutcor.rutgers.edu
 
	The code may be used and modified for not-for-profit use.
	This notice must be remained.	
====================================================================*/

#ifndef _MAXFLOW_H
#define _MAXFLOW_H


/*==================================================================*/
/*========  NETWORK type for the algorithm  ========================*/
/*========  see maxflow_typedef.h           ========================*/
/*==================================================================*/

#include <stdio.h>
#include "maxflow_typedef.h"
#include "queue.h"

/*==================================================================*/
/*======  Global variables          ================================*/
/*==================================================================*/

extern DOUBLE Delta;
extern int Vertnum, Edgenum;
extern int It;
extern vertex * Source, * Sink;
extern int * Labarr;
extern queue * Q, * Qv;
extern char Gap;
extern int Inf, Bigrel;


/*==================================================================*/
/*======  Macro definitions         ================================*/
/*==================================================================*/
/*
#define PRINT(a,b)
*/

#define PRINT(quiet, args) 	if (!(quiet)) printf args

#define OUT_EDGE(V, E) ((E)->from == V)

#define RESCAP(V, E) ((OUT_EDGE(V, E)) ? ((E)->cap - (E)->flow) : (E)->flow)

#define OTHER(V, E)  (OUT_EDGE(V, E) ? (E)->to : (E)->from)

#define LAST_EDGE(V) (&((V)->edgelist[(V)->degree -1]))
/*==================================================================*/
/*========   Function declarations   ===============================*/
/*==================================================================*/

int maxflow(network *n, controll *cont);
                              /* Find the max-flow in the initialized 
								 network n using the GOLD alg. 
								 Return the # of iterations*/
void pushrelabel(vertex *v);
void discharge(vertex *v);
void push(vertex *v, edge *e);

void relabel(vertex *v);
                            /* Relabel v and do permutation on
							   v->edgelist */

int big_relabel();  
                           /* Bfs from the sink, searching for the
							  min_cut if exists already. Returns
							  true if mincut is found.
							  Finally search from the source on 
							  the left partition of the cut 
							  (not necessarely min).
							  Relabels all the vertices.
							  Does an init_heap, and init_tree.
							  Calling freqency is heuristic.*/

int search();
                        /* Bfs from source using q, and start_label */

void print_report(void);

void  permute_all();
#endif
