/*=================================================================*/
/*== Big relabel    ===============================================*/
/*=================================================================*/

/* Copyright:
	This program was written by 

	Tamas Badics, 1991,
	Rutgers University, RUTCOR
	P.O.Box 5062
	New Brunswick, NJ, 08903
	e-mail: badics@rutcor.rutgers.edu
 
	The code may be used and modified for not-for-profit use.
	This notice must be remained.	
====================================================================*/

#include <stdio.h>
#include "_maxflow.h"
#include "queue.h"
#include "netio.h"

/*===============================================================*/
static int Maxlev;

int big_relabel(n, big)
     network *n;
     int big;       
                        /* Bfs from the sink, searching for the
						   min_cut if exists already. Returns
						   true if mincut is found.
						   Finally search from the source on the left
						   partition of the cut (not necessarely min).
						   Relabels all the vertices.
						   Does an init_heap, and init_tree.
						   Calling freqency is heuristic.*/
{
	int i;
	int min_cut;
	
#ifdef COUNT
	Bigrel++;
#endif
	
	if (Gap)  Labarr[0] = Maxlev = 0;

	for (i = 1; i <= Vertnum; i++){ 
		n->verts[i].label = Inf;
		if (Gap)  Labarr[i] = 0;
	}
	
	if (big)
	  min_cut = search(Sink, Q, 0);
	else
	  min_cut = 0;

	if (Gap)  Labarr[0] = Maxlev;

	if (Source->label < Vertnum)
	  Source->label = Vertnum;
	else	
	  search(Source, Q, Vertnum);
		
	return (min_cut);
}

/*==================================================================*/
int search(source, q, start_label)
     vertex *source;
     queue *q;
     int start_label;
{
	int i, level, mincut;
	vertex * v, * w;
	edge * e, **ve;

    level = start_label;
	mincut = 1;
	
	init_queue(q);
	enqueue(q, source);
    enqueue(q, NULL);

    while (1){
		v = (vertex *)dequeue(q);
		if (v == NULL){
			if (IS_EMPTY_QUEUE(q))
			  break;

			level++;
			enqueue(q, NULL);
	    }else{
			v->label = level;

			if (Gap && level < Vertnum) (Labarr[level])++;

			ve = v->edgelist;
			for (i = 0; i < v->degree; i++){
				e = ve[i];
				
				if (v == e->from){
					if (((w = e->to)->label == Inf) 
						&& (e->flow > 0))
					  {	
						  if (mincut && w->excess > 0)
							mincut = 0;

						  w->label = -1;
						  enqueue(q, w);
					  }
				}else{ 
					if (((w = e->from)->label == Inf) 
						&& (e->cap > e->flow))
					  {	
						  if (mincut && w->excess > 0)
							mincut = 0;

						  w->label = -1;
						  enqueue(q, w);
					  }
				}
			}
		}
	}
	Maxlev = level;

	return (mincut);
}
/*=================================================================*/
int check_maxflow(n)
     network *n;
{
	int i, j, cutcap, level;
	vertex  * v, * w;
	edge * e;

	for (i = 1; i <= Vertnum; i++)
	  n->verts[i].label = Inf;

	/* A breadth first search from the source through the 
	   residual graph */

    level = 1;

	init_queue(Q);
	enqueue(Q, Source);
    enqueue(Q, NULL);

    while (1){
		v = (vertex *)dequeue(Q);
		if (v == NULL){
			if (IS_EMPTY_QUEUE(Q))
			  break;
			level++;
			enqueue(Q, NULL);
	    }else{
			v->label = level;
			
			for (i = 0; i < v->degree; i++){
				e = v->edgelist[i];
				if ((w = OTHER(v, e))->label == Inf 
					&& RESCAP(v, e) > 0)
				  {	
					  w->label = -1;
					  enqueue(Q, w);
				  }
			}
	    }
	} /* end of search */
	
	for (i = 1, cutcap = 0; i <= Vertnum; i ++){  
		                                 /* counting the minimum
											cut value */
		v = &(n->verts[i]);
		if (v->label < Inf){
			for (j = 0; j < v->degree; j++){
				e = v->edgelist[j];
				if (e->flow > e->cap)
				  print_err("Err: Capacity bound is hurt!", 0);
				if (OUT_EDGE(v, e) && OTHER(v, e)->label == Inf)
				  cutcap += e->cap;
			}
		} 
	}

	if (n->maxflow == cutcap){ 
		return 1;
	}else
	  return 0;
}

