/*=================================================================*/
/*== Initialization ===============================================*/
/* PLUS SOURCE ADDED */
/*=================================================================*/
/* Copyright:
	This program was written by 

	Tamas Badics, 1991,
	Rutgers University, RUTCOR
	P.O.Box 5062
	New Brunswick, NJ, 08903
	e-mail: badics@rutcor.rutgers.edu
 
	The code may be used and modified for not-for-profit use.
	This notice must be remained.	
====================================================================*/

#include <stdio.h>
#include <stdlib.h>
#include "_maxflow.h"
#include "netio.h"
#include "macros.h"

#define FEAS 1
#define NOFEAS 0

/*=== Global variable definitions =========*/

vertex * Source, * Sink;
int Vertnum, Edgenum, Itnum, Inf, Flow_bound;

char Gap;

DOUBLE Delta;

queue * Q, *Qv;
int * Labarr;    /* array of existing labels [0..Vertnum]:
					Labarr[i] = # of vertices whose label is i */

#ifdef DEBUG
network * Net;
#endif

/*=========================================*/

int maxflow_warm_start(network *n, int need_preflow, controll *cont);
void init_preflow(network *n);
void search_cut(network *n, queue *q);

/*=================================================================*/
int maxflow_cold_start(char *prob_file, network **n, controll *cont)
                     
                 
                    
                           /* Reads the problem from the file.
							  Then start the GOLD algorithm.
							  Return false if infeasible.
							  Return the network via n.
							  See maxflow_typedef.h for controll.*/
{
	int quiet = cont->quiet;
	int ret;
	
	PRINT( quiet, ("I am reading the network...\n"));
	*n = read_net(prob_file);

#ifdef DEBUG
	Net = *n;
#endif
	
	ret = maxflow_warm_start(*n, 1, cont);
	
	return ret;
}	

/*=================================================================*/
int maxflow_warm_start(network *n, int need_preflow, controll *cont)
                
                      
                    
	                          /* After the allocations, with
								 a prepared network * n, and
								 a given preflow, it will
								 start the maxflow algorithm */
{
	vertex * nv, * v; 
	int i; 
	int quiet = cont->quiet;
	
	PRINT( quiet, ("Initialize is started\n"));
	
	/*---- Fill up the global variables ----*/
	n->maxflow = 0;
	Source = n->source;
	Sink = n->sink;
	Vertnum = n->vertnum;
	Edgenum = n->edgenum;
	Itnum = 0;
	Inf = 3 * Vertnum;

	/*---- Default values of the frequencies ----*/
	if (cont->stall_freq == -1)
		cont->stall_freq = Vertnum / 2;
	if (cont->relab_freq == -1)
	  cont->relab_freq = (int)Edgenum;
	if (cont->cut_freq == -1)
	  cont->cut_freq = (char)0;
	if (cont->gap == -1)
	  cont->gap = (char)1;

	if (need_preflow)
	  init_preflow(n);
	else{ 
		Delta = 0; 
		/*---- Counting the maximal excess for Delta ----------*/
		for( i = 1, nv = n->verts; i <= Vertnum; i++){
			v = &(nv[i]);
			if (v->excess > Delta && v != Sink && v!= Source)
			  Delta = v->excess;
		}
	}

	if (cont->gap) Labarr = (int *)calloc(Vertnum, sizeof(int));

	PRINT( quiet, ("Initialize is finished\n"));
	
	if (Delta == 0 && Sink->excess == 0)     
	  return NOFEAS;              /* No feasible solution */ 

	Q = new_queue(2*Vertnum);
	Flow_bound = (*(Source->edgelist))->flow;
	search_cut(n, Q);
	(*(Source->edgelist))->cap = Flow_bound + 1;
	
	if (big_relabel(n, 1) && Sink->excess == 0)
	  return NOFEAS;              /* No feasible solution */ 

	(*(Source->edgelist))->cap = Flow_bound;
	(*(Source->edgelist))->flow = Flow_bound;
	(*(Source->edgelist))->to->excess = Flow_bound;
	Source->excess = -Flow_bound;
	
	Qv = new_queue(Vertnum);
	enqueue(Qv, (*(Source->edgelist))->to);

	Itnum = maxflow(n, cont);
	return FEAS;
}
/*==================================================================*/
void init_preflow(network *n)
                
	                 /* Add all the capacities going into the Sink,
						and assign this cut-value as capacity to
						the artificial edge from the new source */
{
	int i;
	vertex * v;
	edge * e;
	
	Delta = 0;

	v = Sink;
	
	for( i = 0; i < v->degree; i++){
		e = v->edgelist[i];
		if( !OUT_EDGE(v, e)){
			Delta += e->cap;
		}
	}

	Source->edgelist[0]->flow = Delta;
	Source->edgelist[0]->cap = Delta;
	Source->edgelist[0]->to->excess = Delta;
	Source->excess = -Delta;
	
}
/*==================================================================*/
void search_cut(network *n, queue *q)
{
	int i, level;
	double cutcap = 0;
	vertex * v, * w;
	edge * e, **ve;
	
	for( i = 1; i <= Vertnum; n->verts[i].label = Inf, i++);

    level = 0;
	init_queue(q);
	enqueue(q, Sink);
    enqueue(q, NULL);

    while (1){
		v = (vertex *)dequeue(q);
		if (v == NULL){
			if (IS_EMPTY_QUEUE(q))
			  break;

			level++;
			if (cutcap < Flow_bound)
			  Flow_bound = (int)cutcap;
			cutcap = 0;
			enqueue(q, NULL);
	    }else{
			v->label = level;
			ve = v->edgelist;
			for (i = 0; i < v->degree; i++){
				e = ve[i];
				
				if (v == e->to){
					if ((w = e->from)->label == Inf || w->label == -1)
					  { 
						  cutcap += (e->cap);
						  if (w->label == Inf){ 
							  w->label = -1;
							  enqueue(q, w);
						  }
					  }
				}
			}
		}
	}
}



