/* acceptFlowData.h */

#ifndef _TERMDATA_H_
#define _TERMDATA_H_

class acceptFlowData
{
 public:

  /* CONSTRUCTOR */
  acceptFlowData(/*in*/ int id, /*in*/ int safid, 
                 /*in*/ int afid, /*in*/ int amount);
  /* Constructor 
   * PRE:  id is the vertex id of the vertex that sent the flow.
   *       itemNumber is the push identifier.  id and itemNumber
   *           should uniquely identify an accepted flow.
   *       amount is the amount of flow that needs to be pushed.
   * POST: A proper termData object is instantiated.
   *****************************************************************/

  /* MODIFICATION MEMBER FUNCTIONS for termData */
  void modifyAmount(int amount);
  /* adds the amount to m_amount
   * PRE: 0 <= amount + m_amount <= amount originally supplied
   *****************************************************************/

  acceptFlowData operator++();
  acceptFlowData operator++(int x);
  /* This operator is used to record that another vertex has been 
   *     pushed to.
   * PRE:
   * POST:
   *****************************************************************/

  acceptFlowData operator--();
  acceptFlowData operator--(int x);
  /* This operator is used to record that another vertex has sent 
   *     a termination.
   * PRE:
   * POST:
   *****************************************************************/

  /* CONSTANT MEMBER FUNCTIONS for termData */
  bool done() const;
  /* done returns true when all the flow has been pushed, and
   *     all the responses have been received.
   * PRE:  NONE
   * POST: returns true if done; otherwise, returns false.
   *****************************************************************/

  int getAmount() const;
  /* returns the amount of flow that still needs to be pushed
   * PRE:  NONE
   * POST: the value of m_amount is returned.
   *****************************************************************/

  int getAFId() const;
  int getSourceId() const;
  int getSourceAFId() const;

 private:
  int m_amount, m_numResponses, m_sourceId, m_sourceAFId,
      m_acceptFlowId;
};
#endif
