#include <stdlib.h>
#include <fstream.h>
#include <iostream.h>
#include <list>
#include <map>
#include <string>

struct flowData
{
  int from, to;
  double amount;
};

#define OPS_NUM 4
typedef map<int, double> FMAP;
typedef list<flowData> FLIST;

/* function prototypes */
int look_up(char* s);
void readSystem(FMAP &fm, int id, const string& fileName);
void readFlowData( FLIST &fl, int id, const string& fileName);

int main(int argc, char** argv)
{
  /* variables */
  FMAP fm;
  FLIST fl;
  FMAP::iterator fmitr;
  FLIST::iterator flitr;
  string confFileName, flowFileName;
  int id = 0;
  int i;

  /* process the command line arguments */
  for (i = 1; i < argc; i++)
    {
      switch (look_up(argv[i]))
	{
	case 0: // -conf
	  i++;
	  if (i < argc)
	    confFileName.assign(argv[i]);
	  break;

	case 1: // -id
	  i++;
	  if (i < argc)
	    id = atoi(argv[i]);
	  break;

	case 2: // -fd
	case 3: // --flowdata
	  i++;
	  if (i < argc)
	    {
	      flowFileName.assign(argv[i]);
	    }
	  break;

	default:
	  break;
	}
    } /* end of for loop cycling through command line inputs */

  /* check to make sure the command line inputs were all there */
  if ( ! confFileName.size() || !id || ! flowFileName.size())
    {
      cerr << "Usage:  " << argv[0] << " -conf network.conf -id virtualSinkId " << endl
	   << "\t-fd flowFileName.simple" << endl;
    }
  else {

    /* read in the two files */
    readSystem(fm, id, confFileName);
    readFlowData(fl, id, flowFileName);

    cout << "Done reading in the files." << endl
         << "The map has " << fm.size() << " items." << endl
         << "The list has " << fl.size() << " items." << endl;

    /* scan through the list to make sure all the loads were met */
    for (flitr = fl.begin(); flitr != fl.end(); flitr++)
      {
	fmitr = fm.find(flitr->from);
	if (fmitr != fm.end())
	  {
	    if (fmitr->second != flitr->amount)
	      cout << "The load at bus " << flitr->from << " was not met." << endl
		   << "\tCapacity:  " << fmitr->second << endl
		   << "\tFlow:      " << flitr->amount << endl;
	  }
	else {
	  cerr << "The arc from " << flitr->from << " to " << flitr->to
	       << " was not in the config file." << endl;
	}
      }
  }

  cout << argv[0] << " has finished." << endl;
}

int look_up(char* s)
{

  int i;
  char* ops[OPS_NUM];

  ops[0] = "-conf";
  ops[1] = "-id";
  ops[2] = "-fd";
  ops[3] = "--flowdata";

  for (i = 0; i < OPS_NUM; i++)
    {
      if (strcmp(ops[i], s) == 0)
	{
	  return i;
	}
    }
  return -1;
}

void readSystem(FMAP &fm, int id, const string& fileName)
{
  /* variables */
  ifstream data;
  flowData fd;
  char readChar;

  /* initialize variables */
  data.open(fileName.data());

  if (data)
    {
      /* loop through the entire file */
      while (! data.eof() )
	{
	  /* only process arc lines */
	  readChar = data.get();

	  switch (readChar)
	    {
	    case 'a':
	      /* read in the data */
	      data >> fd.from >> fd.to >> fd.amount;
	      data.ignore(1024, '\n');

	      /* only record the virtual source data */
	      if ( fd.to == id )
		{
		  fm.insert(FMAP::value_type(fd.from, fd.amount));
		}
	      break;

	    default:
	      data.ignore(1024, '\n');
	      break;
	    }
	}

      /* close the file */
      data.close();
    }
  else {
    /* report the error */
    cout << "Could not open " << fileName << endl;
    exit(EXIT_FAILURE);
  }
}

void readFlowData( FLIST &fl, int id, const string& fileName)
{
  /* variables */
  ifstream data;
  char readChar;
  flowData fd;

  /* initialize variables */
  data.open(fileName.data());

  if (data)
    {
      /* read through the file */
      while (! data.eof())
	{
	  data >> readChar;

	  /* only process flow lines */
	  switch (readChar)
	    {
	    case 'f':
	      /* process the flow line */
	      data >> fd.from >> fd.to >> fd.amount;
	      data.ignore(1024, '\n');

	      /* only keep the ones connected to the virutal sink */
	      if ( fd.to == id )
		{
		  fl.push_back(fd);
		}
	      break;

	    default:
	      data.ignore(1024, '\n');
	      break;
	    }
	}

      /* close the file */
      data.close();
    }
  else {
    cout << "Could not open " << fileName << endl;
    exit(EXIT_FAILURE);
  }
}
