/*  createConfig.cpp - This program was designed to create a 
 *     configuration file for the distributed max flow 
 *     algorithm.
 *
 *  programmer:     Austin Armbruster
 *  creation date:  1 November 2000
 *  last modified:  
 *
 *  modifications:
 */

#include <stdlib.h>
#include <iostream.h>
#include <iomanip.h>
#include <fstream.h>
#include <ctype.h>
#include <string.h>
#include <math.h>
#include <map>
#include <list>

#include "confVertex.h"
#include "confEdge.h"

#ifndef MAX
#define MAX(a,b) ((a)<(b) ? (b) : (a))
#endif

typedef map<int, confVertex> MICV;
typedef list<confEdge> CEL;

/* Function Prototypes */
bool addVertex(MICV &vmap, int &maxId);
bool addEdge(CEL &elist, const int &maxId);
bool setSource(MICV& vmap, const int& maxId);
bool setSink(MICV& vmap, const int& maxId);
bool writeFile(const MICV& vmap, const CEL& elist, const confVertex& contVert);
bool setControllerParams(confVertex& contVert);
bool readFile(MICV& vmap, CEL& elist, confVertex& contVert, int& maxId);
bool incrementByThousand(MICV& vmap, confVertex& contVert);
bool firstAvailablePorts(MICV& vmap, confVertex& contVert);
bool changeAllServers(MICV& vmap, confVertex& contVert);

int main(/*int argc, char** argv */)
{
  /* variables */
  MICV vmap;
  CEL elist;
  confVertex contVert;

  char readChar, choice;
  int maxId = 0;
  bool done = false;
  bool confModified = false;

  /* check the command line inputs */

  /* get the user's input */
  while (!done)
    {

      /* cout the menu */

      /* add vertex */
      cout << "Please choose an option:" << endl 
	   << "\t(1)  Add Vertex" << endl;

      /* add edge*/
      if (maxId > 1)
	cout << "\t(2)  Add Edge" << endl;
      else 
	cout << "\tCannot Create an Edge Yet" << endl;

      /* set source or sink */
      if (maxId > 0)
	cout << "\t(3)  Set Source" << endl
	     << "\t(4)  Set Sink" << endl;
      else 
	cout << "\tCannot Set a Source or Sink Yet" << endl;

      /* Set Controller Parameters */
      cout << "\t(5)  Set Controller Parameters" << endl;

      /* change server/port values */
      if ((elist.size() >= vmap.size()-1) && (elist.size() > 1)
	  && (contVert.getPort() != 0))
	cout << "\t(6)  Increment All Ports by 1000" << endl
	     << "\t(7)  Set Ports to First After Max in Configuration" << endl
	     << "\t(8)  Set All Servers to New Value" << endl;

      /* read in a file */
      if (vmap.size() == 0)
	cout << "\t(9)  Read in a Configuration File" << endl;
      else 
	cout << "\t(9)  Read in a Configuration File" << endl
	     << "\t\tWARNING:  This will destroy all current data!" << endl;

      /* write the file */
      if ((elist.size() >= vmap.size()-1) && (elist.size() > 1)
	  && (contVert.getPort() != 0))
	cout << "\t(0)  Write Configuration File" << endl;
      else 
	cout << "\t(0)  Write the Configuration File" << endl
	     << "\t\tIt is Incomplete." << endl; 

      /* Quit */
      cout << "\t(q)  Quit" << endl;

      /* read in the choice and then clear the rest of the input
       * through the new line character */
      cin >> choice;
      while ((readChar = cin.get()) != '\n')
	;

      switch(choice)
	{
	case '1':
	  confModified = addVertex(vmap, maxId) || confModified;
	  break;
	  /* end of add vertex */

	case '2': /* add edge */
	  confModified = addEdge(elist, maxId) || confModified;
	  break;
	  /*end of add edge */

	case '3': /* set source */
	  confModified = setSource(vmap, maxId) || confModified;
	  break;
	  /* end of set source */

	case '4': /* set sink */
	  confModified = setSink(vmap, maxId) || confModified;
	  break;
	  /* end of set sink */

	case '5': /* set control vertex */
	  confModified = setControllerParams(contVert) || confModified;
	  break;
	  /* end set control vertex */

	case '6': /* increment by 1000 */
	  confModified = incrementByThousand(vmap, contVert) || confModified;
	  break;

	case '7': /* set to first available */
	  confModified = firstAvailablePorts(vmap, contVert) || confModified;
	  break;

	case '8': /* set all servers */
	  confModified = changeAllServers(vmap, contVert) || confModified;
	  break;

	case '9': /* read a file */
	  confModified = readFile(vmap, elist, contVert, maxId) || confModified;
	  break;

	case '0': /* Write to file */
	  confModified = !(writeFile(vmap, elist, contVert)) && confModified;
	  break;

	case 'q': /* quit */
	case 'Q': /* quit */
	  cout << "Are you sure you want to quit?" << endl
	       << "(y/n):  ";
	  cin >> readChar;
	  if (readChar == 'y' || readChar == 'Y')
	    {
	      done = true;
	      if (confModified)
		{
		  cout << "The file has not been saved!  Save Now?" << endl
		       << "(y/n):  ";
		  cin >> readChar;
		  cin.ignore(1024, '\n');
		  if (readChar == 'y' || readChar == 'Y')
		    {
		      writeFile(vmap, elist, contVert);
		    }
		}
	    }
	  break;

	default:
	  break;
	}
    }

}

/*
 *  addVertex
 ********************************************************************/
bool addVertex(MICV& vmap, int &maxId)
{
  confVertex *cv;
  char readChar;
  int port;
  size_t length, serverLengthMax, portLengthMax;
  char *server, *portString, *newString;
  bool invalidInput = true;
  bool rtnValue = false;

  serverLengthMax = portLengthMax = 255;
  maxId++;

  /* read in the server */
  server = new char[serverLengthMax];
  server[0] = '\0';
  length = 0;
  cout << endl << "What is the server for vertex " 
       << maxId << "?" << endl;
  while ((readChar = cin.get()) != '\n')
    {
      length++;
      if (length > serverLengthMax-1)
	{
	  serverLengthMax *= 2;
	  newString = new char[serverLengthMax];
	  strcpy(newString, server);
	  delete [] server;
	  server = newString;
	}
      strncat(server, &readChar, 1);
    }

  /* read in the flow port */
  while ( invalidInput )
    {
      /* assume the user will input valid data */
      invalidInput = false;

      portString = new char[portLengthMax];
      portString[0] = '\0';
      length = 0;
      cout << "What is the flow port for vertex " << maxId
	   << "?" << endl;
      readChar = cin.get();
      while (readChar != '\n')
	{
	  /* make sure the user enters only digits */
	  if (isdigit(readChar))
	    {
	      length++;
	      if (length > portLengthMax-1)
		{
		  portLengthMax *= 2;
		  newString = new char[portLengthMax];
		  strcpy(newString, portString);
		  delete [] portString;
		  portString = newString;
		}
	      strncat(portString, &readChar, 1);
	      readChar = cin.get();
	    }
	  else {
	    while( (readChar = cin.get()) != '\n')
	      ;

	    /* release this unused mem */
	    delete [] portString;

	    invalidInput = true;
	    cout << "Port must be a positive integer < 60000!\n" << endl;
	  }
	} /* done reading in data */

      if (!invalidInput)
	{
	  port = atoi(portString);
	  if (port < 0)
	    {
	      invalidInput = true;
	      cout << "Port must be a positive integer < 60000!\n" << endl;
	    }
	}
    } /* while invalidInput */

  cout << "Are you sure you want to add vertex " << maxId << " with" << endl
       << "\tserver:  " << server << endl
       << "\tport:    " << port << endl
       << "(y/n):  ";
  cin >> readChar;
  if (readChar == 'y' || readChar == 'Y')
    {
      cv = new confVertex();
      cv->setServer(server);
      cv->setPort(port);
      vmap.insert(MICV::value_type(maxId, *cv));
      delete cv;
      rtnValue = true;
    }
  else {
    maxId--;
  }

  /* release unneeded mem */
  delete [] server;
  delete [] portString;

  return rtnValue;
}
/* end of addVertex */

/*
 * addEdge
 ************************************************************/
bool addEdge(CEL &elist, const int &maxId)
{
  char readChar;
  int from, to;
  size_t length, capLengthMax, fromLengthMax, toLengthMax;
  char *capString, *fromString, *toString, *newString;
  int cap, maxLimit;
  bool invalidInput = true;
  bool rtnValue = false;

  capLengthMax = fromLengthMax = toLengthMax = 255;

  /* read in the from vertex */
  while ( invalidInput )
    {
      /* assume the user will input valid data */
      invalidInput = false;

      fromString = new char[fromLengthMax];
      fromString[0] = '\0';
      length = 0;
      cout << "What is the vertex id from which this edge points?"
	   << endl;
      readChar = cin.get();
      while (readChar != '\n')
	{
	  /* make sure the user enters only digits */
	  if (isdigit(readChar))
	    {
	      length++;
	      if (length > fromLengthMax-1)
		{
		  fromLengthMax *= 2;
		  newString = new char[fromLengthMax];
		  strcpy(newString, fromString);
		  delete [] fromString;
		  fromString = newString;
		}
	      strncat(fromString, &readChar, 1);
	      readChar = cin.get();
	    }
	  else {
	    while( (readChar = cin.get()) != '\n')
	      ;

	    /* release this unused mem */
	    delete [] fromString;

	    invalidInput = true;
	    cout << "The vertex id must be between 1 and "
		 << maxId << "!\n" << endl;
	  }
	} /* done reading in data */

      if (!invalidInput)
	{
	  from = atoi(fromString);
	  if (from < 1 || from > maxId)
	    {
	      invalidInput = true;
	      cout << "The vertex id must be between 1 and "
		   << maxId << "!\n" << endl;
	    }
	}
    } /* while from - invalidInput */

  /* reset invalidInput to read in "to" */
  invalidInput = true;

  /* read in the to vertex id */
  while ( invalidInput )
    {
      /* assume the user will input valid data */
      invalidInput = false;

      toString = new char[toLengthMax];
      toString[0] = '\0';
      length = 0;
      cout << "What is the vertex id to which this edge points? " 
	   << endl;
      readChar = cin.get();
      while (readChar != '\n')
	{
	  if (isdigit(readChar))
	    {
	      length++;
	      if (length > toLengthMax-1)
		{
		  toLengthMax *= 2;
		  newString = new char[toLengthMax];
		  strcpy(newString, toString);
		  delete [] toString;
		  toString = newString;
		}
	      strncat(toString, &readChar, 1);
	      readChar = cin.get();
	    }
	  else {
	    while( (readChar = cin.get()) != '\n')
	      ;

	    /* relesase unused mem */
	    delete [] toString;

	    invalidInput = true;
	    cout << "The vertex id must be between 1 and " << maxId
		 << "\n" << endl;
	  }
	} /* done reading in data */

      if (!invalidInput)
	{
	  to = atoi(toString);
	  if (to < 1 || to > maxId)
	    {
	      invalidInput = true;
	      cout << "The vertex id must be between 1 and " << maxId
		   << "\n" << endl;
	    }
	}
    } /* while to - invalidInput */

  /* reset invalidInput to read in cap */
  invalidInput = true;

  /* read in the capacity*/
  while ( invalidInput )
    {
      /* assume the user will input valid data */
      invalidInput = false;

      capString = new char[toLengthMax];
      capString[0] = '\0';
      length = 0;
      cout << "What is the max limit of this edge? " 
	   << endl;
      while ((readChar = cin.get()) != '\n')
	{
	  length++;
	  if (length > capLengthMax-1)
	    {
	      capLengthMax *= 2;
	      newString = new char[capLengthMax];
	      strcpy(newString, capString);
	      delete [] capString;
	      capString = newString;
	    }
	  strncat(capString, &readChar, 1);
	} /* done reading in data */

	  maxLimit = atoi(capString);
	  if (cap == 0)
	    {
	      invalidInput = true;
	      cout << "The capacity must be greater than 0 and not " 
		   << "insanely huge!\n" << endl;
	    }
    } /* while maxLimit - invalidInput */

  /* reset to read in cap */
  delete [] capString;
  invalidInput = true;

  /* read in the capacity*/
  while ( invalidInput )
    {
      /* assume the user will input valid data */
      invalidInput = false;

      capString = new char[toLengthMax];
      capString[0] = '\0';
      length = 0;
      cout << "What is the capacity of this edge? " 
	   << endl;
      while ((readChar = cin.get()) != '\n')
	{
	  length++;
	  if (length > capLengthMax-1)
	    {
	      capLengthMax *= 2;
	      newString = new char[capLengthMax];
	      strcpy(newString, capString);
	      delete [] capString;
	      capString = newString;
	    }
	  strncat(capString, &readChar, 1);
	} /* done reading in data */

	  cap = atoi(capString);
	  if (cap == 0)
	    {
	      invalidInput = true;
	      cout << "The capacity must be greater than 0 and not " 
		   << "insanely huge!\n" << endl;
	    }
    } /* while cap - invalidInput */

  cout << "Are you sure you want to add the edge" << endl
       << "\tfrom:       " << from << endl
       << "\tto:         " << to << endl
       << "\tmax limit:  " << maxLimit << endl
       << "\tcapacity:   " << cap << endl
       << "(y/n):  ";
  cin >> readChar;
  if (readChar == 'y' || readChar == 'Y')
    {
      elist.push_back(confEdge(from, to, maxLimit, cap));
      rtnValue = true;
    }

  /* release unneeded mem */
  delete [] fromString;
  delete [] toString;
  delete [] capString;

  return rtnValue;
}
/* end of addEdge */

/*
 * setSource
 ************************************************************/
bool setSource(MICV& vmap, const int &maxId)
{
  MICV::iterator itr;
  char readChar;
  int id;
  size_t length, lengthMax;
  char *idString, *newString;
  bool invalidInput = true;
  bool rtnVal = false;

  lengthMax = 255;

  /* read in the source id */
  while ( invalidInput )
    {
      /* assume the user will input valid data */
      invalidInput = false;

      idString = new char[lengthMax];
      idString[0] = '\0';
      length = 0;
      cout << "What is the source vertex id?"
	   << endl;
      readChar = cin.get();
      while (readChar != '\n')
	{
	  /* make sure the user enters only digits */
	  if (isdigit(readChar))
	    {
	      length++;
	      if (length > lengthMax-1)
		{
		  lengthMax *= 2;
		  newString = new char[lengthMax];
		  strcpy(newString, idString);
		  delete [] idString;
		  idString = newString;
		}
	      strncat(idString, &readChar, 1);
	      readChar = cin.get();
	    }
	  else {
	    while( (readChar = cin.get()) != '\n')
	      ;

	    /* release this unused mem */
	    delete [] idString;

	    invalidInput = true;
	    cout << "The vertex id must be between 1 and "
		 << maxId << "!\n" << endl;
	  }
	} /* done reading in data */

      if (!invalidInput)
	{
	  id = atoi(idString);
	  if (id < 1 || id > maxId)
	    {
	      invalidInput = true;
	      cout << "The vertex id must be between 1 and "
		   << maxId << "!\n" << endl;
	    }
	}
    } /* while from - invalidInput */

  cout << "Are you sure you want to set " << id 
       << " as a source vertex?" << endl
       << "(y/n):  ";
  cin >> readChar;
  if (readChar == 'y' || readChar == 'Y')
    {
      itr = vmap.find(id);
      (itr->second).setIsSource();
      rtnVal = true;
    }

  /* release unneeded mem */
  delete [] idString;

  return rtnVal;
}
/* end setSource */

/*
 * setSink
 ************************************************************/
bool setSink(MICV& vmap, const int &maxId)
{
  MICV::iterator itr;
  char readChar;
  int id;
  size_t length, lengthMax;
  char *idString, *newString;
  bool invalidInput = true;
  bool rtnVal = false;

  lengthMax = 255;

  /* read in the source id */
  while ( invalidInput )
    {
      /* assume the user will input valid data */
      invalidInput = false;

      idString = new char[lengthMax];
      idString[0] = '\0';
      length = 0;
      cout << "What is the sink vertex id?"
	   << endl;
      readChar = cin.get();
      while (readChar != '\n')
	{
	  /* make sure the user enters only digits */
	  if (isdigit(readChar))
	    {
	      length++;
	      if (length > lengthMax-1)
		{
		  lengthMax *= 2;
		  newString = new char[lengthMax];
		  strcpy(newString, idString);
		  delete [] idString;
		  idString = newString;
		}
	      strncat(idString, &readChar, 1);
	      readChar = cin.get();
	    }
	  else {
	    while( (readChar = cin.get()) != '\n')
	      ;

	    /* release this unused mem */
	    delete [] idString;

	    invalidInput = true;
	    cout << "The vertex id must be between 1 and "
		 << maxId << "!\n" << endl;
	  }
	} /* done reading in data */

      if (!invalidInput)
	{
	  id = atoi(idString);
	  if (id < 1 || id > maxId)
	    {
	      invalidInput = true;
	      cout << "The vertex id must be between 1 and "
		   << maxId << "!\n" << endl;
	    }
	}
    } /* while from - invalidInput */

  cout << "Are you sure you want to set " << id 
       << " as a sink vertex?" << endl
       << "(y/n):  ";
  cin >> readChar;
  if (readChar == 'y' || readChar == 'Y')
    {
      itr = vmap.find(id);
      (itr->second).setIsSink();
      rtnVal = true;
    }

  /* release unneeded mem */
  delete [] idString;

  return rtnVal;
}

/*
 *  writeFile
 ****************************************************************/
bool writeFile(const MICV& vmap, const CEL& elist, const confVertex& contVert)
{
  char readChar, *fileName, *newString;
  size_t length, lengthMax;
  bool done = false;

  ofstream confFile;
  MICV::const_iterator vitr, vTo, vFrom;
  CEL::const_iterator eitr;

  lengthMax = 255;

  while (!done)
    {
      /* assume a good filename was entered */
      done = true;

      /* read in the filename */
      fileName = new char[lengthMax];
      fileName[0] = '\0';
      length = 0;
      cout << endl << "What is the configuration file name?" << endl;
      while ((readChar = cin.get()) != '\n')
	{
	  length++;
	  if (length > lengthMax-1)
	    {
	      lengthMax *= 2;
	      newString = new char[lengthMax];
	      strcpy(newString, fileName);
	      delete [] fileName;
	      fileName = newString;
	    }
	  strncat(fileName, &readChar, 1);
	}

      cout << "Are you sure you want to save to " 
	   << fileName << "?" << endl
	   << "(y/n):  ";
      cin >> readChar;
      if (readChar == 'y' || readChar == 'Y')
	{
	  confFile.open(fileName, ios::out);

      /* check to make sure the file opened */
	  if( confFile.fail())
	    {
	      cout << "Could not open the file:  " << fileName << endl
		   << "Would you like to try another file? (y,n)  ";
	      cin >> readChar;
	      if (readChar == 'y' || readChar == 'Y')
		{
		  done = false;
		}
	    }

	}
      else {
	cout << "Would you like to try another file? (y,n)  ";
	cin >> readChar;
	cin.ignore(1024, '\n');
	if (readChar == 'y' || readChar == 'Y')
	  {
	    done = false;
	  }
      }


      /* release the mem */
      delete [] fileName;
    }

  /* make sure the output was created */
  if (!confFile)
    {
      return false;
    }

  /* output comments and p line */
  confFile << "c This is a configuration file for the distributed max" << endl
	   << "c flow program written by Austin Armbruster." << endl
	   << "c" << endl
	   << "c The algorithm is from Goldberg and Tarjan" << endl
	   << "c" << endl
	   << "c p max numVert numEdge contServer contPort termW8Port" << endl
	   << "c a   from  to   upperCapacity sName1 flowP1 distP1 sName2 flowP2 distP2" 
	   << endl
	   << "p max\t" << vmap.size() << "\t" << elist.size() << "\t" 
	   << contVert.getServer() << "\t"
	   << /* control Port */ contVert.getPort() << endl;

  /* write all the sources and sinks */
  for (vitr = vmap.begin(); vitr != vmap.end(); vitr++)
    {
      if ((vitr->second).isSource())
	{
	  confFile << "n\t" << vitr->first << " s" << endl;
	}
      else if ((vitr->second).isSink())
	{
	  confFile << "n\t" << vitr->first << " t" << endl;
	}
    } 

  /* write all the vertex data */
  for (vitr = vmap.begin(); vitr != vmap.end(); vitr++)
    {
      confFile << "v " << setw(5) << vitr->first << " " 
	       << (vitr->second).getServer() << " "
	       << (vitr->second).getPort() << endl;
    }

  /* write all the edges */
  for (eitr = elist.begin(); eitr != elist.end(); eitr++)
    {
      vFrom = vmap.find(eitr->getFrom());
      vTo = vmap.find(eitr->getTo());
      confFile << "a  " << setw(5) << vFrom->first << "  "
	       << setw(5) << vTo->first << "  "
	       << setw(10) << eitr->getMaxLimit() << "  "
	       << setw(10) << eitr->getUpperCapacity() << "  "
	       << (vFrom->second).getServer() << "  " 
	       << (vFrom->second).getPort() << "  "
	       << (vTo->second).getServer() << "  " 
	       << (vTo->second).getPort() << endl;
    }

  confFile.close();

  cout << "file created" << endl;
  return true;
}
/* end of write file */

/* 
 *  setControllerParams
 ***************************************************************/
bool setControllerParams(confVertex& contVert)
{
  char readChar;
  int port;
  size_t length, serverLengthMax, portLengthMax;
  char *server, *portString, *newString;
  bool invalidInput = true;
  bool rtnVal = false;

  serverLengthMax = portLengthMax = 255;

  /* read in the server */
  server = new char[serverLengthMax];
  server[0] = '\0';
  length = 0;
  cout << endl << "What is the server for the controller?" << endl;
  while ((readChar = cin.get()) != '\n')
    {
      length++;
      if (length > serverLengthMax-1)
	{
	  serverLengthMax *= 2;
	  newString = new char[serverLengthMax];
	  strcpy(newString, server);
	  delete [] server;
	  server = newString;
	}
      strncat(server, &readChar, 1);
    }

  /* read in the port */
  while ( invalidInput )
    {
      /* assume the user will input valid data */
      invalidInput = false;

      portString = new char[portLengthMax];
      portString[0] = '\0';
      length = 0;
      cout << "What is the network ready port for the controller?" << endl;
      readChar = cin.get();
      while (readChar != '\n')
	{
	  /* make sure the user enters only digits */
	  if (isdigit(readChar))
	    {
	      length++;
	      if (length > portLengthMax-1)
		{
		  portLengthMax *= 2;
		  newString = new char[portLengthMax];
		  strcpy(newString, portString);
		  delete [] portString;
		  portString = newString;
		}
	      strncat(portString, &readChar, 1);
	      readChar = cin.get();
	    }
	  else {
	    while( (readChar = cin.get()) != '\n')
	      ;

	    /* release this unused mem */
	    delete [] portString;

	    invalidInput = true;
	    cout << "Network Ready Port must be a positive integer < 60000!\n" << endl;
	  }
	} /* done reading in data */

      if (!invalidInput)
	{
	  port = atoi(portString);
	  if (port < 0)
	    {
	      invalidInput = true;
	      cout << "Nework Ready Port must be a positive integer < 60000!\n" << endl;
	    }
	}
    } /* while invalidInput */

  cout << "Are you sure you want to run the controller with" << endl
       << "\tserver:                   " << server << endl
       << "\tnetwork ready port:       " << port << endl
       << "(y/n):  ";
  cin >> readChar;
  if (readChar == 'y' || readChar == 'Y')
    {
      contVert.setServer(server);
      contVert.setPort(port);
      rtnVal = true;
    }

  /* release unneeded mem */
  delete [] server;
  delete [] portString;

  return rtnVal;
}
/* end of setControllerParams */

/*
 *  void readFile(MICV& vmap, CEL& elist, confVertex& contVert, int& maxId);
 ***************************************************************/
bool readFile(MICV& vmap, CEL& elist, confVertex& contVert, int& maxId)
{
  char readChar, *fileName, *newString, line[1024];
  char server[255];
  int id, id2, port, numEdges;
  int maxLimit, cap;
  size_t length, lengthMax;
  bool done = false;
  bool firstArc = true;
  long int firstArcPos;
  bool rtnVal = false;

  ifstream confFile;
  list<int> sourceList;
  list<int> sinkList;
  list<int>::iterator sitr;
  confVertex* cv;

  lengthMax = 255;

  vmap.clear();
  elist.clear();

  while (!done)
    {
      /* assume a good filename was entered */
      done = true;

      /* read in the filename */
      fileName = new char[lengthMax];
      fileName[0] = '\0';
      length = 0;
      cout << endl << "What is the configuration file name?" << endl;
      while ((readChar = cin.get()) != '\n')
	{
	  length++;
	  if (length > lengthMax-1)
	    {
	      lengthMax *= 2;
	      newString = new char[lengthMax];
	      strcpy(newString, fileName);
	      delete [] fileName;
	      fileName = newString;
	    }
	  strncat(fileName, &readChar, 1);
	}

      confFile.open(fileName, ios::in);
      /* check to make sure the file opened */
      if( confFile.fail())
	{
	  cout << "Could not open the file:  " << fileName << endl
	       << "Would you like to try another file? (y,n)  ";
	  cin >> readChar;
	  cin.ignore(1024, '\n');
	  if (readChar == 'y' || readChar == 'Y')
	    {
	      done = false;
	    }
	}
      rtnVal = true;
    }

  if (confFile)
    {

      while(!confFile.eof())
	{
	  readChar = confFile.get();
	  switch(readChar)
	    {
	    case 'c':
	      confFile.getline(line, 1024, '\n');
	      break;

	    case 'p':
	      confFile >> line >> maxId >> numEdges >> server 
		       >> port; 
	      confFile.ignore(1024, '\n');
	      contVert.setServer(server);
	      contVert.setPort(port);
	      break;

	    case 'n':
	      confFile >> id >> readChar;
	      if (readChar == 's')
		{
		  sourceList.push_back(id);
		}
	      else {
		sinkList.push_back(id);
	      }
	      confFile.ignore(1024, '\n');
	      break;

	    case 'v':
	      confFile >> id >> server >> port;
	      confFile.ignore(1024, '\n');
	      cv = new confVertex;
	      cv->setServer(server);
	      cv->setPort(port);
	      vmap.insert(MICV::value_type(id, *cv));
	      delete cv;
	      break;

	    case 'a':
	      /* ignore all arcs on first pass. */
	      if (firstArc)
		{
		  confFile.putback(readChar);
		  firstArcPos = confFile.tellg();
		  firstArc = false;
		}
	      confFile.ignore(1024, '\n');
	      break;

	    default:
	      confFile.getline(line, 1024, '\n');
	      break;
	    }
	}
      confFile.close();
      confFile.open(fileName, ios::in);
      confFile.seekg(firstArcPos);
      while(!confFile.eof())
	{
	  readChar = confFile.get();
	  switch(readChar)
	    {
	    case 'a':
	      confFile >> id >> id2 >> maxLimit >> cap;
	      confFile.ignore(1024, '\n');
	      elist.push_back(confEdge(id, id2, maxLimit, cap));
	      break;

	    default:
	      confFile.ignore(1024, '\n');
	      break;
	    }
	}

      confFile.close();

      /* set the sources */
      for (sitr = sourceList.begin(); sitr != sourceList.end(); sitr++)
	{
	  (vmap.find(*sitr)->second).setIsSource();
	}

      /* set the sinks */
      for (sitr = sinkList.begin(); sitr != sinkList.end(); sitr++)
	{
	  (vmap.find(*sitr)->second).setIsSink();
	}
    }
  /* release the mem */
  delete [] fileName;

  return rtnVal;
}

bool incrementByThousand(MICV& vmap, confVertex& contVert)
{
  char readChar;
  bool rtnValue = false;
  MICV::iterator vitr;

  cout << "Are you sure you want to increment all ports by 1000?" << endl
       << "(y/n):  ";
  cin >> readChar;
  if (readChar == 'y' || readChar == 'Y')
    {
      rtnValue = true;

      for (vitr = vmap.begin(); vitr != vmap.end(); vitr++)
	{
	  (vitr->second).setPort((vitr->second).getPort()+1000);
	}

      contVert.setPort(contVert.getPort()+1000);
    }

  return rtnValue;
}

bool firstAvailablePorts(MICV& vmap, confVertex& contVert)
{
  char readChar;
  bool rtnValue = false;
  MICV::iterator vitr;
  int portNumber = 0;

  cout << "Are you sure you want to adjust all port numbers to the first number "
       << endl << "after the max port in the current configuration?" << endl
       << "(y/n):  ";
  cin >> readChar;
  if (readChar == 'y' || readChar == 'Y')
    {
      rtnValue = true;

      /* find the maximum port number */
      for (vitr = vmap.begin(); vitr != vmap.end(); vitr++)
	{
	  portNumber = MAX(portNumber, (vitr->second).getPort());
	}

      portNumber = MAX(portNumber, contVert.getPort());

      /* set the port numbers */
      contVert.setPort(++portNumber);

      for (vitr = vmap.begin(); vitr != vmap.end(); vitr++)
	{
	  (vitr->second).setPort(++portNumber);
	}
    }

  return rtnValue;
}

bool changeAllServers(MICV& vmap, confVertex& contVert)
{

  char readChar, *server, *newString;
  bool rtnValue = false;
  MICV::iterator vitr;
  size_t length, serverLengthMax;

  serverLengthMax = 255;

  /* read in the server */
  server = new char[serverLengthMax];
  server[0] = '\0';
  length = 0;
  cout << endl << "What is the server name?" << endl;
  while ((readChar = cin.get()) != '\n')
    {
      length++;
      if (length > serverLengthMax-1)
	{
	  serverLengthMax *= 2;
	  newString = new char[serverLengthMax];
	  strcpy(newString, server);
	  delete [] server;
	  server = newString;
	}
      strncat(server, &readChar, 1);
    }

  cout << "Are you sure you want to change all server names to "
       << server << "?" << endl
       << "(y/n):  ";
  cin >> readChar;
  if (readChar == 'y' || readChar == 'Y')
    {
      rtnValue = true;

      for (vitr = vmap.begin(); vitr != vmap.end(); vitr++)
	{
	  (vitr->second).setServer(server);
	}

      contVert.setServer(server);
    }

  delete [] server;

  return rtnValue;
}
