#include <stdlib.h>
#include <iostream.h>
#include <stdio.h>
#include <unistd.h>
#include <string.h>
#include <fcntl.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <sys/wait.h>
#include <list>
#include <sys/times.h>
#include <limits.h>

#define OPS_NUM 9
int look_up(char* s);

int main(int argc, char** argv)
{
  // variables
  int numVert, id;
  bool done = false;
  #ifndef _NO_TIME_
  clock_t start, stop;
  struct tms buf;
  #endif

  char line[1024];
  char dum[15], server[255], hostName[255];
  char intAsString[7];

  char* confBaseFileName = 0;
  char* confFileName = 0;
  char* outFileName = 0;
  char* outBaseFileName = 0;
  char* programName = 0;

  bool outFileDesired = false;
  bool verbose = false;
  bool uniqueOutput = false;
  bool uniqueInput = false;

  char* outArgv[9];
  int outArgc = 5;

  FILE* f;
  int i, outFieldNum;

  /* get the host name */
  gethostname(hostName, 255);

  // make sure the user gave the proper command line args
  for (i = 1; i < argc; i++)
    {
      switch (look_up(argv[i]))
	{
	case 0: // -conf
	  i++;
	  if (i < argc)
	    confBaseFileName = argv[i];
	  break;

	case 1: // -pgm
	  i++;
	  if (i < argc)
	    programName = argv[i];
	  break;

	case 2: // --verbose
	case 3: // -v
	  verbose = true;
	  outArgc++;
	  break;

	case 4: // -out
	  outFileDesired = true;
	  i++;
	  if (i < argc)
	      outBaseFileName = argv[i];
	  outArgc+=2;
	  break;

	case 5:
	case 6:
	  uniqueOutput = true;
	  break;

	case 7:
	case 8:
	  uniqueInput = true;
	  break;

	default:
	  break;
	}
    }

  //if((cBFN == 0)      || (pgmN == 0)  || (outFD == true && oBFN == 0))
  if (!confBaseFileName || !programName || (outFileDesired && !outBaseFileName))
    {
      cerr << "Improper Usage" << endl
	   << "Use:  " << argv[0] << " -conf network.conf -pgm maxFlowPgmName" << endl
	   << "\t[-v | --verbose] [-out outBaseFileName [-uo | --uniqueout ]]" << endl
	   << "\t[-ui | --uniqueinput] "<< endl
	   << "uniqueinput will make the input file of each of the created vertices"
	   << " be [network.conf].vertexId;" << endl
	   << "otherwise, the input file will be [network.conf]" << endl
	   << "uniqueoutput also adds the .vertexId to the outBaseFileName." << endl
	   << "verbose turns on comment lines." << endl;
    }
  else {

    /* go ahead and set the program file name into the output arguments array */
    outArgv[0] = programName;

    /* set the -conf field */
    outArgv[1] = new char[6];
    strcpy(outArgv[1], "-conf");

    /* set the -id field */
    outArgv[3] = new char[4];
    strcpy(outArgv[3], "-id");

    /* set -v field, if needed */
    if (verbose)
      {
	outArgv[5] = new char[3];
	strcpy(outArgv[5], "-v");
      }

    /* set -out field, if needed */
    if (outFileDesired)
      {
	if (verbose)
	  {
	    outFieldNum = 6;
	  }
	else {
	  outFieldNum = 5;
	}
	outArgv[outFieldNum] = new char[5];
	strcpy(outArgv[outFieldNum], "-out");
      }

    /* set the null pointer */
    outArgv[outArgc] = 0;

    /* other fields can be unique to the vertex, so set them later */

    /* open the configuration file to get the number of vertices */
    if ((f = fopen(confBaseFileName, "r")) == 0)
      {
	perror( "Initialize Network:file open");
	exit(EXIT_FAILURE);
      }

    #ifndef _NO_TIME_
    start = times(&buf);
    #endif
    while( (0 != fgets(line, 1024, f)) && !done)
      {
	switch(line[0])
	  {
	  case 'c':
	    break;

	  case 'p':
	    sscanf(line, "%s%s%d", dum, dum, &numVert);
            #ifndef _PRODUCTION_
	    if (verbose)
	      cout << "The number of vertices is:  " << numVert << endl;
            #endif
	    break;
	  case 'v':
	    sscanf(line, "%s%d%s", dum, &id, server);
	    if (!strcmp(server,hostName))
	      {
		/* create a vertex */

		/* set the id field */
		sprintf(intAsString,"%d", id);
		outArgv[4] = intAsString;

		/* set the configuration file field */
		if (uniqueInput)
		  {
		    confFileName = new char[strlen(confBaseFileName)
					   +1+strlen(intAsString)];
		    strcpy(confFileName, confBaseFileName);
		    strcat(confFileName, ".");
		    strcat(confFileName, intAsString);
		  }
		else {
		  confFileName = confBaseFileName;
		}
		outArgv[2] = confFileName;

		/* set the out filed if needed */
		if (outFileDesired)
		  {
		    if (uniqueOutput)
		      {
			outFileName = new char[strlen(outBaseFileName)+
					      1+strlen(intAsString)];
			strcpy(outFileName, outBaseFileName);
			strcat(outFileName, ".");
			strcat(outFileName, intAsString);
		      }
		    else {
		      outFileName = outBaseFileName;
		    }

		    outArgv[outFieldNum+1] = outFileName;
		  }

                #ifndef _PRODUCTION_
		if (verbose)
		  {
		    cout << "About to start vertex " << id << endl;

		    for (i=0; i<outArgc; i++)
		      {
			cout << "\toutArgv[" << i << "]:  " 
			     << outArgv[i] << endl;
		      }
		  }
                #endif

		if (fork() == 0) // child
		  {
		    execv(programName, outArgv);
		    perror("child Fork");
		    exit(EXIT_FAILURE);
		  }
		else {
		  /* release mem associated with individual vertices */
		  if (uniqueInput)
		    delete [] confFileName;
		  if (outFileDesired && uniqueOutput)
		    delete [] outFileName;
		}

	      }
	  }
      }

    #ifndef _NO_TIME_
    stop = times(&buf);
    cout << "Time to create all processes:  " 
	 << ((double)(stop - start))/((double)CLK_TCK) << endl
	 << "\tstart:  " << start << endl
	 << "\tstop:   " << stop << endl;
    #endif
    fclose(f);
    delete [] outArgv[1];
    delete [] outArgv[3];
    if (verbose)
      delete [] outArgv[5];
  }

  // wait for the processes to end
  for (i=0; i<numVert; i++)
    wait(0);
  return EXIT_SUCCESS;
}

int look_up(char* s)
{

  int i;
  char* ops[OPS_NUM];

  ops[0] = "-conf";
  ops[1] = "-pgm";
  ops[2] = "--verbose";
  ops[3] = "-v";
  ops[4] = "-out";
  ops[5] = "--uniqueoutput";
  ops[6] = "-uo";
  ops[7] = "--uniqueinput";
  ops[8] = "-ui";

  for (i = 0; i < OPS_NUM; i++)
    {
      if (strcmp(ops[i], s) == 0)
	{
	  return i;
	}
    }
  return -1;
}
