/*  edge.cpp - Implementation file for the edge class.
 *
 *  programmer:     Austin Armbruster
 *  creation date:  15 October 2000
 *  last modified:
 */
#include <stdlib.h>
#include <stdio.h>
#include <strings.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <netdb.h>
#include <math.h>

#include "edge.h"

// CONSTRUCTOR for edge
edge::edge(int direction, int id, int distance, char* serverName, 
           int port, int inUpperLimit, int inLowerLimit, int inUpperCap, 
           int inLowerCap, int inCost)
{
  /* variables */
  struct hostent *hp;

  /* initialize member variables */
  m_flow = 0;
  m_upperLimit = inUpperLimit;
  m_lowerLimit = inLowerLimit;
  m_upperCapacity = inUpperCap;
  m_lowerCapacity = inLowerCap;
  m_cost = inCost;

  m_direction = direction;
  m_ovDistance = distance;
  m_ovId = id;

  m_set = false;

  /* initialize the address */
  bzero((char *) &m_ovAddr, sizeof(m_ovAddr));
  m_ovAddr.sin_family = AF_INET;
  m_ovAddr.sin_port = htons(port);
  hp = gethostbyname(serverName);
  if (hp == 0)
    {
      fprintf(stderr, "edge.m_ovid=%d:  unknown host\n", m_ovId);
      exit(1);
    }
  bcopy( hp->h_addr_list[0], (char*)&m_ovAddr.sin_addr, hp->h_length);
}

// MODIFICATION MEMBER FUNCTIONS for edge

void edge::modifyFlow(int inDelta, int inDirection, bool mfDirection)
{
  int changeVal;
  if (mfDirection == _FORWARD_)
    {
      m_flow += inDirection*inDelta;
    }
  else {
    changeVal = (-1)*inDirection*inDelta - rf(_FORWARD_);
    if (changeVal > 0)
      {
        m_upperCapacity += changeVal;
        m_lowerCapacity -= changeVal;
      }
    m_flow += -inDirection*inDelta;
  }
}

void edge::setovDistance(int arg)
{
  m_ovDistance = arg;
  return;
}

void edge::setEdge()
{
  m_set = true;
  return;
}

// CONSTANT MEMBER FUNCTIONS for edge

int edge::upperCapacity() const
{
  return m_upperCapacity;
}

int edge::rf(bool mfDirection) const
{
  int rtnValue;

  if (mfDirection == _FORWARD_)
    {
      if (m_direction == GOINGTO)
        {
          rtnValue = m_upperCapacity-m_flow;
        }
      else {
        rtnValue = -m_lowerCapacity - m_flow;
      }
    }
  else if (m_set)
    {
      rtnValue = 0;
    }
  else {
    if (m_direction == GOINGTO)
      rtnValue = +m_flow - m_lowerLimit;
    else
      rtnValue = m_upperLimit + m_flow;
  }

  return rtnValue;
}

int edge::ovDistance() const
{
  return m_ovDistance;
}

sockaddr_in edge::getAddr() const
{
  return m_ovAddr;
}

int edge::ovId() const
{
  return m_ovId;
}

int edge::getFlow() const
{
  return m_flow;
}

int edge::getMaxLimit() const
{
  return m_upperLimit;
}

int edge::direction() const
{
  return m_direction;
}

/* friend functions */
bool operator<( edge& s1,  edge& s2)
{
  return (s1.m_ovId < s2.m_ovId);
}

