#include <sys/types.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <netdb.h>
#include <iostream.h>
#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <fstream.h>
#include <errno.h>
#include <strings.h>
#include <vector>
#include <sys/times.h>
#include <limits.h>

#include "mflib.h"
#include "mfcontroller.h"
#include "message.h"
#include "safeSocketConnect.h"

struct sockaddr_in getAddr(const char* server, int port);

mfcontroller::mfcontroller(const bool& verbose, const char* confFileName)
{
  /* initialize variables */
  m_verbose = verbose;

  /* go ahead and get the configuration information */
  readData(confFileName);
}

void mfcontroller::vertexListen()
{
  #ifndef _PRODUCTION_
  if (m_verbose)
    cout << "At the beginning of controller" << endl;
  #endif

  /* variables */
  int readysoc, ns, i;
  bool terminated=false;
  bool mfDirection = _FORWARD_;
  struct message msg;
  struct sockaddr_in ready;
  struct sockaddr_in peer;
  #ifndef __linux__
    int peerlen = sizeof(peer);
  #else
    socklen_t peerlen = sizeof(peer);
  #endif
  #ifndef _NO_TIME_
  clock_t start, stop;
  struct tms buf;
  #endif

  /* initialize variables */
  const int trueval = 1;

  #ifndef _PRODUCTION_
  if (m_verbose)
    cout << "m_readyPort:  " << m_readyPort << endl;
  #endif
  #ifndef _NO_TIME_
  start = times(&buf);
  #endif
  bzero((char *) &ready, sizeof(ready));
  bzero((char *) &peer, peerlen);

  ready.sin_family = AF_INET;
  ready.sin_port = htons(m_readyPort);
  peer.sin_family = AF_INET;

  /* create the ready socket */
  readysoc = socket(AF_INET, SOCK_STREAM, 0 );
  if( readysoc < 0 )
    {
      perror("controller:socket:readysoc");
      exit(EXIT_FAILURE);
    }
  setsockopt(readysoc, SOL_SOCKET, SO_REUSEADDR, (char*) &trueval, sizeof(trueval));
  if (bind(readysoc, (sockaddr*) &ready, sizeof(ready)) == -1)
    {
      perror("controllerThread:bind:readysoc");
      close(readysoc);
      exit(EXIT_FAILURE);
    }

  listen(readysoc, m_numVertices);

  #ifndef _NO_TIME_
  stop = times(&buf);
  cout << "controller:  time to setup socket:  " 
       << ((double)(stop-start))/((double)CLK_TCK) << endl
       << "\tstart:  " << start << endl
       << "\tstop:   " << stop << endl;
  #endif
  #ifndef _PRODUCTION_
  if (m_verbose)
    cout << "controller: controller ready for connections" << endl;
  #endif

  while (!terminated)
    {
      #ifndef _NO_TIME_
      start = times(&buf);
      #endif
      /* read in the ready message */
      for (i=0; i<m_numVertices; i++)
        {
          ns = accept(readysoc, (sockaddr*) &peer, &peerlen);
          if (ns < 0)
            {
              perror("controller:accept:readysoc");
              close(readysoc);
              exit(EXIT_FAILURE);
          }
          close(ns);
        }
      #ifndef _NO_TIME_
      stop = times(&buf);
      cout << "controller:  time to accept ready:  "
	   << ((double)(stop-start))/((double)CLK_TCK) << endl
	   << "\tstart:  " << start << endl
	   << "\tstop:   " << stop << endl;
      #endif

      /* all ready */
      #ifndef _PRODUCTION_
      if (m_verbose)
        cout << "controller:  read in all ready" << endl
             << "\tmfDirection:  " << mfDirection << endl;
      #endif

      if (mfDirection == _FORWARD_)
        {
	  #ifndef _NO_TIME_
	  start = times(&buf);
	  #endif
          /* inform the necessary clients that the system is ready */
          ns = safeSocketConnect(m_sourceAddr);
          if (ns < 0)
            {
              perror("controller:safeSocketConnect - send ready to source");
              close(readysoc);
            }
          msg.id = 0;
          msg.type = message::READY;
          write(ns, &msg, MSGSIZE);
          close(ns);
	  #ifndef _NO_TIME_
	  stop = times(&buf);
	  cout << "controller:  time to send ready in foward direction:  " 
	       << ((double)(stop-start))/((double)CLK_TCK) << endl
	       << "\tstart:  " << start << endl
	       << "\tstop:   " << stop << endl;
	  start = times(&buf);
	  #endif

          /* read in termination */
          #ifndef _PRODUCTION_
          if (m_verbose)
            cout << "controller:  waiting on source to end" << endl;
          #endif
          ns = accept(readysoc, (sockaddr*) &peer, &peerlen);
	  #ifndef _NO_TIME_
	  stop = times(&buf);
	  cout << "controller:  time to read in termination from source:  "
	       << ((double)(stop-start))/((double)CLK_TCK) << endl
	       << "\tstart:  " << start << endl
	       << "\tstop:   " << stop << endl;
	  #endif
          if (ns < 0)
            {
              perror("controller:accept:readysoc");
              close(readysoc);
              exit(EXIT_FAILURE);
          }
          close(ns);

          /* check loads */
	  #ifndef _NO_TIME_
	  start = times(&buf);
	  #endif
          ns = safeSocketConnect(m_sinkAddr);
	  #ifndef _NO_TIME_
	  stop = times(&buf);
	  cout << "controller:  time to connect to sink for load check:  "
	       << ((double)(stop-start))/((double)CLK_TCK) << endl
	       << "\tstart:  " << start << endl
	       << "\tstop:   " << stop << endl;
	  #endif
          if (ns < 0)
            {
              perror("controller:safeSocketConnect - send ready");
              close(readysoc);
            }
          msg.type = message::CHECKLOADS;
          write(ns, &msg, MSGSIZE);

          #ifndef _PRODUCTION_
          if (m_verbose)
            cout << "controller:  waiting on check loads check" << endl;
          #endif
          /* determine if the system is done */
          read(ns, &msg, MSGSIZE);
          close(ns);
          if (msg.reply== message::ACCEPT)
            {
              terminated = true;
              msg.type = message::TERMINATED;
            }
          else {
            mfDirection = !mfDirection;
            msg.type = message::CHANGEMFDIRECTION;
          }
//terminated = true;
//msg.type = message::TERMINATED;
        }
      else {
        /* inform the necessary clients that the system is ready */
        ns = safeSocketConnect(m_sinkAddr);
        if (ns < 0)
          {
            perror("controller:safeSocketConnect - send ready to source");
            close(readysoc);
          }
        msg.type = message::READY;
        write(ns, &msg, MSGSIZE);
        close(ns);

        /* read in termination */
        ns = accept(readysoc, (sockaddr*) &peer, &peerlen);
        if (ns < 0)
          {
            perror("controller:accept:readysoc");
            close(readysoc);
            exit(EXIT_FAILURE);
        }
        close(ns);
        terminated = true;
        msg.type = message::TERMINATED;
      }

      /* inform all the clients of the current state of the system */
      for (i=0; i< m_numVertices; i++)
        {
          ns = safeSocketConnect(m_vertices[i]);
          if (ns < 0)
            {
              perror("controller:safeSocketConnect - send state");
              close(readysoc);
            }
          write(ns, &msg, MSGSIZE);
          close(ns);
        }
    }
  #ifndef _PRODUCTION_
  if (m_verbose)
    cout << "controller:  I have reached the end!" << endl;
  #endif

  return;
}

void mfcontroller::readData(const char* confFileName)
{
  ifstream confFile;
  bool pline = false;
  char readChar, dum[255];
  int numVert, readyPort, i, cnt=0, port;

  /* open the file */
  confFile.open(confFileName, ios::in);

  if (!confFile)
    {
      cerr << "controller:  " << confFileName << " could not be opened." << endl
	   << "\t controller shutting down." << endl;
      exit(EXIT_FAILURE);
    }

  while(!confFile.eof())
    {
      confFile >> readChar;
      switch (readChar)
	{
	case 'p':
	  confFile >> dum >> numVert >> dum >> dum >> readyPort;
          m_vertices.reserve(numVert);
	  pline = true;
	  confFile.ignore(1024, '\n');
	  break;

	case 'n':
	  confFile >> i >> dum;
          if (dum[0] == 's')
            m_sourceId = i;
          else if (dum[0] == 't')
            m_sinkId = i;

	  confFile.ignore(1024, '\n');
	  break;

        case 'v':
          confFile >> i >> dum >> port;
          m_vertices[cnt] = getAddr(dum, port);
          if (i == m_sinkId)
            m_sinkAddr = m_vertices[cnt];
          else if (i == m_sourceId)
            m_sourceAddr = m_vertices[cnt];
          cnt++;
          confFile.ignore(1024, '\n');
          break;
	default:
 	  confFile.ignore(1024, '\n');
	  break;
	}
    }

  m_readyPort = readyPort;
  m_numVertices = numVert;
  confFile.close();
}


struct sockaddr_in getAddr(const char* server, int port)
{
  struct hostent *hp;
  struct sockaddr_in addr;

  bzero((char*) &addr, sizeof(addr));
  addr.sin_family = AF_INET;
  addr.sin_port = htons(port);

  hp = gethostbyname(server);
  if (hp == 0)
    {
      cout << "controller:  " << server << " unknown host." << endl;
      cerr << "controller:  " << server << " unknown host." << endl;
      exit(1);
    }

  bcopy(hp->h_addr_list[0], (char*) &addr.sin_addr, hp->h_length);

  return addr;
}
