/*  mfcontroller.h - Header file for the mfcontroller class
 *     mfcontroller is to be used as the controller in the
 *     distributed max flow program.  The algorithm for
 *     max flow is due to Goldberg and Tarjan.  The 
 *     termination algorithm is due to S. T. Huang.
 *
 *  programmer:
 *  creation date:  
 *  last modified:
 */

#ifndef _MFCONTROLLER_H_
#define _MFCONTROLLER_H_
#include <vector>
#include <netinet/in.h>

class mfcontroller
{
 public:
  /* CONSTRUCTOR for mfcontroller */
  mfcontroller(const bool& verbose, const char* confFileName);
  /* default constructor 
   * PRE:  verbose determins if there is output.
   *       confFileName must be a valid configuration file for the
   *          distributed max flow algorithm.
   * POST: All variables are initialized
   *****************************************************************/

  /* MODIFICATION MEMBER FUNCTIONS for mfcontroller */
  void vertexListen();
  /* listen opens sockets and first listens for vertices that are 
   *    ready, spawning a thread for each vertex.  It then waits 
   *    for the algorithm to terminate, accumulating the terminated
   *    weights in the process.
   * PRE:  NONE
   * POST: The algoriths has terminated
   *****************************************************************/

 private:
  /* PRIVATE MEMBER FUNCTIONS for mfcontroller */
  void readData(const char* confFileName);
  /* readData reads in the necessary data from the configuration 
   *    file
   * PRE:  confFileName is a valid configuration file.
   * POST: m_readyPort, m_termWeightPort, and m_numVertices are
   *          all initialized.
   *****************************************************************/

  bool m_verbose;
  int m_readyPort, m_numVertices, m_sourceId, m_sinkId;
  vector<struct sockaddr_in> m_vertices;
  struct sockaddr_in m_sinkAddr, m_sourceAddr;
};

#endif
