/*  safeSocketConnect.h - implementation of a safe way to make connections
 *     to SOCK_STREAM sockets.  It attempts to make the connection up to 
 *     in the input number of times, with a default of 5.  
 *  PRE:  peer is the sockaddr_in that you want to connect to.
 *        peerlen is the sizeof peer
 *        numAttempts is the number of times you want to try to connect
 *           to the other socket.
 *  POST: If the connection is made, the socket is returned; otherwise,
 *           -1 is returned.
 */

#include <sys/types.h>
#include <sys/socket.h>
#include <unistd.h>

int safeSocketConnect(sockaddr_in& peer, int numAttempts = 500)
{
  /* variables */
  int soc;
  int rtnValue = -1;
  int numTries = 0;
  bool connected = false;

  while (!connected /* && (numTries < numAttempts)*/)
    {
      /* create socket */
      soc = socket( AF_INET, SOCK_STREAM, 0);
      if (soc < 0)
	{
	  numTries++;
	  usleep(2);
	}
      else {

	/* request connection */
	if( connect( soc, ((struct sockaddr*)(&peer)), sizeof(peer))
	    ==-1)
	  {
	    numTries++;
	    close(soc);
	  }
	else {
          #ifndef _PRODUCTION_
	    cout << "Made a connection" << endl;
          #endif
	  connected = true;
          rtnValue = soc;
	}
      }
    }

  return rtnValue;
}

