#include <stdlib.h>
#include <stdio.h>
#include <iostream.h>
#include <signal.h>
#include <math.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <netdb.h>
#include <unistd.h>
#include <strings.h>
#include <assert.h>
#include <sys/times.h>
#include <limits.h>

#include "message.h"
#include "sendFlowData.h"
#include "acceptFlowData.h"
#include "safeSocketConnect.h"
#include "edge.h"
#include "vertex.h"

#define _LISTEN_SIZE_ 500

vertex::vertex(int id, char* fileName, bool verbose)
{
  /* variables */
  #ifndef _NO_TIME_
  clock_t start, stop;
  struct tms buf;

  cout << id << ":  time at start of vertex:  " << times(&buf) << endl;
  #endif

  /* determine if messages should be announced */
  m_verbose = verbose;
  #ifndef _PRODUCTION_
  if (m_verbose) cout << id << ":  At the beginning of the constructor" << endl;
  #endif

  // initialize values for member variables
  #ifndef _NO_TIME_
  start = times(&buf);
  #endif
  m_isSource = m_isSink = false;
  m_id = id;
  m_distance = _INITIAL_DISTANCE_;
  m_nextAcceptId = 0;
  m_currentAcceptId = 0;

  m_terminated = false;
  m_mfDirection = _FORWARD_;
  #ifndef _NO_TIME_
  stop = times(&buf);
  cout << m_id << ":  time to init member variables:  " 
       << ((double)(stop-start))/((double)CLK_TCK) << endl;
  #endif

  // read in the network
  #ifndef _PRODUCTION_
  if (m_verbose) cout << m_id << ":  About to read the network" << endl;
  #endif
  #ifndef _NO_TIME_
  start = times(&buf);
  #endif
  readNetwork(fileName);
  #ifndef _NO_TIME_
  stop = times(&buf);
  cout << m_id << ":  time to read file:  " 
       << ((double)(stop-start))/((double)CLK_TCK) << endl;
  #endif


  #ifndef _NO_TIME_
  start = times(&buf);
  #endif
  /* reserve space for the afdVector */
  m_afdVector.reserve(m_numVertices);

  /* setup the select variables */
  m_maxSocFD = m_listenSoc+1;

  FD_ZERO(&m_fullReadMask);
  FD_SET(m_listenSoc, &m_fullReadMask);
  #ifndef _NO_TIME_
  stop = times(&buf);
  cout << m_id << ":  time to initialize variables that depend on conf:  " 
       << ((double)(stop-start))/((double)CLK_TCK) << endl;
  #endif

  // run max flow
  #ifndef _PRODUCTION_
  if (m_verbose) cout << m_id << ":  about to start maxFlow()" << endl;
  #endif
  maxFlow();
}

vertex::~vertex()
{
  /* close sockets */
  close(m_listenSoc);
}

// MODIFICATION MEMBER FUNCTIONS for vertex

void vertex::readNetwork(char* fileName)
{
  FILE* f;
  int numVert, numEdge;
  char dum[10];
  int i, j, c = 0, maxLimit, lowerLimit, cap, amount;
  char line[1024];
  char serverNamei[255], serverNamej[255];
  int porti, portj;
  bool pline = false;
  bool recordedPersonalData = false;
  int distance;

  /* initialize variables */
  amount = 0;

  // attempt to open the file
  if (fileName == 0)
    {
      cerr << "ERROR:  The filename was not provided" << endl;
      exit(EXIT_FAILURE);
    }
  else  if ((f=fopen(fileName, "r")) == 0)
    {
      cerr << "File " << fileName << " could not be opened" << endl;
      exit(EXIT_FAILURE);
    }

  while ( 0 != fgets( line, 1024, f))
    {
      switch (line[0])
	{
	case 'c':  // just a comment
	  break;

	case 'p':
	  sscanf(line, "%s%s%d%d%s%d", dum, dum, &numVert, &numEdge, 
		 serverNamei, &porti);
          #ifndef _PRODUCTION_
	  if (m_verbose) 
	    cout << m_id << ":  p line " << endl
		 << "\tserverName:  " << serverNamei << endl
		 << "\tporti:  " << porti << endl;
          #endif

          controllerListenerSetup(serverNamei, porti);

	  m_numVertices = numVert;

	  /* record that the p line was present */
	  pline = true;
	  break;
	case 'n':
	  if (!pline)
	    {
	      cerr << m_id << ":  No problem line in input file! @ n" << endl;
	      exit(EXIT_FAILURE);
	    }
	  sscanf( line, "%s%d%s", dum, &i, dum);
	  if( dum[0] == 's')
	    {
              m_sourceId = i;
	      if (i == m_id) m_isSource = true;
	    }
	  if( dum[0] == 't')
	    {
              m_sinkId = i;
	      if (i == m_id) 
                {
                  m_isSink = true;
                  m_distance = 0;
                }
	    }
	  break;
	case 'a':
	  if (!pline)
	    {
	      cerr << m_id << ":  No problem line in input file! @ a" << endl;
	      exit(EXIT_FAILURE);
	    }
	  if (c == numEdge)
	    {
	      cerr << "More edges in the input file than given!"
		   << endl;
	      exit(EXIT_FAILURE);
	    }
         
	  sscanf( line, "%s%d%d%d%d%s%d%s%d", dum, &i, &j, &maxLimit,
		  &cap, serverNamei, &porti, serverNamej, &portj);
	  if( i == 0 || j == 0)
	    {
	      cerr << m_id << ":  ERROR:  0 index in input file!" << endl;
	      exit(EXIT_FAILURE);
	    }

	  if (i == m_id)
	    {
              #ifndef _PRODUCTION_
	      if (m_verbose)
		cout << m_id << ":  about to make an edge" << endl
		     << m_id << ":  a line " << endl
                     << "\tmaxLimit:  " << maxLimit << endl
		     << "\tcapacity:  " << cap << endl
		     << "\tserverNamei:  " << serverNamei << endl
		     << "\tporti:  " << porti << endl
		     << "\tserverNamej:  " << serverNamej << endl
		     << "\tportj:  " << portj << endl;
              #endif _PRODUCTION_

              /* create the listen socket, if necessary */
              if (!recordedPersonalData)
                {
                  listenSocketSetup(porti);
                  recordedPersonalData = true;
                }

              /* add the capacity to the amount if this is the source */
              if (m_isSource)
                amount += cap;

              /* determine the other vertex's distance */
              if (j == m_sourceId)
                distance = m_numVertices;
              else if (j==m_sinkId)
                distance = 0;
              else
                distance = _INITIAL_DISTANCE_;

	      /* create the new edge, mem released in destructor */
	      lowerLimit = (((j==m_sinkId) || (j==m_sourceId)) ? 0 : -maxLimit);
	      m_edgeMap.insert(make_pair(j, edge(edge::GOINGTO, j, distance,
						 serverNamej, portj, 
                                                 maxLimit, lowerLimit, cap, cap)));
	    }
	  else if(j == m_id)
	    {
              /* create listen socket, if necessary */
              if (!recordedPersonalData)
                {
                  listenSocketSetup(portj);
                  recordedPersonalData = true;
                }

              /* determine the other vertex's distance */
              if (i == m_sourceId)
                distance = m_numVertices;
              else if (i==m_sinkId)
                distance = 0;
              else
                distance = _INITIAL_DISTANCE_;

	      // Create the new edge, release mem in destructor
	      lowerLimit = (((i==m_sinkId) || (i==m_sourceId)) ? 0 : -maxLimit);
	      m_edgeMap.insert(make_pair(i, edge(edge::COMINGFROM, i, distance,
						 serverNamei, porti, 
                                                 maxLimit, lowerLimit,
						 cap, -cap)));
	    }

	  c++;
	  break;
	}
    } /* end while */
  #ifndef _PRODUCTION_
  if (m_verbose)
    cout << m_id << ":  after looping through network" << endl;
  #endif

  if (!pline)
    {
      cerr << "ERROR:  no problem line!" << endl;
      exit(EXIT_FAILURE);
    }

  fclose(f);

  if (c < numEdge)
    {
      cerr << "Less edges in the input file than given!" << endl;
      exit(EXIT_FAILURE);
    }

  if(m_isSource) 
    {
      m_distance = numVert;

      /* create the afd*/
      m_afdVector.push_back(acceptFlowData(0, 0, 0, amount));

      #ifndef _PRODUCTION_
      if (m_verbose)
        cout << m_id << ":  (m_afdVector[0]).getAmount():  "
             << (m_afdVector[0]).getAmount() << endl;
      #endif
    }
}

//  void vertex::init_flow()
//  {
//    /*  Initialize preflow -  This step was done in the instantiation of
//     *                        the edges.  This includes setting the flows
//     *                        for the edges connected to the sources.  This
//     *                        is done while reading the input file.  During
//     *                        the reading of the file, the excess is also
//     *                        set for those connected to the source.
//     *
//     *  Initialize labels -   This step was done during the instantiation
//     *                        of the vertices.  Labels means the distance
//     *                        labels.  Vertex labels are completed in the 
//     *                        readNetwork function, which is called during 
//     *                        the instantiation of the vertex.
//     *
//     *  Initialize excesses - This was done by during readNetwork.  See 
//     *                        preflow above.
//     */
//  }

void vertex::pulse()
{
  #ifndef _PRODUCTION_
  if (m_verbose) cout << m_id << ":  Starting a pulse" << endl;
  #endif
  map<int,edge>::iterator ei;

  // <STAGE 1>   
  for (ei = m_edgeMap.begin(); (active() && ei != m_edgeMap.end());
       ei++)
    {
      if( (ei->second).rf(m_mfDirection) > 0 && properDistance(ei->second) )
	push(ei->second);
    }

  if (active() && m_responseList.empty())
    {
      relabel();
      pulse();
    }
  return;
}

void vertex::receivedResponse(sendFlowData& sfd)
{
  /* variables */
  struct message msg;
  acceptFlowData& afd = m_afdVector[m_currentAcceptId];
  #ifndef _NO_TIME_
  clock_t start, stop;
  struct tms buf;
  #endif

  /*****************************************************************
   * Process the Response
   *****************************************************************/

  /* Receive response */
  #ifndef _NO_TIME_
  start = times(&buf);
  #endif
  read(sfd.m_soc, &msg, MSGSIZE);
  #ifndef _NO_TIME_
  stop = times(&buf);
  cout << m_id << ":  time to read in a flow response:  "
       << ((double)(stop-start))/((double)CLK_TCK) << endl;
  #endif

  /* make sure the amount that was sent for flow is the amount replied 
   * back */
  assert(sfd.m_delta == msg.delta);

  /* close the socket */
  close(sfd.m_soc);

  /* remove the socket from the ones that need to be listened to */
  FD_CLR(sfd.m_soc, &m_fullReadMask);

  /* react to the response. */
  if (msg.reply == message::ACCEPT)
    {
      #ifndef _NO_TIME_
      start = times(&buf);
      #endif
      #ifndef _PRODUCTION_
      if (m_verbose)
	cout << m_id << ":  received an ACCEPT from " 
             << (sfd.m_ce)->ovId() << endl;
      #endif
      /* modify the amount flowing over the edge */
      #ifndef _PRODUCTION_
      if (m_verbose)
        cout << m_id << ":  flow before accept:  "
             << (sfd.m_ce)->getFlow() << endl;
      #endif
      (sfd.m_ce)->modifyFlow(msg.delta, edge::GOINGTO, m_mfDirection);
      #ifndef _PRODUCTION_
      if (m_verbose)
        cout << m_id << ":  flow after accept:  "
             << (sfd.m_ce)->getFlow() << endl;
      #endif
      /* determine if all the flow for the current afd has been pushed */
      #ifndef _PRODUCTION_
      if (m_verbose)
        cout << m_id << ":  determining if pushed all of afd in "
             << "receivedResponse" << endl
             << "\tm_responseList.empyt():  " << m_responseList.empty() 
             << endl
             << "\tafd.getAmount():         " << afd.getAmount() << endl;
      #endif
      if (m_responseList.empty() && !(afd.getAmount()))
        m_currentAcceptId++;
      #ifndef _NO_TIME_
      stop = times(&buf);
      cout << m_id << ":  time to process flow accept response:  "
           << ((double)(stop-start))/((double)CLK_TCK) << endl;
      #endif
    }
  else {
    #ifndef _NO_TIME_
    start = times(&buf);
    #endif
    #ifndef _PRODUCTION_
    if (m_verbose)
      cout << m_id << ":  received a REJECT from " 
           << (sfd.m_ce)->ovId() << endl;
    #endif
    /* modify the excess and numResponses for a rejected flow */
    afd.modifyAmount(msg.delta);
    afd--;
    #ifndef _NO_TIME_
    stop = times(&buf);
    cout << m_id << ":  time to process flow reject response:  "
         << ((double)(stop-start))/((double)CLK_TCK) << endl;
    #endif
  }

  #ifndef _NO_TIME_
  start = times(&buf);
  #endif
  /* store the distance that was returned, if it is new */
  if ((sfd.m_ce)->ovDistance() < msg.distance)
    (sfd.m_ce)->setovDistance(msg.distance);
  #ifndef _NO_TIME_
  stop = times(&buf);
  cout << m_id << ":  time to store distance from a response:  " 
       << ((double)(stop-start))/((double)CLK_TCK) << endl;
  #endif

  /*****************************************************************
   * <STAGE 2 and 3>
   * if a relabel process is needed, relabel 
   *   relabelling will also broadcast a new distance, if necessary 
   * then start back at stage 1 again 
   *****************************************************************/
  if ( active() && m_responseList.empty()) 
    {
      relabel();
      pulse();
    }

  /*****************************************************************
   * Stage 4 is not necessary, as it is handled when a response
   *   is received from the other nodes
   *****************************************************************/

  return;
}

inline bool vertex::active()
{
  #ifndef _PRODUCTION_
  if (m_verbose) cout << m_id << ":  checking active" << endl;
  #endif

  return (!m_isSource && !m_isSink && (m_currentAcceptId != m_nextAcceptId)
          && ((m_afdVector[m_currentAcceptId]).getAmount() > 0));
}

void vertex::push(edge& ce)
{
  #ifndef _PRODUCTION_
  if (m_verbose) cout << m_id << ":  starting to push to " << ce.ovId() << endl;
  #endif

  /* variables */
  int delta;
  struct message msg;
  int soc;
  sockaddr_in peer;
  acceptFlowData& afd = m_afdVector[m_currentAcceptId];

  #ifndef _NO_TIME_
  clock_t start, stop;
  struct tms buf;
  #endif

  /* initialize variables */
  peer= ce.getAddr();

  #ifndef _PRODUCTION_
  if (m_verbose)
    cout << m_id << ":  in push " << ce.ovId() << endl
         << "\tce.rf(m_mfDirection):  " << ce.rf(m_mfDirection) << endl;
  #endif

  /* determine how much flow should be sent and update this 
   * vertex's excess */
  delta = min(afd.getAmount(), ce.rf(m_mfDirection));
  afd++;
  afd.modifyAmount(-delta);

  /* Send the flow to the other vertex */
  #ifndef _PRODUCTION_
  if (m_verbose)
    cout << m_id << ":  actually sending " << delta << " units of flow to " 
	 << ce.ovId() << endl;
  #endif

  /* create socket */
  #ifndef _NO_TIME_
  start = times(&buf);
  #endif
  soc = safeSocketConnect(peer);
  #ifndef _NO_TIME_
  stop = times(&buf);
  cout << m_id << ":  time to connect for flow to " << ce.ovId() << " :  "
       << ((double)(stop-start))/((double)CLK_TCK) << endl;
  #endif
  if (soc < 0)
    {
      cout << m_id << ":  Failure:sendFlow:safeSocketConnect"
           << endl;
      fprintf(stderr, "%d:  ", m_id);
      perror("Failure:sendFlow:safeSocketConnect");
      exit(EXIT_FAILURE);
    }

  #ifndef _NO_TIME_
  start = times(&buf);
  #endif
  /* add the socket to the responseList */
  m_responseList.push_back(sendFlowData(soc,delta, &ce));

  /* add the socket to the set to listen to */
  m_maxSocFD = max(m_maxSocFD, soc+1);
  FD_SET(soc, &m_fullReadMask);

  /* setup the message */
  msg.delta = delta;
  msg.id = m_id;
  msg.distance = m_distance;
  msg.type = message::SENDFLOW;
  msg.sourceAFId = afd.getAFId();
  #ifndef _NO_TIME_
  stop = times(&buf);
  cout << m_id << ":  time to setup flow message:  " 
       << ((double)(stop-start))/((double)CLK_TCK) << endl;
  #endif

  #ifndef _PRODUCTION_
  if (m_verbose)
    cout << m_id << ":  sending afid of " << msg.sourceAFId 
         << " to " 
	 << ce.ovId() << endl;
  #endif

  /* Send the flow to the other vertex */
  #ifndef _NO_TIME_
  start = times(&buf);
  #endif
  write(soc, &msg, MSGSIZE);
  #ifndef _NO_TIME_
  stop = times(&buf);
  cout << m_id << ":  time to write flow message to " << ce.ovId() << " :  " 
       << ((double)(stop-start))/((double)CLK_TCK) << endl;
  #endif
}

void vertex::relabel()
{
  #ifndef _PRODUCTION_
  if (m_verbose)
    cout << m_id << ":  about to relabel, current distance is " 
         << m_distance << endl;
  #endif

  /* variables */
  map<int,edge>::iterator ei;
  int minDist = 0;

  #ifndef _NO_TIME_
  clock_t start, stop;
  struct tms buf;
  start = times(&buf);
  #endif
  for (ei = m_edgeMap.begin(); ei != m_edgeMap.end(); ei++)
    {
      /* only residual edges are to be considered */
      if( (ei->second).rf(m_mfDirection) > 0)
	{
	  if (minDist)
	    {
	      minDist = min(minDist, (ei->second).ovDistance() +1);
	    }
	  else {
	    minDist = (ei->second).ovDistance() +1;
	  }
	}
    }
  #ifndef _NO_TIME_
  stop = times(&buf);
  cout << m_id << ":  time to determine if a relabel is needed:  " 
       << ((double)(stop-start))/((double)CLK_TCK) << endl;
  #endif

  // determine if a new distance has been found
  if (m_distance != minDist)
    {
      #ifndef _PRODUCTION_
      if (m_verbose)
	cout << m_id << ":  a new distance was found, it is " << minDist << endl;
      #endif
      m_distance = minDist;

      // Broadcast the new distance to the other vertices
      #ifndef _NO_TIME_
      start = times(&buf);
      #endif
      for (ei = m_edgeMap.begin(); ei != m_edgeMap.end(); ei++)
	{
	  sendDistance(ei->second);
	}
      #ifndef _NO_TIME_
      stop = times(&buf);
      cout << m_id << ":  time to send out new distance:  " 
	   << ((double)(stop-start))/((double)CLK_TCK) << endl;
     #endif
    }
}

inline bool vertex::properDistance(edge& ce)
{
  #ifndef _PRODUCTION_
  if (m_verbose)
    cout << m_id << ":  checking proper distance to " << ce.ovId() 
	 << " which has a distance of " << ce.ovDistance() << endl;
  #endif

  return ( m_distance == ce.ovDistance() + 1);
}

void vertex::listenSocketSetup(int& port)
{
  /* variables */
  const int trueval = 1;
  struct sockaddr_in self;
  #ifndef _NO_TIME_
  clock_t start, stop;
  struct tms buf;
  start = times(&buf);
  #endif

  /* initialize variables */
  bzero((char*) &self, sizeof(self));
  self.sin_family = AF_INET;
  self.sin_addr.s_addr = INADDR_ANY;
  self.sin_port = htons(port);

  /* setup listing to the socket */
  m_listenSoc = socket(AF_INET, SOCK_STREAM, 0);
  if (m_listenSoc < 0)
    {
      perror("Failed:  listenSocketSetup:socket");
      exit(EXIT_FAILURE);
    }

  setsockopt(m_listenSoc, SOL_SOCKET, SO_REUSEADDR,(char*) &trueval, 
             sizeof(trueval));
  if (bind(m_listenSoc, (sockaddr*) &self, sizeof(self)) == -1 )
    {
      fprintf(stderr, "%d:  ", m_id);
      perror("Failed:  listenSocketSetup:bind");
      close(m_listenSoc);
      exit(EXIT_FAILURE);
    }
  if( listen(m_listenSoc, _LISTEN_SIZE_) == -1)
    {
      perror("Failure:  listenSocketSetup:listen");
      close(m_listenSoc);
      exit(EXIT_FAILURE);
    }
  #ifndef _NO_TIME_
  stop = times(&buf);
  cout << m_id << ":  time to setup listen socket:  " 
       << ((double)(stop-start))/((double)CLK_TCK) << endl;
  #endif
}

void vertex::acceptFlow(message& msg, int& ns)
{
  /* variables */
  struct message response;
  map<int,edge>::iterator ei;
  #ifndef _NO_TIME_
  clock_t start, stop;
  struct tms buf;
  start = times(&buf);
  #endif

  #ifndef _PRODUCTION_
  if (m_verbose)
    cout << m_id << ":  received a message to accept flow" << endl;
  #endif

  /* determine if the flow will be accepted */
  if ((msg.distance == m_distance +1) || (msg.id == m_sourceId) ||
      (msg.id == m_sinkId))
    {
      #ifndef _NO_TIME_
      stop = times(&buf);
      cout << m_id << ":  time to determine if flow will be accepted:  "
           << ((double)(stop-start))/((double)CLK_TCK) << endl;
      #endif
      #ifndef _PRODUCTION_
      if (m_verbose)
        cout << m_id << ":  accepting " << msg.delta << " units of flow from " 
             << msg.id << " along with an afid of " 
             << msg.sourceAFId << endl;
      #endif

      // Go ahead and return a positive reply
      #ifndef _NO_TIME_
      start = times(&buf);
      #endif
      response.reply = message::ACCEPT;
      response.delta = msg.delta;
      response.id = m_id;
      response.distance = m_distance;
      #ifndef _NO_TIME_
      stop = times(&buf);
      cout << m_id << ":  time to setup message of accepting flow:  "
           << ((double)(stop-start))/((double)CLK_TCK) << endl;
      start = times(&buf);
      #endif
      write(ns, &response, MSGSIZE);
      #ifndef _NO_TIME_
      stop = times(&buf);
      cout << m_id << ":  time to write message of accepting flow:  " 
           << ((double)(stop-start))/((double)CLK_TCK) << endl;
      #endif

      // find the edge that this vertex is attached to
      #ifndef _NO_TIME_
      start = times(&buf);
      #endif
      ei = m_edgeMap.find(msg.id);
      #ifndef _PRODUCTION_
      if (m_verbose)
        cout << m_id << ":  modifying edge by " <<msg.delta << endl
             << "\tm_mfDirection:  " << m_mfDirection << endl;
      #endif
      (ei->second).modifyFlow(msg.delta, edge::COMINGFROM, m_mfDirection);
      #ifndef _PRODUCTION_
      if (m_verbose)
        cout << m_id << ":  new flow to " << (ei->second).ovId()
             << " is " << (ei->second).getFlow() << endl;
      #endif
      #ifndef _NO_TIME_
      stop = times(&buf);
      cout << m_id << ":  time to find edge and modify flow for "
           << "accepted flow:  " 
	   << ((double)(stop-start))/((double)CLK_TCK) << endl;
      #endif


      /* create the acceptFlowData */
      if (!m_isSink && !m_isSource)
        {
          #ifndef _NO_TIME_
          start = times(&buf);
          #endif
          m_afdVector.push_back(acceptFlowData(msg.id, msg.sourceAFId, 
                                               m_nextAcceptId, 
                                               msg.delta));
          m_nextAcceptId++;
          #ifndef _PRODUCTION_
          if (m_verbose)
            cout << m_id << ":  updated afdVector" << endl;
          #endif
          #ifndef _NO_TIME_
          stop = times(&buf);
          cout << m_id << ":  time to add accepted flow to afdVector:  "
               << ((double)(stop-start))/((double)CLK_TCK) << endl;
          #endif

          /* determine if a pulse needs to be started */
          if (active() && m_responseList.empty())
            pulse();
        }
      else {
        /* send term message */
        acceptFlowData afd = acceptFlowData(msg.id, msg.sourceAFId, 0,0);
        sendTerm(afd);
      }
    } /* if can accept this flow */
  else {
    #ifndef _NO_TIME_
    stop = times(&buf);
    cout << m_id << ":  time to determine if flow will be accepted:  "
         << ((double)(stop-start))/((double)CLK_TCK) << endl;
    #endif
    #ifndef _PRODUCTION_
    if (m_verbose)
      cout << m_id << ":  rejecting " << msg.delta 
           << " units of flow from " << msg.id << endl;
    #endif
    #ifndef _NO_TIME_
    start = times(&buf);
    #endif
    response.reply = message::REJECT;
    response.delta = msg.delta;
    response.id = m_id;
    response.distance = m_distance;
    #ifndef _NO_TIME_
    stop = times(&buf);
    cout << m_id << ":  time to setup rejection of flow message:  "
         << ((double)(stop-start))/((double)CLK_TCK) << endl;
    start = times(&buf);
    #endif
    write(ns, &response, MSGSIZE);
    #ifndef _NO_TIME_
    stop = times(&buf);
    cout << m_id << ":  time to write rejection of flow message:  " 
	 << ((double)(stop-start))/((double)CLK_TCK) << endl;
    #endif
  }

  /* End of Function */
  return;
}

void vertex::acceptDistance(message &msg, int& ns)
{
  #ifndef _PRODUCTION_
  if (m_verbose)
    cout << m_id << ":  accepting a distance of " << msg.distance 
         << " from " << msg.id << endl;
  #endif

  /* variables */
  map<int,edge>::iterator ei;
  #ifndef _NO_TIME_
  clock_t start, stop;
  struct tms buf;
  start = times(&buf);
  #endif

  /* find the appropriate edge that connects to this vertex */
  ei = m_edgeMap.find(msg.id);

  /* make sure the edge was found */
  if( ei == m_edgeMap.end())
    {
      fprintf(stderr,"Failure:  acceptNewDistances:EdgeNotFound");
    }
  else {
    (ei->second).setovDistance(msg.distance);
  }

  #ifndef _NO_TIME_
  stop = times(&buf);
  cout << m_id << ":  time to find and record accepted Distance:  " 
       << ((double)(stop-start))/((double)CLK_TCK) << endl;
  #endif
  return;        
}

void vertex::sendDistance(edge& ce)
{
  /* variables */
  int soc;
  struct message msg;
  struct sockaddr_in peer;
  #ifndef _NO_TIME_
  clock_t start, stop;
  struct tms buf;
  #endif

  /* initialize variables */
  peer = ce.getAddr();

  /* create socket and request a connection*/
  #ifndef _NO_TIME_
  start = times(&buf);
  #endif
  soc = safeSocketConnect(peer);
  #ifndef _NO_TIME_
  stop = times(&buf);
  cout << m_id << ":  time to connect for distance:  " 
       << ((double)(stop-start))/((double)CLK_TCK) << endl;
  #endif
  if (soc < 0)
    {
      cout << m_id << ":  Failure: sendDistance:saveSocketConnect"
           << endl;
      perror("Failure: sendDistance:safeSocketConnect");
      exit(EXIT_FAILURE);
    }

  /* write the distance */
  #ifndef _NO_TIME_
  start = times(&buf);
  #endif
  msg.distance = m_distance;
  msg.id = m_id;
  msg.type = message::SENDDISTANCE;
  #ifndef _NO_TIME_
  stop = times(&buf);
  cout << m_id << ":  time to setup distance message:  "
       << ((double)(stop-start))/((double)CLK_TCK) << endl;
  #endif
  #ifndef _PRODUCTION_
  if (m_verbose)
    cout << m_id << ":  sending a distance of " << m_distance << " to " 
	 << ce.ovId() << endl;
  #endif
  #ifndef _NO_TIME_
  start = times(&buf);
  #endif
  write(soc, &msg, MSGSIZE);
  #ifndef _NO_TIME_
  stop = times(&buf);
  cout << m_id << ":  time to write distance message to " << ce.ovId() << " :  " 
       << ((double)(stop-start))/((double)CLK_TCK) << endl;
  #endif

  /* close the socket */
  close(soc);

  return;
}


void vertex::maxFlow()
{
  /* variables */
  sendFlowData sfd;
  list<sendFlowData>::iterator ritr;
  int n, nready, ns, soc;
  message msg;
  fd_set readMask;
  struct sockaddr_in peer, addr;
  #ifndef __linux__
    int peer_len;
  #else
    socklen_t peer_len;
  #endif
  #ifndef _NO_TIME_
  clock_t start, stop;
  struct tms buf;
  start = times(&buf);
  #endif

  /* initialize variables */
  bzero((char*) &peer, sizeof(peer));
  peer.sin_family = AF_INET;
  peer_len = sizeof(peer);

  /* send to the controller that this vertex is ready */
  addr = m_controllerAddr;
  soc = safeSocketConnect(addr);
  #ifndef _NO_TIME_
  stop = times(&buf);
  cout << m_id << ":  time to tell controller ready:  " 
       << ((double)(stop-start))/((double) CLK_TCK) << endl;
  #endif
  if (soc < 0)
    {
      cout << m_id << ":  could not connect to controller" << endl;
      perror("failed controller connect");
      exit(1);
    }

  /* run the algorithm */
  while (!m_terminated)
    {
      /* wait for a message */
      #ifndef _PRODUCTION_
      if (m_verbose)
        cout << m_id << ":  about to select" << endl;
      #endif
      readMask = m_fullReadMask;
      #ifndef _NO_TIME_
      start = times(&buf);
      #endif
      nready = select(m_maxSocFD, &readMask, 0, 0, 0);
      #ifndef _NO_TIME_
      stop = times(&buf);
      cout << m_id << ":  time to wait on select:  " 
	   << ((double)(stop-start))/((double)CLK_TCK) << endl;
      #endif

      /* process all the ready messages */
      while (nready)
        {
          if (FD_ISSET(m_listenSoc, &readMask))
            {
              #ifndef _PRODUCTION_
              if (m_verbose)
                cout << m_id << ":  just before accept in maxFlow" << endl;
              #endif
              ns = accept(m_listenSoc, (sockaddr*) &peer, &peer_len);
              #ifndef _PRODUCTION_
              if (m_verbose)
                cout << m_id << ":  just after accept in maxFlow" << endl;
              #endif
              if(ns <0)
                {
                  #ifndef _PRODUCTION_
                  if (m_verbose)
                    cout << m_id << ":  failure:  maxflow:accept" << endl;
                  #endif
                  perror("Failure:  maxflow:accept");
                }
              else {
                #ifndef _NO_TIME_
                start = times(&buf);
                #endif
                n=read(ns, &msg, MSGSIZE);
                #ifndef _NO_TIME_
                stop = times(&buf);
                cout << m_id << ":  time to read in message in maxFlow:  "
                     << ((double)(stop-start))/((double)CLK_TCK) << endl;
                #endif
                /* check to make sure the message was read in */
                if(n != MSGSIZE)
                  {
                    perror("Failure:  maxflow:read");
                    close(m_listenSoc);
                    close(ns);
                    exit(EXIT_FAILURE);
                  }

                switch(msg.type)
                  {
                  case message::SENDFLOW: 
                    acceptFlow(msg, ns);
                    break;
                  case message::SENDDISTANCE:
                    acceptDistance(msg, ns);
                    break;
                  case message::SENDTERM:
                    acceptTerm(msg);
                    break;
                  case message::TERMINATED:
                  case message::CHANGEMFDIRECTION:
                  case message::READY:
                  case message::CHECKLOADS:
                    controllerListener(msg, ns);
                    break;
                  #ifndef _PRODUCTION_
                  default:
                    if (m_verbose)
                      cout << m_id << ":  received a message from " 
                           << msg.id << endl;
                  #endif
                  }
                close(ns);
              }
              FD_CLR(m_listenSoc, &readMask);
            }
          else {
            #ifndef _NO_TIME_
            start = times(&buf);
            #endif
            for (ritr = m_responseList.begin(); 
                 ritr != m_responseList.end() && 
		   !(FD_ISSET(ritr->m_soc, &readMask));
                 ritr++)
	      ;
            sfd = *ritr;
            m_responseList.erase(ritr);
            #ifndef _NO_TIME_
            stop = times(&buf);
            cout << m_id << ":  time to find and remove a response:  "
                 << ((double)(stop-start))/((double)CLK_TCK) << endl;
            #endif
            receivedResponse(sfd);
            FD_CLR(sfd.m_soc, &readMask);
          }

          nready--;
        }
    }

  #ifndef _PRODUCTION_
  if (m_verbose)
    cout << m_id << ":  Maxflow has terminated" << endl;
  #endif

  return;
}

ostream& operator<<(ostream& out, const vertex& s)
{
  /* variables */
  map<int,edge>::const_iterator ei;
  #ifndef _NO_TIME_
  clock_t start, stop;
  struct tms buf;
  start = times(&buf);
  #endif

  /* output the results */
  out << "The following is the local output for vertex " << s.m_id 
      << endl;
  for (ei = s.m_edgeMap.begin(); ei != s.m_edgeMap.end(); ei++)
    {
      if((ei->second).direction() == edge::GOINGTO)
	{
	  cout << "f\t" << s.m_id << "\t" << (ei->second).ovId() 
	       << "\t" << (ei->second).getFlow() 
               << "\t" << (ei->second).getMaxLimit() << endl;
	}
      else 
	{
	  cout << "f\t" << (ei->second).ovId() << "\t" << s.m_id
	       << "\t" << (ei->second).getFlow()
               << "\t" << (ei->second).getMaxLimit() << endl;
	}
    } /* while loop to print flows. */
  #ifndef _NO_TIME_
  stop = times(&buf);
  cout << s.m_id << ":  time to print results:  " 
       << ((double)(stop-start))/((double)CLK_TCK) << endl;
  #endif

  return out;
}

void vertex::controllerListenerSetup(const char* server, int port)
{
  /* variables */
  struct sockaddr_in peer;
  int peerlen;
  struct hostent *hp;

  /* initialize variables */
  peerlen = sizeof(peer);
  bzero((char*) &peer, peerlen);
  peer.sin_family = AF_INET;
  peer.sin_port = htons(port);

  /* fill in peer address */
  hp = gethostbyname(server);
  if(hp == 0)
    {
      perror("listenForTermination:gethostbyname");
      exit(EXIT_FAILURE);
    }
  bcopy(hp->h_addr_list[0], (char*)&peer.sin_addr, hp->h_length);

  m_controllerAddr = peer;
}

void vertex::controllerListener(struct message& msg, int ns)
{
  #ifndef _PRODUCTION_
  if (m_verbose)
    cout << m_id << ":  in controllerListener" << endl;
  #endif

  /* variables */
  int amount, soc;
  sockaddr_in peer;
  map<int,edge>::iterator ei;
  #ifndef _NO_TIME_
  clock_t start, stop;
  struct tms buf;
  #endif

  /* determine what needs to be done */
  switch(msg.type)
    {
    case message::TERMINATED:
      /* record that the calculation is done */
      m_terminated = true;
      break;
    case message::CHANGEMFDIRECTION:
      #ifndef _PRODUCTION_
      if (m_verbose)
        cout << m_id << ":  changing mf direction" << endl
	     << "Changing Directions:  " << endl << *this;
      #endif

      #ifndef _NO_TIME_
      start = times(&buf);
      #endif
      /* record that the direction needs to be changed */
      m_mfDirection = !m_mfDirection;

      /* reset this vertex's distance */
      if (m_id == m_sinkId)
        m_distance = m_numVertices;
      else if (m_id==m_sourceId)
        m_distance = 0;
      else
        m_distance = _INITIAL_DISTANCE_;      

      #ifndef _PRODUCTION_
      if (m_verbose)
	cout << m_id << ":  about to reset distances for direction change" << endl;
      #endif
      /* determine the other vertex's distance */
      for (ei = m_edgeMap.begin(); ei != m_edgeMap.end(); ei++)
        {
          if ((ei->first) == m_sinkId)
            {
              (ei->second).setovDistance(m_numVertices);
              if ((ei->second).getFlow() == (ei->second).getMaxLimit())
                (ei->second).setEdge();
            }
          else if ((ei->first)==m_sourceId)
            (ei->second).setovDistance(0);
          else
            (ei->second).setovDistance(_INITIAL_DISTANCE_);
        }
      #ifndef _PRODUCTION_
      if (m_verbose)
	cout << m_id << ":  done reseting distances for direction change" << endl;
      #endif
      #ifndef _NO_TIME_
      stop = times(&buf);
      cout << m_id << ":  time to setup for change direction:  "
           << ((double)(stop-start))/((double)CLK_TCK) << endl;
      start = times(&buf);
      #endif
      /* communicate that this vertex is ready */
      peer = m_controllerAddr;
      soc = safeSocketConnect(peer);
      #ifndef _NO_TIME_
      stop = times(&buf);
      cout << m_id << ":  time to connect to controller after direction "
           << "change:  "
           << ((double)(stop-start))/((double)CLK_TCK) << endl;
      #endif
      if (soc < 0)
        {
          cout << m_id << ":  change direction ready failed" << endl;
          cerr << m_id << ":  change direction ready failed" << endl;
          exit(1);
        }
      close(soc);
      break;

    case message::READY:
      /* start to push out the new needed value */
      #ifndef _PRODUCTION_
      if (m_verbose)
        cout << m_id << ":  maxFlow knows that the system is ready" << endl;
      #endif

      /* pulse out any preflow */
      if (m_isSource)
        {
          #ifndef _NO_TIME_
          start = times(&buf);
          #endif
          #ifndef _PRODUCTION_
          if (m_verbose) 
            cout << m_id << ":  source entering preflow" << endl;
          #endif
          for (ei = m_edgeMap.begin(); ei != m_edgeMap.end(); ei++)
            {
              #ifndef _PRODUCTION_
              if (m_verbose) 
                cout << m_id << ":  in preflow for loop" << endl;
              #endif
              if ((ei->second).direction() == edge::GOINGTO)
                {
                  push(ei->second);
                  #ifndef _PRODUCTION_
                  if (m_verbose)
                    cout << m_id << ":  source pushing out initial flow"
                         << endl;
                  #endif
                }
            }
          #ifndef _NO_TIME_
          stop = times(&buf);
          cout << m_id << ":  time for source to push preflow:  "
               << ((double)(stop-start))/((double)CLK_TCK) << endl;
          #endif
        }
      else if (m_isSink)
        {
          #ifndef _NO_TIME_
          start = times(&buf);
          #endif
          #ifndef _PRODUCTION_
          if (m_verbose) cout << m_id << ":  sink entering preflow" << endl;
          #endif
          for (ei = m_edgeMap.begin(); ei != m_edgeMap.end(); ei++)
            {
              #ifndef _PRODUCTION_
              if (m_verbose) 
                cout << m_id << ":  in preflow for loop for sink" << endl;
              #endif
              if ((ei->second).rf(_REVERSE_) > 0)
                {
                  push(ei->second);
                  #ifndef _PRODUCTION_
                  if (m_verbose)
                    cout << m_id << ":  sink pushing out initial flow" 
                         << endl;
                  #endif
                }
            }
          #ifndef _NO_TIME_
          stop = times(&buf);
          cout << m_id << ":  time to send out preflow for sink:  " 
               << ((double)(stop-start))/((double)CLK_TCK) << endl;
          #endif
        }
      break;
    case message::CHECKLOADS:
      if (m_isSink)
        {
	  #ifndef _NO_TIME_
	  start = times(&buf);
	  #endif
          amount = 0;
          for (ei=m_edgeMap.begin(); 
               ei!=m_edgeMap.end();
               ei++)
            {
              #ifndef _PRODUCTION_
              if (m_verbose)
                cout << m_id << ":  the following info links to "
                     << (ei->second).ovId() << endl
                     << "\tflow:       " << (ei->second).getFlow() << endl
                     << "\tmaxLimit:   " << (ei->second).getMaxLimit()
                     << endl
                     << "\tupper cap:  " << (ei->second).upperCapacity() 
                     << endl
                     << "\trf:         " << (ei->second).rf(_REVERSE_) 
                     << endl;
              #endif
              amount += (ei->second).rf(_REVERSE_);
            }
          if (amount > 0)
            {
              #ifndef _PRODUCTION_
              if (m_verbose)
                cout << m_id << ":  check loads reject" << endl;
              #endif
              msg.reply = message::REJECT;
              m_afdVector[m_currentAcceptId] = acceptFlowData(0, 0, 0, 
                                                              amount);
            }
          else {
            #ifndef _PRODUCTION_
            if (m_verbose)
              cout << m_id << ":  check loads accept" << endl;
            #endif
            msg.reply = message::ACCEPT;
          }
          write(ns, &msg, MSGSIZE);
	  #ifndef _NO_TIME_
	  stop = times(&buf);
	  cout << m_id << ":  time to check loads including message:  " 
	       << ((double)(stop-start))/((double)CLK_TCK) << endl;
	  #endif
        }
      break;
    }
  return;
}

void vertex::sendTerm(acceptFlowData& afd)
{
  /* variables */
  sockaddr_in peer;
  map<int,edge>::iterator ei;
  message msg;
  int soc;
  #ifndef _NO_TIME_
  clock_t start, stop;
  struct tms buf;
  #endif

  if (! afd.getSourceId())
    {
      /* connect to the other vertex */
      peer = m_controllerAddr;
      #ifndef _NO_TIME_
      start = times(&buf);
      #endif
      soc = safeSocketConnect(peer);
      #ifndef _NO_TIME_
      stop = times(&buf);
      cout << m_id << ":  time to tell controller source/sink done:  " 
	   << ((double)(stop-start))/((double)CLK_TCK) << endl;
      #endif
      if (soc < 0)
        {
          perror("failed connect to controller in sendterm");
          cout << "failed connect to controller in sendterm" << endl;
          exit(1);
        }
      close(soc);
    }
  else {
    #ifndef _PRODUCTION_
    if (m_verbose) 
      cout << m_id << ":  sending term to " << afd.getSourceId() << endl;
    #endif
    /* get source address */
    #ifndef _NO_TIME_
    start = times(&buf);
    #endif
    ei = m_edgeMap.find(afd.getSourceId());
    peer = (ei->second).getAddr();
    #ifndef _NO_TIME_
    stop = times(&buf);
    cout << m_id << ":  time to find peer in sendTerm:  " 
	 << ((double)(stop-start))/((double)CLK_TCK) << endl;
    #endif

    /* connect to the other vertex */
    #ifndef _NO_TIME_
    start = times(&buf);
    #endif
    soc = safeSocketConnect(peer);
    #ifndef _NO_TIME_
    stop = times(&buf);
    cout << m_id << ":  time to connect to peer in sendTerm:  " 
	 << ((double)(stop-start))/((double)CLK_TCK) << endl;
    #endif
    if (soc < 0)
      {
        perror("failed connect in sendterm");
        cout << "failed connect in sendterm" << endl;
        exit(1);
      }

    /* setup the message */
    #ifndef _NO_TIME_
    start = times(&buf);
    #endif
    msg.type = message::SENDTERM;
    msg.sourceAFId = afd.getSourceAFId();
    msg.id = m_id;
    #ifndef _NO_TIME_
    stop = times(&buf);
    cout << m_id << ":  time to setup message to peer in send term:  " 
	 << ((double)(stop-start))/((double)CLK_TCK) << endl;
    #endif

    /* send the message */
    #ifndef _NO_TIME_
    start = times(&buf);
    #endif
    write(soc, &msg, MSGSIZE);
    #ifndef _NO_TIME_
    stop = times(&buf);
    cout << m_id << ":  time to write message to peer in send term:  " 
	 << ((double)(stop-start))/((double)CLK_TCK) << endl;
    #endif

    /* close the socket */
    close(soc);
  }
  return;
}

void vertex::acceptTerm(message& msg)
{
  #ifndef _NO_TIME_
  clock_t start, stop;
  struct tms buf;
  start = times(&buf);
  #endif

  #ifndef _PRODUCTION_
  if (m_verbose) cout << m_id << ":  made it into acceptTerm" << endl;
  #endif
  /* variables */
  acceptFlowData& afd = m_afdVector[msg.sourceAFId];

  /* reduce the number of responses needed */
  afd--;

  /* send the Termination to the source if necessary */
  if (afd.done())
    sendTerm(afd);

  #ifndef _NO_TIME_
  stop = times(&buf);
  cout << m_id << ":  time to run acceptTerm:  "
       << ((double)(stop-start))/((double)CLK_TCK) << endl;
  #endif

  return;
}
