#include <GL/glut.h>
#include <stdlib.h>
#include <math.h>
#include <sys/time.h>
#include <unistd.h>
#include <iostream.h>
#include <time.h>
#include <vector>
#include <fstream.h>

class node_c
{
public:
	float loc[2];
};

class line_c
{
public:
	int node[2];
	int current;
	int max;
	int enabled;
};
static const float border = 100;

static vector<node_c> nodes;
static vector<line_c> lines;
static int width = 0; // screen width and height
static int height = 0;
static int offsetx = 0;
static int offsety = 0;
static int mouse_pressed = 0;
static int mouse_lastx;
static int mouse_lasty;

void display(void)
{
	/* clear all pixels */
	glClear(GL_COLOR_BUFFER_BIT);
	glBegin( GL_LINES );
	int stop = lines.size();
	for( int i = 0; i < stop; i++)
	{
		if(!lines[i].enabled)
			continue;
		float ratio=(float)lines[i].current/lines[i].max;
		if( ratio < .5 )
			glColor3f( 1-ratio, 0.0, 0.0);
		else
			glColor3f( 0.0, ratio, 0.0);
		glVertex3f(nodes[lines[i].node[0]].loc[0], // X, node 0
			nodes[lines[i].node[0]].loc[1], 0.0); // Y, node 0
		glVertex3f(nodes[lines[i].node[1]].loc[0], // X, node 1
			nodes[lines[i].node[1]].loc[1], 0.0); // Y, node 1
	}
	glEnd();
	glFlush();
	glutSwapBuffers();
}

void init(void)
{
	/* open input file and read nodes and lines */
	node_c n;
	line_c l;
	ifstream in("basic_lines.txt");
	char cmd;
	while( !in.eof())
	{
		in >> cmd;
		if(in.eof())
			break;
		if( cmd == 'n')
		{
			in >> n.loc[0] >> n.loc[1];
			nodes.push_back(n);
		}
		else
		if( cmd == 'l')
		{
			in >> l.node[0] >> l.node[1];
			in >> l.current >> l.max;
			l.enabled = 1;
			lines.push_back(l);
		}
		else
		{
			cerr << "Invaild input file\n";
			exit(1);
		}
	}
#if DEBUG > 3
	int stop = nodes.size();
	for( unsigned int i = 0; i < stop; i++)
		cout << nodes[i].loc[0] << ',' << nodes[i].loc[1] << endl;
#endif /* DEBUG */
	/* set clearing (background) color */
	glClearColor(0.0, 0.0, 0.0, 0.0);
	glClear(GL_COLOR_BUFFER_BIT);
	glShadeModel(GL_FLAT);
}

/* Executed if the window is resized or moved etc. */
void reshape(int w, int h)
{
	width=w;
	height=h;
	glViewport(0, 0, (GLsizei) w, (GLsizei) h);
	glMatrixMode(GL_PROJECTION);
	glLoadIdentity();
	glOrtho((float)offsetx/width*border, border+(float)offsetx/width*border,
		border+(float)offsety/height*border,
		(float)offsety/height*border, -1.0, 1.0);
	glMatrixMode(GL_MODELVIEW);
	glLoadIdentity();
}

void enable_all()
{
	int stop = lines.size();
	for( int i = 0; i < stop; i++)
		lines[i].enabled=1;
}

/* disable the line at location x,y (scene coordinates) */
void disable( float x, float y)
{
	node_c n0,n1; // a working set of nodes
	int bestline=-1;
	float bestslope=-1;
	int stop = lines.size();
	for(int l=0; l<stop; l++)
	{
		n0=nodes[lines[l].node[0]];
		n1=nodes[lines[l].node[1]];
		/* check if it is in the rectangle bounded by these points */
		// check X direction
		if( x<n0.loc[0] || x>n1.loc[0])
			continue;
		/* points upper left and lower right */
		if( n0.loc[1]>n1.loc[1] )
		{
			if( y>n0.loc[1] || y<n1.loc[1] )
				continue;
		}
		else // points lower left and upper right
		{
			if( y<n0.loc[1] || y>n1.loc[1] )
				continue;
		}
		/* in the bounding box */
		/* take slope and then the absolute difference between
		 * slope of the line and the slope of the mouse click and
		 * one point.
		 */
		float slope=(float)(n0.loc[1]-n1.loc[1])/(n0.loc[0]-n1.loc[0]);
		float mslope=(float)(n0.loc[1]-y)/(n0.loc[0]-x);
		slope=fabs(fabs(slope)-fabs(mslope));
		if( bestslope==-1 || slope < bestslope)
		{
			bestslope=slope;
			bestline=l;
		}
	}
	if( bestline!=-1 )
	{
		lines[bestline].enabled=0;
		cout << "Disbling line connecting nodes " <<
			lines[bestline].node[0] << " and " <<
			lines[bestline].node[1] << endl;
	}
	else
		cout << "Didn't find a line at (" << x << ',' << y << ")\n";
}

void pan_window( int oldx, int oldy, int curx, int cury)
{
	offsetx += oldx - curx;
	offsety += oldy - cury;
	reshape(width,height);
	glutPostRedisplay();
}

void mouse(int button, int state, int x, int y)
{
	float relx, rely;
	// only interested in mouse presses
	if(state==GLUT_DOWN)
	switch (button)
	{
	case GLUT_LEFT_BUTTON:
		relx=(float)x/width*border;
		rely=(float)y/height*border;
		cout << "Click at (" << x << ',' << y << "), (" <<
			relx << ',' <<
			rely << ")\n";
		disable(relx, rely);
		glutPostRedisplay();
		break;
	case GLUT_MIDDLE_BUTTON:
		mouse_pressed =1;
		mouse_lastx = x;
		mouse_lasty = y;
		break;
	case GLUT_RIGHT_BUTTON:
		enable_all();
		glutPostRedisplay();
		break;
	default:
		break;
	}
	if(state==GLUT_UP)
	switch (button)
	{
	case GLUT_LEFT_BUTTON:
		break;
	case GLUT_MIDDLE_BUTTON:
		mouse_pressed =0;
		pan_window(mouse_lastx, mouse_lasty, x, y);
		mouse_lastx = x;
		mouse_lasty = y;
		break;
	case GLUT_RIGHT_BUTTON:
		break;
	default:
		break;
	}
}

void mouse_motion( int x, int y)
{
	/* Return if we aren't interested in this movement */
	if(!mouse_pressed)
		return;

	pan_window(mouse_lastx, mouse_lasty, x, y);
	mouse_lastx = x;
	mouse_lasty = y;
}

int main( int argc, char** argv)
{
	glutInit( &argc, argv);
	glutInitDisplayMode( GLUT_SINGLE | GLUT_RGB);
	glutInitWindowSize( 250, 250);
	glutCreateWindow(argv[0]);
	init(); // once sent OpenGL commands
	// will be called when the window needs to be displayed
	glutDisplayFunc(display);
	glutReshapeFunc(reshape);
	glutMouseFunc(mouse);
	glutMotionFunc(mouse_motion);
	glutMainLoop();
	return 0;
}

