#include <GL/glut.h>
#include <stdlib.h>
#include <math.h>
#include <sys/time.h>
#include <unistd.h>
#include <iostream.h>
#include <time.h>
#include <vector>
#include <fstream.h>
#include <string>
#include <stdio.h>
#include <GL/osmesa.h>
#include <png.h>

static float ymin = 50;
static float ymax = 100;
static int window_size=250;

static int width = window_size; // screen width and height
static int height = window_size;

static void printstring( void *font, char *string)
{
	int len, i;
	len=(int) strlen(string);
	for( i =0; i < len; i++)
		glutBitmapCharacter(font, string[i]);
}

static void printnum( float raster_x, float raster_y, int num )
{
	char array[10];
	glRasterPos2f(raster_x, raster_y);
	snprintf(array, sizeof(array), "%d", num);
	printstring(GLUT_BITMAP_HELVETICA_12, array);
}

static void draw_grid()
{
	const float major = .5;
	const float minor = .3;
	const int division=5;
	float color;
	for( int i=(int)ymin+division; i < (int)ymax; i+=division)
	{
		glBegin( GL_LINES);
			color = i%10? minor:major;
			glColor3f(color,color,color);
			glVertex3f( 0, i, 0);
			glVertex3f( width, i, 0);
		glEnd();
		glColor3f(1,1,1);
		printnum(0, i, (int)i);
	}
}

static void init(void)
{
	/* set clearing (background) color */
	glClearColor(0.0, 0.0, 0.0, 0.0);
	glClear(GL_COLOR_BUFFER_BIT);
	glShadeModel(GL_FLAT);
	glFlush();
}

/* Executed if the window is resized or moved etc. */
static void reshape(int w, int h)
{
	width = w;
	height = h;
	glViewport(0, 0, (GLsizei) w, (GLsizei) h);
	glMatrixMode(GL_PROJECTION);
	glLoadIdentity();
	glOrtho(0, width, ymin, ymax, -1.0, 1.0);
	glMatrixMode(GL_MODELVIEW);
	glLoadIdentity();
}

static int write_file_png( char * fname, unsigned char * buffer, int w, int h)
{
	FILE *file = fopen(fname, "w");
	if( !file )
	{
		perror("file open");
		return -1;
	}

	png_struct *png_ptr = png_create_write_struct(PNG_LIBPNG_VER_STRING,
		(png_voidp)NULL, (png_error_ptr)NULL, (png_error_ptr)NULL);
	if(!png_ptr)
	{
		fclose(file);
		return -1;
	}

	/* if something goes wrong */
	if(setjmp(png_ptr->jmpbuf))
	{
		printf("png error\n");
		return -1;
	}
	
	png_info *info_ptr = png_create_info_struct(png_ptr);
	if( !info_ptr)
	{
		png_destroy_write_struct(&png_ptr,(png_infopp)NULL);
		fclose(file);
		return -1;
	}
	if(setjmp(png_ptr->jmpbuf))
	{
		png_destroy_write_struct(&png_ptr,(png_infopp)NULL);
		fclose(file);
		return -1;
	}
	png_init_io(png_ptr, file);
	png_set_IHDR(png_ptr, info_ptr, w, h, 8, PNG_COLOR_TYPE_RGB_ALPHA,
		PNG_INTERLACE_NONE, PNG_COMPRESSION_TYPE_DEFAULT,
		PNG_FILTER_TYPE_DEFAULT);
	png_write_info(png_ptr, info_ptr);
	/* want to be able to give the png routines an array of lines
	 * so convert make an array and fill it with pointers to
	 * the beginning of each line.  We would have to do this anyway
	 * since line 0 is the top of the screen for png and the bottom
	 * for OpenGL.
	 */
	unsigned char **row_pointers = new unsigned char*[h];
	for( int i=0, offset=w*(h-1)*sizeof(OSMESA_RGBA);
		i<h; i++,offset-=w*sizeof(OSMESA_RGBA))
		row_pointers[i]=buffer+offset;
	
	/* write the png image */
	png_write_image(png_ptr, row_pointers);

	png_write_end(png_ptr, NULL);
	png_destroy_write_struct(&png_ptr, &info_ptr);

	fclose(file);
	return 0;
}

int main( int argc, char** argv)
{
	width = 320;
	height = 240;
	unsigned char *buffer = new unsigned char[320*240*sizeof(OSMESA_RGBA)];
	if( (int)buffer%4 )
	{
		cerr << "Error buffer is not a multiple of 4\n";
		return -1;
	}
	OSMesaContext context = OSMesaCreateContext( OSMESA_RGBA, NULL);
	if(!context)
	{
		cerr << "Error creating context\n";
		return -1;
	}
	if(!OSMesaMakeCurrent(context,buffer,GL_UNSIGNED_BYTE,width,height))
	{
		cerr << "Error making MesaContext current\n";
		return -1;
	}
	init();
	reshape(width,height);
	glClear(GL_COLOR_BUFFER_BIT);
	glFinish();
	draw_grid();
	glFinish();
	if(write_file_png("/tmp/test.png", buffer, width, height)==-1)
	{
		printf("Error writing png file\n");
		return -1;
	}
	cout << "Success wiring png file\n";
	OSMesaDestroyContext( context);
	return 0;
}

