#ifndef _CONTROL_H_
#define _CONTROL_H_

class guiControl
{
  friend class guiMaster;

 public:
    guiControl();
    virtual ~guiControl();
   
    virtual void Display(void);
    virtual void Idle(void);
    virtual void Mouse(int key, int state, int x, int y);
    virtual void Motion(int x, int y);
    virtual void PassiveMotion(int x, int y);
    virtual void Keyboard(unsigned char key, int x, int y);
    virtual void Special(int key, int x, int y);
    virtual void Visibility(int state);
    virtual void Reshape(int w, int h);
    virtual void Entry(int state);
    virtual void Place(int x, int y);
    
    int get_WindowID();
    int get_ParentID();
    int get_Width();
    int get_Height();
    int get_Cursor();
    color_t get_bgColor();
    int get_hHint(int w, int h);
    int get_wHint(int w, int h);
    int get_xHint(int w, int h);
    int get_yHint(int w, int h);

    bool isVisible();
    bool isEnabled();
    
    void setVisible(bool value);
    virtual void setEnabled(bool value);
    void setbg_Display(void (*func)(void));
    void setbg_Idle(void (*func)(void));
    void setbg_Mouse(void (*func)(int,int,int,int));
    void setbg_Motion(void (*func)(int,int));
    void setbg_Passivemotion(void (*func)(int,int));
    void setbg_Keyboard(void (*func)(unsigned char,int,int));
    void setbg_Special(void (*func)(int,int,int));
    void setbg_Visibility(void (*func)(int));
    void setbg_Reshape(void (*func)(int,int));
    void setbg_Entry(void (*func)(int));
    void setbg_Color(color_t value);
    void set_hHint(int val);
    void set_wHint(int val);
    void set_xHint(int val);
    void set_yHint(int val);
    void set_hHint(int (*val)(int,int));
    void set_wHint(int (*val)(int,int));
    void set_xHint(int (*val)(int,int));
    void set_yHint(int (*val)(int,int));
         
     
 protected:
    virtual void Init(int ID, int Parent, int w, int h);

    int isInit;
    int Parent_ID;
    int Window_ID;
    int Width;
    int Height;
    int Cursor;
    int State;
    int Visible;
    int Enabled;
    int xHint;
    int yHint;
    int hHint;
    int wHint;


    int (*xHintFunc)(int,int);
    int (*yHintFunc)(int,int);
    int (*hHintFunc)(int,int);
    int (*wHintFunc)(int,int);
    
    color_t bg_color;
    void (*bg_display)(void);
    void (*bg_idle)(void);
    void (*bg_mouse)(int,int,int,int);
    void (*bg_motion)(int,int);
    void (*bg_passivemotion)(int,int);
    void (*bg_keyboard)(unsigned char,int,int);
    void (*bg_special)(int,int,int);
    void (*bg_visibility)(int);
    void (*bg_reshape)(int,int);
    void (*bg_entry)(int);
   
};

inline int guiControl::get_WindowID() {
  return Window_ID;
};

inline int guiControl::get_Width() {
  return Width;
};

inline int guiControl::get_Height() {
  return Height;
};

inline int guiControl::get_Cursor() {
  return Cursor;
};

inline color_t guiControl::get_bgColor() {
  return bg_color;
};

inline bool guiControl::isVisible() {
  return Visible;
};

inline bool guiControl::isEnabled() {
  return Enabled;
};

inline void guiControl::setbg_Display(void (*func)(void)){
  bg_display = func;
};

inline void guiControl::setbg_Idle(void (*func)(void)) {
  bg_idle = func;
};

inline void guiControl::setbg_Mouse(void (*func)(int,int,int,int)){
  bg_mouse = func;
};

inline void guiControl::setbg_Motion(void (*func)(int,int)){
  bg_motion = func;
};

inline void guiControl::setbg_Passivemotion(void (*func)(int,int)){
  bg_passivemotion = func;
};

inline void guiControl::setbg_Keyboard(
                              void (*func)(unsigned char,int,int)){
  bg_keyboard = func;
};

inline void guiControl::setbg_Special(void (*func)(int,int,int)){
  bg_special = func;
};

inline void guiControl::setbg_Visibility(void (*func)(int)){
  bg_visibility = func;
};

inline void guiControl::setbg_Reshape(void (*func)(int,int)){
  bg_reshape = func;
};

inline void guiControl::setbg_Entry(void (*func)(int)){
  bg_entry = func;
};

inline void guiControl::setbg_Color(color_t value){
  bg_color.r = value.r;
  bg_color.g = value.g;
  bg_color.b = value.b;
  bg_color.a = value.a;
};

inline void guiControl::set_hHint(int val) {
  hHint = val;
  hHintFunc = 0;
}
inline void guiControl::set_wHint(int val){
  wHint = val;
  wHintFunc = 0;
}
inline void guiControl::set_xHint(int val){
  xHint = val;
  xHintFunc = 0;
}

inline void guiControl::set_yHint(int val){
  yHint = val;
  yHintFunc = 0;
}

inline void guiControl::set_hHint(int (*val)(int,int)) {
  hHintFunc = val;
  hHint = -1;
}
inline void guiControl::set_wHint(int (*val)(int,int)) {
  wHintFunc = val;
  wHint = -1;
}
inline void guiControl::set_xHint(int (*val)(int,int)) {
  xHint = -1;
  xHintFunc = val;
}

inline void guiControl::set_yHint(int (*val)(int,int)) {
  yHint = -1;
  yHintFunc = val;
}

inline int guiControl::get_hHint(int w, int h)
{
  if (hHintFunc) 
    return (*hHintFunc)(w,h);
  else
    return hHint;
};

inline int guiControl::get_wHint(int w, int h)
{
  if (wHintFunc) 
    return (*wHintFunc)(w,h);
  else
    return wHint;
};

inline int guiControl::get_xHint(int w, int h)
{
  if (xHintFunc) 
    return (*xHintFunc)(w,h);
  else
    return xHint;
};

inline int guiControl::get_yHint(int w, int h)
{
  if (yHintFunc) 
    return (*yHintFunc)(w,h);
  else
    return yHint;
};
#endif
