#include "master.h"
#include "control.h"
#include "container.h"

#include <iostream>

#define SW_TO_CURRENT(x)   SetCurrentControl(glutGetWindow()); \
                           currentcontrol->x

guiMaster glApplication;
                          
void glApplication_Display(void) {
   glApplication.Display();
}

void glApplication_Idle(void) {
   glApplication.Idle();
}

void glApplication_Mouse(int key, int state, int x, int y) {
   glApplication.Mouse(key,state,x,y);
}

void glApplication_Motion(int x, int y) {
   glApplication.Motion(x,y);
}

void glApplication_PassiveMotion(int x, int y) {
   glApplication.PassiveMotion(x,y);
}

void glApplication_Keyboard(unsigned char key, int x, int y) {
   glApplication.Keyboard(key,x,y);
}

void glApplication_Special(int key, int x, int y) {
   glApplication.Special(key,x,y);
}

void glApplication_Visibility(int state) {
   glApplication.Visibility(state);
}

void glApplication_Reshape(int w, int h) {
   glApplication.Reshape(w,h);
}

void glApplication_Entry(int state) {
   glApplication.Entry(state);
}

guiMaster::guiMaster(void) {
   MouseState = 0;
   currentcontrol = 0;
}

guiMaster::~guiMaster(void) {
   WindowMap::const_iterator i;

   /*iterate over the map and delete the top level controls**
   **do not have to worry about rmove from map because     **
   **it's next on garabage collection                      */
   for(i = windows.begin(); !windows.empty(); i = windows.begin()){
      delete i->second;
   }
   
}

void guiMaster::SetupWindow(int winID) {
  //cout << "Setup up window:  " << winID << endl;
   glutSetWindow(winID);
   glutDisplayFunc(glApplication_Display);
   glutIdleFunc(glApplication_Idle);
   glutMouseFunc(glApplication_Mouse);
   glutMotionFunc(glApplication_Motion);
   glutPassiveMotionFunc(glApplication_PassiveMotion);
   glutKeyboardFunc(glApplication_Keyboard);
   glutSpecialFunc(glApplication_Special);
   glutVisibilityFunc(glApplication_Visibility);
   glutReshapeFunc(glApplication_Reshape);
   glutEntryFunc(glApplication_Entry);
}

guiControl *guiMaster::Find_by_ID(int ID){
   guiControl *returnval = 0;
   ControlMap::const_iterator result;

   result = controls.find(ID);
   if (result != windows.end())
      returnval = result->second;

   return returnval;
}

int guiMaster::Request_Window(char name[], guiControl *binding) {
   int ID = -1;
   int w,h;
   w=binding->get_wHint(-1,-1);
   h=binding->get_hHint(-1,-1);

   if (binding) {
      glutInitWindowSize(w,h);
      ID = glutCreateWindow(name);

      binding->Init(ID, -1, w, h);
      controls[ID] = binding;
      windows[ID] = binding;
   }

   return ID;
}

int guiMaster::Request_SubWindow(guiContainer *parent, 
                                 guiControl *binding) {
   int ID = -1;
   int x,y,w,h,Parent_ID, Height, Width;
   

   Parent_ID=parent->get_WindowID();
   Height=parent->get_Height();
   Width=parent->get_Width();

   x=binding->get_xHint(Height,Width);
   y=binding->get_yHint(Height,Width);
   w=binding->get_wHint(Height,Width);
   h=binding->get_hHint(Height,Width);

   if (binding) {
      ID = glutCreateSubWindow(Parent_ID,x,y,w,h);
      binding->Init(ID, Parent_ID,w,h);
      controls[ID] = binding;
   }

   return ID;
}

void guiMaster::Destroy_Window(int ID) {
   //cout << "Destroy Window:  " << ID << endl;
   if(controls.find(ID) == windows.end())
      return;

   glutDestroyWindow(ID);
   windows.erase(ID);

   if(windows.find(ID) == windows.end())
      windows.erase(ID);
}

void guiMaster::Display() {
   SW_TO_CURRENT(Display());
}

void guiMaster::Idle() {
   if (app_idle) 
      (*app_idle)();
   
   SW_TO_CURRENT(Idle());
}

void guiMaster::Mouse(int key, int state, int x, int y) {
   int bitmap = 0;
   
   switch(key) {
      case GLUT_LEFT_BUTTON:
         bitmap = MOUSESTATE_LEFT;
         break;
      case GLUT_RIGHT_BUTTON:
         bitmap = MOUSESTATE_RIGHT;
         break;
      case GLUT_MIDDLE_BUTTON:
         bitmap = MOUSESTATE_MIDDLE;
         break;
   }
   
   if (state == GLUT_DOWN)
      MouseState = MouseState | bitmap;
   else
      MouseState = MouseState & (~bitmap);

   SW_TO_CURRENT(Mouse(key,state,x,y));
}

void guiMaster::Motion(int x, int y) {
   SW_TO_CURRENT(Motion(x,y));
}

void guiMaster::PassiveMotion(int x, int y) {
   SW_TO_CURRENT(PassiveMotion(x,y));
}

void guiMaster::Keyboard(unsigned char key, int x, int y) {
   SW_TO_CURRENT(Keyboard(key,x,y));
}

void guiMaster::Special(int key, int x, int y) {
   SW_TO_CURRENT(Special(key,x,y));
}

void guiMaster::Visibility(int state) {
   SW_TO_CURRENT(Visibility(state));
}

void guiMaster::Reshape(int w, int h) {
   SW_TO_CURRENT(Reshape(w,h));
}

void guiMaster::Entry(int state) {
   SW_TO_CURRENT(Entry(state));
}


