#ifndef _MASTER_H_
#define _MASTER_H_

#include <GL/glut.h>
#include <iostream>
#include <map>
#include <list>

#define MOUSESTATE_LEFT 1
#define MOUSESTATE_RIGHT 2
#define MOUSESTATE_MIDDLE 3

class guiControl;
class guiContainer;
class guiWindow;
class guiSubWindow;

typedef struct _color {
  double r;
  double g;
  double b;
  double a;
}color_t;

typedef map<int, guiControl *, less< int > > ControlMap;
typedef map<int, guiControl *, less< int > > WindowMap;

void glApplication_Display(void);
void glApplication_Idle(void);
void glApplication_Mouse(int key, int state, int x, int y);
void glApplication_Motion(int x, int y);
void glApplication_PassiveMotion(int x, int y);
void glApplication_Keyboard(unsigned char key, int x, int y);
void glApplication_Special(int key, int x, int y);
void glApplication_Visibility(int state);
void glApplication_Reshape(int w, int h);
void glApplication_Entry(int state);

class guiMaster
{
  friend class guiControl;
  friend class guiContainer;
  friend class guiWindow;
  friend class guiSubWindow;
  
   friend void glApplication_Display(void);
   friend void glApplication_Idle(void);
   friend void glApplication_Mouse(int key, int state, int x, int y);
   friend void glApplication_Motion(int x, int y);
   friend void glApplication_PassiveMotion(int x, int y);
   friend void glApplication_Keyboard(unsigned char key, int x, int y);
   friend void glApplication_Special(int key, int x, int y);
   friend void glApplication_Visibility(int state);
   friend void glApplication_Reshape(int w, int h);
   friend void glApplication_Entry(int state);

 public:
    guiMaster();
    ~guiMaster();

    void Init(int *argc, char **argv);
    void Start(void);
    void SetupWindow(int winID);
    guiControl *Find_by_ID(int ID);
    guiControl *CurrentControl();
    int Mouse_Info();
    void Set_Application_Idle(void (*func)());
     
 protected:
    int Request_Window(char name[], guiControl *binding);
    int Request_SubWindow(guiContainer *parent, 
                          guiControl *binding);
    void Destroy_Window(int ID);
    
    void SetCurrentControl(int ID);

    void Display(void);
    void Idle(void);
    void Mouse(int key, int state, int x, int y);
    void Motion(int x, int y);
    void PassiveMotion(int x, int y);
    void Keyboard(unsigned char key, int x, int y);
    void Special(int key, int x, int y);
    void Visibility(int state);
    void Reshape(int w, int h);
    void Entry(int state);

 private:

    void (*app_idle)();
    int clear_flags;
    int MouseState;
    guiControl *currentcontrol;
    ControlMap controls;
    WindowMap windows;

};

inline void guiMaster::Init(int *argc, char **argv){
  glutInit(argc, argv);
  glutInitDisplayMode(GLUT_DEPTH | GLUT_DOUBLE | GLUT_RGBA);
};

inline void guiMaster::Start(void){
  glutMainLoop();
};

inline guiControl *guiMaster::CurrentControl() {
  return currentcontrol;
};

inline int guiMaster::Mouse_Info() {
  return MouseState;
}

inline void guiMaster::Set_Application_Idle(void (*func)(void)){
  app_idle = func;
}

inline void guiMaster::SetCurrentControl(int ID)
{
     currentcontrol = Find_by_ID(ID);
     /*glutSetWindow(ID); //Not Sure on this line*/
}

extern guiMaster glApplication;

#endif
