// glImage2dBitmap.cpp: implementation of the glImage2dBitmap class.
//
//////////////////////////////////////////////////////////////////////

#include "glImage2dBitmap.h"

#include <math.h>
#include <memory.h>
#include <stdio.h>
#include <stdlib.h>

#define MAX(X,Y) ( (X>Y)?X:Y )
#define MIN(X,Y) ( (X<Y)?X:Y )

// ***
// Image manipulation support
void GetSubImage(const WORD sw,const WORD sh,const BYTE *simg,
				 const WORD left,const WORD right,
				 const WORD top,const WORD bottom,
				 BYTE *dimg                                   );
void Resample(const WORD sw,const WORD sh,const BYTE *simg,
			  const WORD dw,const WORD dh,BYTE *dimg       );

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

const WORD sm_cachesize=5;

glImage2dBitmap::glImage2dBitmap()
{
	WORD i;

	m_gensize=sm_cachesize+1;
	m_gen=new GLI2DB_IMAGE[m_gensize];
	for(i=0;i<m_gensize;i++)
	{
		m_gen[i].image=0;
		m_gen[i].w=0;
		m_gen[i].h=0;
		m_gen[i].subleft=0;
		m_gen[i].subright=0;
		m_gen[i].subtop=0;
		m_gen[i].subbottom=0;
	}
	m_current=0;
}

glImage2dBitmap::~glImage2dBitmap()
{
	DeleteImages();
	delete [] m_gen;
}

void glImage2dBitmap::DeleteImages()
{
	WORD i;

	for(i=0;i<m_gensize;i++)
		if(m_gen[i].image!=0)
		{
			delete [] m_gen[i].image;
			m_gen[i].image=0;
		}

	return;
}

void glImage2dBitmap::Resize(const WORD w,const WORD h)
{
	m_current=FindImage(w,h,
		                m_gen[m_current].subleft,m_gen[m_current].subright,
		                m_gen[m_current].subtop,m_gen[m_current].subbottom );
	return;
}

void glImage2dBitmap::Zoom(const GLfloat x,const GLfloat y,
		                   const GLfloat w,const GLfloat h )
{
	WORD left,right,top,bottom;

	left=floor(x*m_gen[0].subright+0.5);
	right=floor((x+w)*m_gen[0].subright+0.5);
	top=floor(y*m_gen[0].subbottom+0.5);
	bottom=floor((y+h)*m_gen[0].subbottom+0.5);

	m_current=FindImage(m_gen[m_current].w,m_gen[m_current].h,
		                left,right,top,bottom                 );

	return;
}

void glImage2dBitmap::Draw(const GLfloat x,const GLfloat y)
{
	glRasterPos2f(x,y+m_gen[m_current].h);
	glPixelStorei(GL_UNPACK_ALIGNMENT, 1);
	glDrawPixels(m_gen[m_current].w,m_gen[m_current].h,GL_RGBA,
		         GL_UNSIGNED_BYTE,m_gen[m_current].image       );
}

bool glImage2dBitmap::Setup(BYTE *image,const WORD w,const WORD h)
{
	WORD i,i1,wc=w<<2;
	BYTE *p,*p1,t;


#ifdef DEBUG
fprintf(stderr, "glImage2dBitmap::Setup - "
		        "Deleting old images\n"    );
#endif
	DeleteImages();

#ifdef DEBUG
fprintf(stderr, "glImage2dBitmap:: - "
		        "Row-swapping original image\n");
#endif
	// Crude & slow, but only done once
	for(i=0;i<(h-i-1);i++)
		for(p=&image[i*wc],p1=&image[(h-i-1)*wc],i1=0;
		    i1<wc;i1++,p++,p1++                          )
		{
			t=*p;
			*p=*p1;
			*p1=t;
		}

#ifdef DEBUG
fprintf(stderr, "glImage2dBitmap:: - "
		        "Storing original image\n");
#endif
	m_gen[0].image=image;
	m_gen[0].w=w;
	m_gen[0].h=h;
	m_gen[0].subleft=0;
	m_gen[0].subright=w-1;
	m_gen[0].subtop=0;
	m_gen[0].subbottom=h-1;

	// Don't delete image
	return false;
}

WORD glImage2dBitmap::FindImage(const WORD w, const WORD h,
                                const WORD left, const WORD right,
				                const WORD top, const WORD bottom )
{
	const WORD subw=right-left+1,subh=bottom-top+1;	
	WORD i,empty=MAXWORD;
	BYTE *subimage=0;

	for(i=0;i<m_gensize;i++)
		if(m_gen[i].image==0)
			empty=MIN(empty,i);
		else
			if((m_gen[i].w==w)&&(m_gen[i].h==h)&&
			   (m_gen[i].subleft==left)&&(m_gen[i].subright==right)&&
			   (m_gen[i].subtop==top)&&(m_gen[i].subbottom==bottom)  )
				break;

	if(i>=m_gensize) // didn't find a match
	{
		if(empty<m_gensize)
			i=empty;
		else
		{
			// Pick one at random to replace
			i=(rand()%(m_gensize-1))+1;
			delete [] m_gen[i].image;
		}
		// Retrieve subimage
		subimage=new BYTE[subh*subw*4];
		GetSubImage(m_gen[0].w,m_gen[0].h,m_gen[0].image,
			        left,right,top,bottom,subimage       );

		// Scale image to correct size
		m_gen[i].image=new BYTE[w*h*4];
//		gluScaleImage(GL_RGBA,
//			          m_gen[0].w,m_gen[0].h,GL_UNSIGNED_BYTE,m_gen[0].image,
//			          w,h,GL_UNSIGNED_BYTE,m_gen[i].image                   );
//		Resample(m_gen[0].w,m_gen[0].h,m_gen[0].image,
//			     w,h,m_gen[i].image                   );
		Resample(subw,subh,subimage,
			     w,h,m_gen[i].image );
//		m_gen[i].w=w;
//		m_gen[i].h=h;
//		m_gen[i].subleft=m_gen[0].subleft;
//		m_gen[i].subright=m_gen[0].subright;
//		m_gen[i].subtop=m_gen[0].subtop;
//		m_gen[i].subbottom=m_gen[0].subbottom;
		m_gen[i].w=w;
		m_gen[i].h=h;
		m_gen[i].subleft=left;
		m_gen[i].subright=right;
		m_gen[i].subtop=top;
		m_gen[i].subbottom=bottom;

		delete [] subimage;
	}

	return i;
}

// ********* Non-class functions ***********

void GetSubImage(const WORD sw,const WORD sh,const BYTE *simg,
				 const WORD left,const WORD right,
				 const WORD top,const WORD bottom,
				 BYTE *dimg                                   )
{
	const WORD dw=right-left+1,dh=bottom-top+1;
	const BYTE *ps;
	BYTE *pd;
	WORD i;

	ps=simg+((sh-bottom-1)*sw+left)*4;
	pd=dimg;

	for(i=0;i<dh;i++)
	{
		memcpy(pd,ps,dw*4);
		ps+=sw*4;
		pd+=dw*4;
	}

	return;
}

void Resample(const WORD sw,const WORD sh,const BYTE *simg,
			  const WORD dw,const WORD dh,BYTE *dimg       )
{
	WORD cdh,csh,haccum;
	WORD cdw,waccum;
	const BYTE *ps;
	BYTE *pd;

    // Faster for expanding
	for(cdh=0,csh=0,haccum=0;csh<sh;csh++)
	{
		// Do this row
		pd=&dimg[cdh*dw*4];
		ps=&simg[csh*sw*4];
		for(cdw=0,waccum=0;cdw<dw;cdw++,pd+=4)
		{
			*(DWORD*)pd=*(DWORD*)ps;
			
			// Get next source col
			waccum+=sw;
			while(waccum>dw)
			{
				waccum-=dw;
				ps+=4;
			}
		}

		// Copy repeated rows
		haccum+=dh;
		ps=&dimg[cdh*dw*4]; // Copy from dimg row from above
		while(haccum>sh)
		{
			haccum-=sh;
			cdh++;
			pd=&dimg[cdh*dw*4];
			memcpy(pd,ps,dw*4);
		}
	}
/**/
	return;
}
