// glImage2dTexture.cpp: implementation of the glImage2dTexture class.
//
//////////////////////////////////////////////////////////////////////

#include "glImage2dTexture.h"

#include <stdio.h>

#define MAX(X,Y) ( (X>Y)?X:Y )
#define MIN(X,Y) ( (X<Y)?X:Y )

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

glImage2dTexture::glImage2dTexture() : glImage2d()
{
	m_texleft=m_textop=0.0f;
	m_texright=m_texbottom=1.0f;

	m_texid=0;
}

glImage2dTexture::~glImage2dTexture()
{
	glDeleteTextures(1,&m_texid);
}

void glImage2dTexture::Draw(const GLfloat x, const GLfloat y)
{
	glPushAttrib(GL_ENABLE_BIT|GL_TEXTURE_BIT|GL_CURRENT_BIT);
	glEnable(GL_TEXTURE_2D);
	glBindTexture (GL_TEXTURE_2D, m_texid);
	glColor3f(1.0,1.0,1.0);
	glBegin(GL_QUADS);
		// Upper-left
		glTexCoord2f(m_texleft,m_textop);
		glVertex3f(x,y,0.0);
		// Upper-right
		glTexCoord2f(m_texright,m_textop);
		glVertex3f(x+m_width,y,0.0);
		// Lower-right
		glTexCoord2f(m_texright,m_texbottom);
		glVertex3f(x+m_width,y+m_height,0.0);
		// Lower-left
		glTexCoord2f(m_texleft,m_texbottom);
		glVertex3f(x,y+m_height,0.0);
	glEnd();
	glPopAttrib();

	return;
}

bool glImage2dTexture::Setup(BYTE *image, const WORD w, const WORD h)
{
	// Convert to texture
	if(!m_texid)
		glGenTextures(1,&m_texid);

#ifdef DEBUG
fprintf(stderr, "glImage2dTexture::Setup - "
		        "Setting parameters\n"      );
#endif
	glBindTexture (GL_TEXTURE_2D, m_texid);
	glPixelStorei (GL_UNPACK_ALIGNMENT, 1);
	glTexParameteri (GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_REPEAT);
	glTexParameteri (GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_REPEAT);
	glTexParameteri (GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR);
	glTexParameteri (GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR);
	//glTexEnvf (GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_MODULATE);
	glTexEnvf (GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_DECAL);

#ifdef DEBUG
fprintf(stderr, "glImage2dTexture::Setup - "
		        "Building Mipmaps\n"       );
#endif
	gluBuild2DMipmaps(GL_TEXTURE_2D,GL_RGB5_A1,w,h,GL_RGBA,GL_UNSIGNED_BYTE,image);
	//gluBuild2DMipmaps(GL_TEXTURE_2D,3,w,h,GL_RGB,GL_UNSIGNED_BYTE,image);

	m_pixelwidth=w;
	m_pixelheight=h;

	m_texleft=m_textop=0.0f;
	m_texright=m_texbottom=1.0f;

#ifdef DEBUG
fprintf(stderr, "glImage2dTexture::Setup - "
		        "Deleting memory\n"         );
#endif
	return true;
}

void glImage2dTexture::Resize(GLfloat w, GLfloat h)
{
	glImage2d::Resize(w,h);
	return;
}

void glImage2dTexture::Zoom(const GLfloat x,const GLfloat y,
                            const GLfloat w,const GLfloat h )
{
	m_texleft=x;
	m_texright=x+w;
	m_textop=y;
	m_texbottom=y+h;
}
