#define _SETUP_CPP

#include "common.h"

void handleArgs(int argc,char **argv);

float FPS_LastSec=0.0,FPS_Avg=0.0;
int FPS_TotalFrames=0,FPS_LastSecTime=0,FPS_LastSecFrames=0; 

char BG_Filename[256]="bg.png";

int main(int argc, char **argv) 
{
    // Process any cmd-line args
    handleArgs(argc,argv);

	glutInit(&argc, argv); 
    glutInitDisplayMode(GLUT_DEPTH | GLUT_DOUBLE | GLUT_RGBA); 
	glutInitWindowPosition(100,100); 
	glutInitWindowSize(320,320); 
	glutCreateWindow("3D Tech- GLUT Tutorial");

	glutReshapeFunc(changeSize);
	DrawSetup();

	// Setup mouse functions
	Mouse_Init();
	SetClickCallback(B_LEFT,OnLeftClick);
	SetClickCallback(B_RIGHT,OnRightClick);
	SetDragCallback(B_LEFT,OnLeftDrag);
	SetDragCallback(B_RIGHT,OnRightDrag);

	// Enable keyboard input
    glutKeyboardFunc(OnKeyPress);

	glutMainLoop();

	DrawKill();

	return 0;
}  

void changeSize(int w, int h)
{
	// Prevent a divide by zero, when window is too short
	// (you cant make a window of zero width).
	if(h == 0)
		h = 1;

	float ratio = 1.0* w / h;

	glViewport(0, 0, w, h);

	DrawResize(w,h);
	return;
}

// Call once per frame
void doFPSCalc(void)
{
	int time=glutGet(GLUT_ELAPSED_TIME);

	// Sum number of frames
	FPS_TotalFrames++;
	FPS_LastSecFrames++;

	// Update once per second
	if( (time-FPS_LastSecTime)>1000 )
	{
		FPS_Avg=1000.0*FPS_TotalFrames/time;
		FPS_LastSec=1000.0*FPS_LastSecFrames/(time-FPS_LastSecTime);

		FPS_LastSecFrames=0;
		FPS_LastSecTime=time;
	}

	return;
}

bool FPSCheck(const float targetFPS)
{
	int time=glutGet(GLUT_ELAPSED_TIME);
	float curFPS;

	curFPS=1000.0*(FPS_LastSecFrames+1)/(time-FPS_LastSecTime);

	return (curFPS<targetFPS);
}

void setView(const VIEWMODE vm)
{
	int w=glutGet(GLUT_WINDOW_WIDTH),
		h=glutGet(GLUT_WINDOW_HEIGHT);

	if(w==0) w=1;
	if(h==0) h=1;

	glMatrixMode(GL_PROJECTION);
	glLoadIdentity(); 

	switch(vm)
	{
		case VIEW_PIXEL:
			gluOrtho2D(0, w, 0, h); 
			glScalef(1, -1, 1); 
			glTranslatef(0, -h, 0); 
			break;

		case VIEW_UNIT:
		default:
			if(w>h)
				gluOrtho2D(-(float)w/h, (float)w/h, -1.0, 1.0);
			else
				gluOrtho2D(-1.0, 1.0, -(float)h/w, (float)h/w);
			glScalef(1, -1, 1);
			break;
	}

	glMatrixMode(GL_MODELVIEW);
	glLoadIdentity();

	return;
}

void renderBitmapString( float x, float y, void *font, char *string)
{ 
	char *c; 
	glRasterPos2f(x, y); 
	for (c=string; *c != '\0'; c++) 
	{ 
		glutBitmapCharacter(font, *c); 
	}

	return;
}  

void handleArgs(int argc,char **argv)
{
    int i;

    for(i=1;i<argc;i++)
    {
        if(argv[i][0]=='-')
            switch(argv[i][1])
            {
                case 's':
                    // Set software
                    setSoftwareMode();
                    break;

                case 'h':
                    // Set hardware
                    setHardwareMode();
                    break;

                case 'f':
                    i++;
                    strncpy(BG_Filename,argv[i],256);
                    break;
            }
    }
    
    return;
}