#include "kdwString.h"

////////////////////////////////////////////////////
//
//    Utilities
//
////////////////////////////////////////////////////

int chartest(char A, char B); 
int chartest(char A, char B) 
{
	return(A==B);
}




////////////////////////////////////////////////////
//
//    Constructors/Destructor
//
////////////////////////////////////////////////////

String::String(void) 
{
	sPtr = new char[1];
	length = 0;
	sPtr[0] = '\0';
}

String::String(const char *s) 
{
	if(s)
	{
		length = strlen(s);
		sPtr = new char[length + 1];
		assert(sPtr != 0); 	//terminate if memory not allocated.
		strcpy(sPtr,s);
	}
	else
	{
		length = 0;
		sPtr = new char[1];
		assert(sPtr != 0); 	//terminate if memory not allocated.
		sPtr[0]=0;
	}
}

String::String(int in) 
{
	int i, j=1;
	for (i=1; in / (j*=10); i++); // i is length of string
//	cout << "length of " << in << ": " << i << "." << endl;
	sPtr = new char[i + 1]; // ints cant be this big!
	assert(sPtr != 0);	//terminate if memory not allocated.

	i=0;
	while (j /= 10)
	{
//		cout << "j = " << j << "in/j" << in/j << endl;
		sPtr[i++] = '0' +  in / j;
		in -= j*(in/j);
	}
	sPtr[i] = 0;
//	cout << sPtr << endl;
	length = strlen(sPtr);
}

String::String(const String &copy) {
	length = copy.length;
	sPtr = new char[length+1];
	assert(sPtr != 0);	//terminate if memory not allocated.
	strcpy(sPtr, copy.sPtr);
}


String::~String() {
	delete [] sPtr;
}





////////////////////////////////////////////////////
//
//    Assignment
//
////////////////////////////////////////////////////


const String &String::operator=(const String &right) 
{
	if (&right != this) 
        {
		delete [] sPtr;
		length = right.length;
		sPtr = new char[length+1];
		assert(sPtr);
		strcpy(sPtr, right.sPtr);
	}
	return(*this);
}


String & String::operator = (const char * s) 
{
  if (s == 0) s = "\0";
  length = strlen(s);
  delete[]sPtr;
  sPtr = new char[length + 1];
  strcpy(sPtr, s);
  return *this;
}


String & String::operator = (char c) 
{
  if ( c == '\0' ) 
  {
    length = 1;
    delete[]sPtr;
    sPtr = new char[length + 1];
    sPtr[0] = '\0';
  } else {
    length = 2;
    delete[]sPtr;
    sPtr = new char[length + 1];
    sPtr[0] = c;
    sPtr[1] = '\0';
  }
  return *this;
}












////////////////////////////////////////////////////
//
//    More complex functions
//
////////////////////////////////////////////////////

String String::Right(char A) const 
{
  const char *p = sPtr;
  //while ( *p && chartest(*p,A) ) p++;
  while ( *p && !chartest(*p,A) ) p++;
  while ( *p && chartest(*p,A) ) p++;
  return (String(p));
}


// Return left part of sPtr up to char A. 
String String::Left(char A) const 
{
  String ret;
  const char *p = sPtr;
//  while ( *p && chartest(*p,A) ) p++;
  while ( *p && !chartest(*p,A) )  ret += *(p++);
  return (ret);
}

//
// Returns the index of begining char of 
// char A[] in the string.
//
int String::Instr(char A[]) const
{
	char* IndexPtr = sPtr;
	char* Ptr = strstr(sPtr,A);
	if (!Ptr) return(-1);
	int i=0;
	
	while(IndexPtr != Ptr) 
	{
		i++;
		IndexPtr++;
	}
	return(i);
}

//
// Pass string to system as a command.
//
void String::System() const 
{
	char* temp = new char[length+1];
	strcpy(temp,sPtr);
	system(temp);
	delete temp;
}

int String::IsNumeric() const 
{
	if (length < 1 ) return 0;

	for(int i = 0; i < length ; i++)
        	if((isdigit(sPtr[i]) == 0) && !((i == 0) && (sPtr[i] == '-') ))
            	return 0;
    	return 1;
}


void String::Clear() 
{
	length = 0;
	strcpy(sPtr,"");
}



////////////////////////////////////////////////////
//
//    General Operators
//
////////////////////////////////////////////////////


String &String::operator+=(const String &right) {
	char *tempPtr = sPtr;
	tempPtr = sPtr;
	length += right.length;
	sPtr = new char[length + 1];
	assert(sPtr != 0);	//terminate if memory not allocated.
	strcpy(sPtr, tempPtr);
	strcat(sPtr, right.sPtr);
	delete [] tempPtr;
	
	return *this;
}


String & String::operator += (char c) {
	if ( c == '\0' ) return *this;
 	char *tempPtr = sPtr;
	length += 1;
  	sPtr = new char[length + 1];
	assert(sPtr != 0);	//terminate if memory not allocated.
	strcpy(sPtr,tempPtr);
	sPtr[length - 1] =c ;
	sPtr[length] = '\0';	
	delete[]tempPtr;
	return *this;
}


String String::operator+(const String &right) const
{
	String r=this->sPtr;
	r+=right;
	return r;
}


String operator + (const char * s1, const String & s2) ;
String operator + (const char * s1, const String & s2) {
  String r(s1);
  r += s2;
  return r;
}





int String::operator!() const { return length == 0; }

int String::operator==(const String &right) const
	{ return strcmp(sPtr, right.sPtr) == 0; }

int String::operator!=(const String &right) const
	{ return strcmp(sPtr, right.sPtr) != 0; }

int String::operator<(const String &right) const
	{ return strcmp(sPtr, right.sPtr) < 0; }

int String::operator>(const String &right) const
	{ return strcmp(sPtr, right.sPtr) > 0; }

int String::operator<=(const String &right) const
	{ return strcmp(sPtr, right.sPtr) <= 0; }

int String::operator>=(const String &right) const
	{ return strcmp(sPtr, right.sPtr) >= 0; }

int String::operator/(const String &right) const {
    if (this->length < 1) return(0);
    return ((int)strstr(sPtr, right.sPtr));
}

int String::operator/(const char c[]) const {
    return((int)strstr(this->sPtr,c));
}

int String::operator/(const char &c) const {
    char *p = sPtr;
    for(int i=0;i<length;i++) if (*p++ == c) return(i+1);
    return(0);
}


int operator / (const char c[], const String d);
int operator / (const char c[], const String d) {
	return((int)strstr(c,d.chars()));
}


char &String::operator[] (int subscript) const {
	assert(subscript >= 0 && subscript < length);
	return sPtr[subscript];
}


String String::operator() (int index, int subLength) {
	assert(index >= 0 && index <= length && subLength > 0 );
	String sub;
	for(int j=index;j<index+subLength;j++) sub += sPtr[j];
	return sub;
}



void String::Replace(char A[], char B) 
{
	int i;
	int LenBig = strlen(sPtr);
	int LenSml = strlen(A);
	int Index = this->Instr(A);

	sPtr[Index] = B;
	for(i=Index+1;i<=LenBig-LenSml;i++) 
	{
		sPtr[i] = sPtr[i+LenSml-1];
		sPtr[i+LenSml-1] = ' ';
	}
}








////////////////////////////////////////////////////
//
//    IO
//
////////////////////////////////////////////////////


ostream &operator << (ostream &output, const String &s) {
	output << s.sPtr;
	return output;
}

istream &operator>>(istream &input, String &s) {
	char temp[100];
	input >> setw(100) >> temp;
	s = temp;
	return input;
}



istream &String::getline(istream &input, int N) {
	length = N;
	sPtr = new char[length+1];
	return(input.getline(sPtr,N));
}



