#include <GL/glut.h>
#include "master.h"
#include <stdlib.h>
#include <stdio.h>
#include <window.h>
#include <bar.h>
#include <button.h>
#include <status.h>
#include <toggle.h>
#include <math.h>
#include <string.h>
#include <ctype.h>

#ifdef WIN32
#define WIN32_LEAN_AND_MEAN
#include <windows.h>
#include <commdlg.h>
#else
#include <unistd.h>
#endif


#include "display/config.h"
#include "display/display.h"
#include "substuff.h"

using namespace std;

char usage[] = " [-show3d] [-nobg] [-aNETWORK_ADDRESS] [-pNETWORK_PORT] conffile";
char ext_usage1[] = "\n     conffile          -- configuration file";
char ext_usage2[] = "     -pNETWORK_PORT    -- NETWORK_PORT = port of backend server";
char ext_usage3[] = "     -aNETWORK_ADDRESS -- NETWORK_ADDRESS = inet addres of backend server";
char ext_usage4[] = "     -nobg             -- turn off background";
char ext_usage5[] = "     -show3d           -- enable 3d mode";		

int enable_3d = 0;

#ifdef WIN32
bool WIN32_GetFname(char *name,const int namesize);
#endif

void print_usage(char name[],char *bad=0);
void print_usage(char name[],char bad[])
{
#ifdef WIN32
char buff[1024];
sprintf(buff,"Bad argument: %s\n\n"
             "Usage: %s conffile <flags>\n"
             "%s\n%s\n%s\n%s\n%s\n",
             bad,name,
             ext_usage1,ext_usage2,ext_usage3,
             ext_usage4,ext_usage5            );
MessageBox(0,buff,"Command Line Usage",MB_OK);
#else
              if(bad!=0)
                 cerr << "Bad argument: " << bad << endl;
				  cerr << endl << "Usage:" << name  << usage << endl;	
			     cerr << ext_usage1 << endl;				
			     cerr << ext_usage2 << endl;				
			     cerr << ext_usage3 << endl;				
			     cerr << ext_usage4 << endl;				
			     cerr << ext_usage5 << endl << endl;				
#endif
	exit(1);
}

guiWindow *Main;
guiSubWindow *swMain,*swMiniMap,*swScrBar1, *swScrBar2;
guiBar *swMenuBar;
guiButton *Button1, *Button2, *Button3, *Button4, *Button5;
guiStatusBar *sBar1;
guiToggle *tButton7, *tButton8;

int sBar_Height = 30;
int swMenuBar_Height = 55;
color_t MenuBar_BG = {0.703f, 0.703f, 0.703f, 1.0f},
         Button_BG = {0.703f,0.703f, 0.703f, 1.0f},
         Button_DR = {0.20f, 0.20f, 0.20f, .8},
         Button_LT = {0.95f, 0.95f, 0.95f, .8},
         Main_BG = {0.0f,0.0f,0.0f,0.0f};

int FullWidth(int w, int h)
{
  return w;
}

int swMain_y(int w, int h) {
  return swMenuBar_Height - 1;
}
int swMain_h(int w, int h) {
  return h - swMenuBar_Height - sBar_Height;
}
int sBar_y(int w, int h) {
  return h - 1 - sBar_Height;
}

void set_status(char *target) {
	sBar1->setString(strlen(target), target);
}

int main(int argc, char **argv)
{
  float w,h;
  char filename[512];
  int i;
  int length;

  Main = new guiWindow();
  swMain = new guiSubWindow();
  swMiniMap = new guiSubWindow();
  swMenuBar = new guiBar();
  Button1 = new guiButton();
  Button2 = new guiButton();
  Button3 = new guiButton();
  Button4 = new guiButton();
  Button5 = new guiButton();
  tButton7 = new guiToggle();
  tButton8 = new guiToggle();
  sBar1 = new guiStatusBar();

  // Hack to make bg loads work
  DB_count=10;

  glApplication.Init(&argc, argv, 25);
  init_icons();
  
  filename[0]='\0';
  print_string = set_status;
  for(i=1;i<argc;i++)
  {
     if(argv[i][0]!='-')
        strcpy(filename,argv[i]);
     else
        switch(argv[i][1])
        {
			  case 'a':
				  length = strlen(&(argv[i][1]));
				  if (length > 511) {
					  strncpy(network_address, &(argv[i][1]),511);
				  }
				  else {
					  strncpy(network_address, &(argv[i][1]), length);
				  }
				  break;
			  case 'p':
				  if(!isdigit(argv[i][2]))
					  print_usage(argv[0],argv[i]);
				  network_port = atoi(&(argv[i][2]));
				  break;
           case 'n':
              if(!strcmp(argv[i],"-nobg"))
                 nodrawbackground=true;
              else
                 print_usage(argv[0],argv[i]);
              break;
              case 's':
				  if(!strcmp(argv[i],"-show3d"))
					  enable_3d=1;
              else
                 print_usage(argv[0],argv[i]);
				  break;
           default:
					  print_usage(argv[0],argv[i]);
	     }
   }
	
#ifdef WIN32
   while(filename[0]=='\0')
   {
      if(!WIN32_GetFname(filename,512))
         // They hit cancel, so quit
         return 0;

      if(config_info.ReadFile(filename))
      {
         MessageBox(0,"Error reading config file","Bad file",MB_OK);
         filename[0]='\0';
      }
   }

#else
  if( config_info.ReadFile(filename) ) {
    cerr << "Error reading config file aborting\n";
	 print_usage(argv[0]);
  }
#endif

  Main->setbg_Color(Main_BG);
  Main->set_hHint(400);
  Main->set_wHint(400);
  Main->Create("Power Grid Simulation");
  Main->SetCursor(GLUT_CURSOR_LEFT_ARROW);
  glApplication.SetIdle(150);

  swMain->set_hHint(swMain_h);
  swMain->set_wHint(FullWidth);
  swMain->set_yHint(swMain_y);
  swMain->set_xHint(0);
  swMain->setbg_Mouse(mouse);
  swMain->setbg_Motion(mouse_motion);
  swMain->setbg_Display(display);
  swMain->setbg_Reshape(reshape);
  swMain->setbg_Idle(idle);

  swMiniMap->setbg_Mouse(mm_mouse);
  swMiniMap->setbg_Motion(mm_mouse_motion);
  swMiniMap->setbg_Display(mm_display);
  swMiniMap->setbg_Reshape(mm_reshape);
  swMiniMap->setbg_Idle(idle);
  swMiniMap->set_xHint(10);
  swMiniMap->set_yHint(10);
  
  // Make proportional to area
  h=fabs(config_info.extent_bottom-config_info.extent_top);
  w=fabs(config_info.extent_right-config_info.extent_left);
  if(w<h)
  {
    swMiniMap->set_wHint((int)(100*w/h));
    swMiniMap->set_hHint(100);
  }
  else
  {
    swMiniMap->set_wHint(100);
    swMiniMap->set_hHint((int)(100*h/w));
  }

  glShadeModel(GL_FLAT);

  swMenuBar->set_hHint(swMenuBar_Height);
  swMenuBar->set_wHint(FullWidth);
  swMenuBar->set_xHint(0);
  swMenuBar->set_yHint(0);
  swMenuBar->setbg_Color(MenuBar_BG);
  
  sBar1->set_hHint(sBar_Height);
  sBar1->set_wHint(FullWidth);
  sBar1->set_xHint(0);
  sBar1->set_yHint(sBar_y);
  sBar1->setbg_Color(MenuBar_BG);
  sBar1->setString(1," ");

  Button1->setbg_Color(Button_BG);
  Button1->setdk_Color(Button_DR);
  Button1->sethl_Color(Button_LT);
  Button1->set_wHint(50);
  Button1->setFace(draw_Quit_Icon);
  Button1->setAction(action_Quit);

  Button2->setbg_Color(Button_BG);
  Button2->setdk_Color(Button_DR);
  Button2->sethl_Color(Button_LT);
  Button2->set_wHint(50);
  Button2->setFace(draw_Off_On_Icon);
  Button2->setAction(action_Off_On);

  Button3->setbg_Color(Button_BG);
  Button3->setdk_Color(Button_DR);
  Button3->sethl_Color(Button_LT);
  Button3->set_wHint(50);
  Button3->setFace(draw_Revert_Icon);
  Button3->setAction(action_Revert);

  Button4->setbg_Color(Button_BG);
  Button4->setdk_Color(Button_DR);
  Button4->sethl_Color(Button_LT);
  Button4->set_wHint(50);
  Button4->setFace(draw_On_Off_Icon);
  Button4->setAction(action_On_Off);

  Button5->setbg_Color(Button_BG);
  Button5->setdk_Color(Button_DR);
  Button5->sethl_Color(Button_LT);
  Button5->set_wHint(50);
  Button5->setFace(draw_Recalc_Icon);
  Button5->setAction(action_Recalc);

  tButton7->setbg_Color(Button_BG);
  tButton7->setdk_Color(Button_DR);
  tButton7->sethl_Color(Button_LT);
  tButton7->set_wHint(50);
  tButton7->setFace(draw_Info_Icon);
  tButton7->setAction(action_Info);

  if(enable_3d)
  {
    tButton8->setbg_Color(Button_BG);
    tButton8->setdk_Color(Button_DR);
    tButton8->sethl_Color(Button_LT);
    tButton8->set_wHint(50);
    tButton8->setFace(draw_Flag3d_Icon);
    tButton8->setAction(action_3d);
    tButton8->forceToggle();
  }
  
  Main->Pack(swMain);
  Main->Pack(swMenuBar);
  Main->Pack(sBar1);
  swMain->Pack(swMiniMap);
  swMenuBar->Pack(Button1);
  swMenuBar->Pack(0);
  swMenuBar->Pack(Button2);
  swMenuBar->Pack(Button3);
  swMenuBar->Pack(Button4);
  swMenuBar->Pack(0);
  swMenuBar->Pack(Button5);
  swMenuBar->Pack(tButton7);
  if(enable_3d)
    swMenuBar->Pack(tButton8);

  action_Revert();

  glApplication.Start();

  return 0;
}

#ifdef WIN32
bool WIN32_GetFname(char *name,const int namesize)
{
	char Filter[]="Powergrid File (*.conf)\0*.conf\0",
		 Extension[]="conf";
	OPENFILENAME ofn;

	ofn.lStructSize=sizeof(OPENFILENAME);
	ofn.hwndOwner=0;
	ofn.lpstrFilter=Filter;
	ofn.lpstrCustomFilter=0;
	ofn.nFilterIndex=1;
	ofn.lpstrFile=name;
	name[0]='\0';
	ofn.nMaxFile=namesize;
	ofn.lpstrFileTitle=0;
	ofn.lpstrInitialDir=0;
	ofn.lpstrTitle=0;
	ofn.Flags=OFN_EXPLORER|OFN_FILEMUSTEXIST|
             OFN_NOCHANGEDIR| // Maybe we don't want this
             OFN_READONLY|OFN_HIDEREADONLY;
	ofn.lpstrDefExt=Extension;

	return GetOpenFileName(&ofn)!=0;
}
#endif
