#include <GL/glut.h>
#include <stdlib.h>
#include <math.h>
#include <sys/time.h>
#include <unistd.h>
#include <iostream.h>
#include <time.h>
#include <vector>
#include <fstream.h>
#include <string>
#include <stdio.h>
#include <GL/osmesa.h>
#include <png.h>
#include "../cgi-bin/image.h"
#include "config.h"
#include "display.h"

static int width = 640; // screen width and height
static int height = 480;
//static int width = 320; // screen width and height
//static int height = 240;

static void init(void)
{
	/* set clearing (background) color */
	glClearColor(0.0, 0.0, 0.0, 0.0);
	glShadeModel(GL_FLAT);
}

static int write_fd_png( int fdout, unsigned char * buffer,
	int w, int h)
{
	FILE *file = fdopen( fdout, "w");
	if( !file )
	{
		perror("file open");
		return -1;
	}

	png_struct *png_ptr = png_create_write_struct(PNG_LIBPNG_VER_STRING,
		(png_voidp)NULL, (png_error_ptr)NULL, (png_error_ptr)NULL);
	if(!png_ptr)
	{
		fclose(file);
		return -1;
	}

	/* if something goes wrong */
	if(setjmp(png_ptr->jmpbuf))
	{
		printf("png error\n");
		return -1;
	}
	
	png_info *info_ptr = png_create_info_struct(png_ptr);
	if( !info_ptr)
	{
		png_destroy_write_struct(&png_ptr,(png_infopp)NULL);
		fclose(file);
		return -1;
	}
	if(setjmp(png_ptr->jmpbuf))
	{
		png_destroy_write_struct(&png_ptr,(png_infopp)NULL);
		fclose(file);
		return -1;
	}
	png_init_io(png_ptr, file);
	png_set_IHDR(png_ptr, info_ptr, w, h, 8, PNG_COLOR_TYPE_RGB_ALPHA,
		PNG_INTERLACE_NONE, PNG_COMPRESSION_TYPE_DEFAULT,
		PNG_FILTER_TYPE_DEFAULT);
	png_write_info(png_ptr, info_ptr);
	/* want to be able to give the png routines an array of lines
	 * so convert make an array and fill it with pointers to
	 * the beginning of each line.  We would have to do this anyway
	 * since line 0 is the top of the screen for png and the bottom
	 * for OpenGL.
	 */
	unsigned char **row_pointers = new unsigned char*[h];
	for( int i=0, offset=w*(h-1)*sizeof(OSMESA_RGBA);
		i<h; i++,offset-=w*sizeof(OSMESA_RGBA))
		row_pointers[i]=buffer+offset;
	
	/* write the png image */
	png_write_image(png_ptr, row_pointers);

	png_write_end(png_ptr, NULL);
	png_destroy_write_struct(&png_ptr, &info_ptr);

	fclose(file);
	return 0;
}

/* unpack and update the current flow info */
//#define CGI_DEBUG 1
#ifdef CGI_DEBUG
	ofstream debug("/tmp/dfries/debug");
#endif /* CGI_DEBUG */

void process_cookie(webInfo *cookie)
{
#ifdef CGI_DEBUG
	debug << "time (seconds) " << time(NULL) << endl;
	debug << "data " << cookie->data << endl;
	debug << "elements " << cookie->numElems << ", data ";
	for( int i = 0; i < cookie->numElems; i++)
		debug << cookie->data[i] << ',';
	debug << endl;
	debug << "x y " << cookie->x << ' ' << cookie->y << endl;
	debug << "action " << cookie->action << endl;
	debug << "deselectall " << cookie->deselectall << endl;
#endif /* CGI_DEBUG */
	/* don't process the cookie info if we want to reset everything */
	if( !cookie->reset )
	{
		if( cookie->data )
			read_config_info( cookie->numElems, cookie->data);
		// zoom based on 1 to 10 with 0 being no zoom
		if( cookie->zoomLevel >0)
			zoom_window(0, 0, 0, -(cookie->zoomLevel-1)*20);
		if( cookie->x!=-1 && cookie->y!= -1)
		{
			if( cookie->action==toggle)
			{
				disable(cookie->x, cookie->y);
			}
			else
			if( cookie->action==moveCenter )
			{
				pan_window( cookie->x, cookie->y, width/2,
					height/2);
			}
		}
		if( cookie->deselectall )
			disable_all_lines();
		
		if( cookie->runBackEnd )
			networked_backend();
	}
	
	write_config_info( &(cookie->numElems), &(cookie->data));
#ifdef CGI_DEBUG
	debug << "write_config_info completed\n";
#endif /* CGI_DEBUG */

}

int main( int argc, char** argv)
{
	/* handle cookie and web form submission information */
	webInfo cookie;
	cookie.reset = cookie.deselectall = false;
	cookie.data = 0;
	cookie.numElems=0;

#define COOKIES 1
#ifdef COOKIES
	readInfoFromPage(cookie);
#endif

	char * infilename = "/tmp/r118org_g.conf";
	//char * infilename = "/home/david/school/cs397/sem2/39xpower/dfries/config/r118org_g.conf";
	
	/* setup the OpenGL drawing canvas */
	unsigned char *buffer = new unsigned
		char[width*height*sizeof(OSMESA_RGBA)];
	if( (int)buffer%4 )
	{
		cerr << "Error buffer is not a multiple of 4\n";
		return -1;
	}
	OSMesaContext context = OSMesaCreateContext( OSMESA_RGBA, NULL);
	if(!context)
	{
		cerr << "Error creating context\n";
		return -1;
	}
	if(!OSMesaMakeCurrent(context,buffer,GL_UNSIGNED_BYTE,width,height))
	{
		cerr << "Error making MesaContext current\n";
		return -1;
	}
	init();
	reshape(width,height);
	glClear(GL_COLOR_BUFFER_BIT);

	/* display OpenGL image */
	if( config_info.ReadFile(infilename)) // read config file
	{
		cerr << "Error reading config file aborting\n";
		return -1;
	}

#ifdef COOKIES
	/* process the user fields to modify what we are displaying */
	process_cookie(&cookie);

	/* render the view */
	render_scene(GL_RENDER);

#ifdef CGI_DEBUG
	debug << "rendered scene\n";
#endif /* CGI_DEBUG */

	/* send any cookies back to web browser and display cgi header */
	displayHeader(cookie,"png");
#endif

#ifdef CGI_DEBUG
	debug << "displaying png\n";
#endif /* CGI_DEBUG */


	/* write out the image */
	// fdopen to take an open filedescriptor and return a FILE *
#ifndef COOKIES
	cout << "Content-Type: image/png\n\n";
	cout << flush;
#endif
	if(write_fd_png(1, buffer, width, height)==-1)
	{
		printf("Error writing png image\n");
		return -1;
	}
#ifdef CGI_DEBUG
	debug << "png image written\n";
#endif /* CGI_DEBUG */
	//cerr << "Success wiring png image " << endl;

	/* clean up */
	OSMesaDestroyContext( context);
	return 0;
}

