#include <iostream>
#include <fstream>
#include <string>
#include "config.h"
#include "../network/iclient.h"

char network_address[512]="rocket.cc.umr.edu";
int network_port=3210;

int networked_backend()
{
	const char * configdata = config_info.WriteData();
	long length = strlen(configdata);
	char * rcvdata = 0;
	int result = client( configdata, length, &rcvdata,
		network_address, network_port);
	if( result == -1 || !result)
	{
		cout << "Error running back end\n";
		return -1;
	}
	else
	{
		//cout << "file read \n" << rcvdata;
		istrstream indata(rcvdata);
		config_info.UpdateFlow( indata);
	}
	return 0;
}


powernode_s::powernode_s()
{
	text=0;
	x=-1;
	y=-1;
}

powernode_s::~powernode_s()
{
	if(text)
		delete[]text;
}

/* copy constructor */
powernode_s::powernode_s( const powernode_s& pnode)
{
	if( pnode.text )
	{
		text = new char [1+(text_length=pnode.text_length)];
		strcpy(text,pnode.text);
	}
	else
	{
		text=0;
		text_length=0;
	}
	x=pnode.x;
	y=pnode.y;
}

/* assignment operator */
powernode_s& powernode_s::operator=( const powernode_s& pnode)
{
	if(this != &pnode)
	{
		// call the delete operator instead of dealing with 
		// cleanup here
		delete this;
		powernode_s(pnode);
	}
	return *this;
}

config_c::config_c()
{
	outputdata=0;
}

config_c::~config_c()
{
	if(outputdata)
	{
		// unfreeze so it should delete it's contents
		outputdata->freeze(0);
		delete outputdata;
	}
}

int config_c::UpdateFlow( istream & data)
{
	char type;
	string tmpstring;
	int parse_error=0;
	int maxflow = 0;
	int from;
	int to;
	int flow;
	vector<powerline_s>::iterator p1;
	p1 = powerline_v.begin();

	while(1)
	{
		// start back to the front if we reach the end
		if(p1 == powerline_v.end() )
			p1 = powerline_v.begin();
		data >> type;
		if( data.eof() )
			break;
		switch(type)
		{
		case 'c':
		//Commented line
			data.ignore(1073741824, '\n');
			break;
		case 's':
		//Max flow value line
			data >> maxflow;
			break;
		case 'f':
		//Update flow on edges
			data >> from >> to >> flow;
			if(p1->node[0] == from && p1->node[1] == to)
			{
				p1->flow = flow;
				p1++;
			}	
			else
			{
				p1 = powerline_v.begin();
				while(p1->node[0] != from || p1->node[1] != to)
				{
					p1++;
					/* end is "Returns an iterator pointing
					 * one past the last element in the
					 * container.
					 */
					if(p1 == powerline_v.end() )
						break;
				}
		
				if(p1!= powerline_v.end() &&
				p1->node[0] == from && p1->node[1] == to)
				{
					p1->flow = flow;
		//			p1->max = maxflow;
				}
				else
				{
					cerr << "Line from " << from <<
					" to " << to << " not found\n";
				}

			}
			break;
		default:
			cerr << "Unknown line while updating edges: " << type << "\n";
			data.ignore(1073741824, '\n');
			break;
		}
	}
	return parse_error;
}

int config_c::ReadFile( const char *filename)
{
	ifstream filein(filename);
	if(!filein)
	{
		return -1;
	}
	char type;
	string tmpstring;
	int parse_error=0;
	int tmp;
	powerline_v.empty();
	char s_t;
	int lastnode=0;
	powernode_s pnode;
        background_s bgnode;
	char buf[80];

	while(1)
	{
		filein >> type;
		if( filein.eof())
			break;
		switch(type)
		{
		case 'c':
		// found a comment
			// skip to next line, should find out how big the file
			// is for this line though
			filein.ignore( 1073741824, '\n');
			break;
                case 'b':
                //Add a background rectangle(image)
                        filein >> bgnode.ul_x >> bgnode.ul_y
                               >> bgnode.lr_x >> bgnode.lr_y;
                        filein.getline(buf,80,'\n');
                        // Cut off trailing whitespace
                        for(tmp=strlen(buf)-1;buf[tmp]==' ';tmp--);
                        buf[++tmp]='\0';
                        // Skip preceeding whitespace
                        for(tmp=0;buf[tmp]==' ';tmp++);
			strcpy(bgnode.name,&buf[tmp]);
			bgnode.loadedin[0]=bgnode.loadedin[1]=
			bgnode.loadedin[2]=bgnode.loadedin[3]=
			bgnode.loadedin[4]=0;
			background_v.push_back(bgnode);
                        break;
		case 'e':
		// Read in extent (size) of working area
		// Same coordinates as geometric location
			filein >> extent_left >> extent_right
			       >> extent_top >> extent_bottom;
			break;
		case 'g':
		// found a geometric x y location info
			filein >> tmp;
			if( tmp!= lastnode+1)
			{
				cerr << "Geometric information must be "
					"consecutive\n";
				parse_error++;
			}
			lastnode=tmp;
			filein >> pnode.x >> pnode.y;
			while(filein.peek() == ' ' || filein.peek() == '\t' )
				filein.ignore();
			filein.get(buf,sizeof(buf));
			pnode.text_length = strlen(buf);
			pnode.text=new char[pnode.text_length+1];
			strcpy(pnode.text,buf);
			powernode_v.push_back(pnode);
			break;
		case 'p':
		// found a max power line
			filein >> tmpstring;
			if(tmpstring!="max")
			{
				cerr << "error in \"p max\"\n";
				filein.ignore( 1073741824, '\n');
				parse_error++;
				continue;
			}
			tmpstring.empty();
			filein >> tmp;
			filein >> sink_source.power_max;
			break;
		case 'n':
		// found a sink/source line
			s_t=0;
			filein >> tmp;
			filein >> s_t;
			switch( s_t)
			{
			case 's':
				sink_source.node[source] = tmp;
				break;
			case 't':
				sink_source.node[sink] = tmp;
				break;
			default:
				cerr << "Unknown sink/source type\n";
				parse_error++;
			}
			break;
		case 'a':
		// found a power line
			powerline_s pl;
			filein >> pl.node[0] >> pl.node[1] >> pl.max;
			/* init the other fields */
			pl.flow=0;
			pl.type=(powerline_enum)0;
			powerline_v.push_back(pl);
			break;
		default:
			cerr << "Unknown config option: \"" << type << "\"\n";
			filein.ignore(1073741824, '\n');
		}
	}
	sink_source_process();
	return parse_error;
}

const char * config_c::WriteData()
{
	if(outputdata)
	{
		// unfreeze so it should delete it's contents
		outputdata->freeze(0);
		delete outputdata;
	}
	outputdata=new ostrstream();
	ostrstream &output=*outputdata;
	/* power_max looks like the number of power lines present,
	 * if that is the case it will have to be calculated.
	 */
	output << "p max " << sink_source.node[sink]
		<< '\t' << sink_source.power_max << endl;
	output << "n " << sink_source.node[source] << "\ts\n";
	output << "n " << sink_source.node[sink] << "\tt\n";
	for( int stop=powerline_v.size(),i=0; i<stop; i++)
	{
		if( powerline_v[i].type == disabled )
		{
			output << "c";
		}
		output << "a\t" << powerline_v[i].node[0]
			<< '\t' << powerline_v[i].node[1]
			<< '\t' << powerline_v[i].max << endl;
	}
	output << (char) 0;
	output.freeze(1);
	return output.str();
}

ostream & operator << (ostream &output, const powerline_enum &e)
{
	switch (e)
	{
	case functional_power_line:
		output << "a functional power line";
		break;
	case disabled:
		output << "Disabled by user";
		break;
	case power_sink:
		output << "Connects to global power_sink";
		break;
	case power_source:
		output << "Connects to global power_source";
	}
	return output;
}

ostream & operator << (ostream &output, const sink_source_enum &sse)
{
	switch(sse)
	{
	case source:
		output << "source";
		break;
	case sink:
		output << "sink";
	}
	return output;
}

ostream & operator << (ostream &output, const sink_source_s &ss)
{
	for( int i=0; i < 2; i++)
		output << "node " << ss.node[i] << " is a " << 
			(sink_source_enum)i << endl;
	output << "There are " << ss.power_max << " power lines total\n";
	return output;
}

ostream & operator << (ostream &output, const powernode_s &pn)
{
	output << "node at (" << pn.x << ',' << pn.y << ") message length "
		<< pn.text_length << ", \"" << pn.text << "\"\n";
	return output;
}

ostream & operator << (ostream &output, const powerline_s &pl)
{
	output << "power line from " << pl.node[0] << " to " << pl.node[1]
		<< " with max power as " << pl.max << ", current flow "
		<< pl.flow << " and " << pl.type << endl;
	return output;
}

ostream & operator << (ostream &output, config_c &c)
{
	int i;
	int stop;
	for( i=0, stop=c.powerline_v.size(); i < stop; i++)
		output << c.powerline_v[i];
	for( i=0, stop=c.powernode_v.size(); i < stop; i++)
		output << c.powernode_v[i];
	output << c.sink_source;

	return output;
}


void config_c::sink_source_process()
{
	/* set each power line to sink_source if either its from node
	 * or two node is connected to the source or sink node
	 */
	for( int stop=powerline_v.size(), i=0; i<stop; i++)
	for( int fromto=0; fromto<2; fromto++)
	{
		if(powerline_v[i].node[fromto]==sink_source.node[sink])
		{
			powerline_v[i].type=power_sink;
			break;
		}
		else
		if(powerline_v[i].node[fromto]==sink_source.node[source])
		{
			powerline_v[i].type=power_source;
			break;
		}
	}
}

config_c config_info;
