#ifndef _CONFIG_H
#define _CONFIG_H 1
#include <strstream>
#include <vector>
#include "glTexture.h"

using namespace std;

/* run the backend by the network and return -1 on failure, 0 if successful */
extern char network_address[512];
extern int network_port;;
int networked_backend();

// Defined if we will be using the sequential maxflow program to run
// // the backend.
#define SEQUENTIAL_MAXFLOW 1

enum powerline_enum {
	functional_power_line, // default power state
	disabled,	// the line was disabled by the user
	power_sink,
	power_source};
		// not an actual power line, but rather used by
			// the power annalysis software

enum sink_source_enum
{
	source,	// "s" node number were the power is generated
	sink};	// "t" node number of the "sink" where all the power goes

struct sink_source_s
{
	int node[2]; // enum sink_source_enum specifies which is 'source' and
		// which is the 'sink'
	int power_max;
		// value "p max sink_node x" x in that line is the number of
		// power lines being listed
};

struct powernode_s
{
	int x;		// geometric X location
	int y;		// geometric Y location
	char *text;	// Text identifier of this node
	int text_length;	// length of the text given excluding the null

	powernode_s();	// constructor
	~powernode_s();	// destructor
	powernode_s( const powernode_s& pnode); // copy constructor
	powernode_s& operator=( const powernode_s& pnode);
		// assignment operator
};

struct powerline_s
{
	int node[2];	// from node[0] to node[1]
	int max;	// max load
	int flow;	// current flow
	powerline_enum type;	// other properties
};

ostream & operator << (ostream &output, const powerline_enum &e);
ostream & operator << (ostream &output, const sink_source_enum &sse);
ostream & operator << (ostream &output, const sink_source_s &ss);
ostream & operator << (ostream &output, const powernode_s &pn);
ostream & operator << (ostream &output, const powerline_s &pl);

// All the info for background textures
struct background_s
{
   glTexture t[5];
   int loadedin[5];
   char name[80];
   // Upper left corner
   float ul_x;
   float ul_y;
   // Lower right corner
   float lr_x;
   float lr_y;
};

class config_c
{
public:
	vector <powerline_s> powerline_v;
	vector <powernode_s> powernode_v;
        vector <background_s> background_v;

	float extent_top;
	float extent_bottom;
	float extent_left;
	float extent_right;

	/* Reads filename into the power line vector after emptying the
	 * vector.  Returns 0 on success, -1 on file failure, and
	 * a positive number indicating the number of parase errors found.
	 */
	int ReadFile( const char *filename);
	/* Returns a pointer to the config file generated from the data
	 * at the time WriteData was called.  The string is overwritten
	 * each time this function is called.
	 */
	const char * WriteData();
	/* write all data values to the output stream including internal
	 * values
	 */
	int UpdateFlow(istream & data);
	/* read in the new flow data from the file generated by the
	 * back end
	 */

	friend ostream &operator << (ostream &output, config_c &c);
	config_c();
	~config_c();

	sink_source_s sink_source;
private:
	/* After the data has been read in disable the logical lines
	 * that aren't really there.
	 */
	void sink_source_process();
	ostrstream *outputdata;
};

extern config_c config_info;

#endif // _CONFIG_H
