#include <GL/glut.h>
#include <stdlib.h>
#include <math.h>
#ifndef WIN32
#include <sys/time.h>
#include <unistd.h>
#endif
#include <iostream>
#include <time.h>
#include <vector>
#include <fstream>
#include <stdio.h>
#include <string.h>
#include "config.h"
#include "display.h"

using namespace std;

static char * generatedfile;
static char * updatedfile;

static void init(void)
{
	/* set clearing (background) color */
	glClearColor(0.0, 0.0, 0.0, 0.0);
	glClear(GL_COLOR_BUFFER_BIT);
	glShadeModel(GL_FLAT);
}

static int animate=1;

void timedRedrawFunc(int discard)
{
	if(animate)
	{
		glutTimerFunc(150,timedRedrawFunc,0);
		glutPostRedisplay();
	}
}

static void file_ops( unsigned char key, int x, int y)
{
	ofstream *output=0;
	switch(key)
	{
	case 'q':
		exit(0);
		break;
	case 'd':
		// dump to file
		output = new ofstream(generatedfile,ios::in|ios::trunc);
		*output << config_info.WriteData();
		output->close();
		cout << "dump finished\n";
		cout << flush;
		break;
	case 'r':
		// dump to file
		file_ops( 'd', 0,0);
		// run maxflow
	// if on afs? hack yes, but it is going away anyway
	#ifdef _BIG_ENDIAN
		cout << "return value " << system( 
		//"/afs/umr.edu/users/dfries/cs39x/39xpower/backend/maxFlow/GOLD"
		"/tmp/maxFlow/GOLD"
		" -in /afs/umr.edu/users/dfries/dump "
		"-out /afs/umr.edu/users/dfries/load -rep s -quiet");
	#else
		cout << "return value " << system( 
		"/home/david/school/cs397/sem2/39xpower/backend/maxFlow/GOLD"
		" -in /tmp/dump "
		"-out /tmp/load -rep s -quiet");
	#endif
		// load from file
		file_ops( 'u', 0,0);
		break;
	case 'n':
		// use the network daemon to run the backend
		networked_backend();
                break;
		
	case 'u':
		// load from file
		cout << "Beginning load\n";
		{
			ifstream filein(updatedfile);
			if(!filein)
			{
				cerr << "Error reading file\n";
			}
			else
			{
				config_info.UpdateFlow(filein);
				filein.close();
			}
		}
		cout << "load finished\n";
		cout << flush;
		glutPostRedisplay();
		break;
	case 's':
		// dump status
		cout << "Here is the requested status\n";
		cout << config_info;
		break;
	case 'a':
		animate = (animate+1)%2;
		glutTimerFunc(150,timedRedrawFunc,0);
		break;
	}
	if(output)
		delete output;
}

void ShowUsage(int errindex,char **argv)
{
	if(errindex>=0)
		cerr << "Invalid argument: " << argv[errindex]
		     << endl << endl;
	cerr << "Usage: " << argv[0] <<
		" configfile generatedfile updatedfile "
		"-single\n";
	cerr << "keyboard, 'd' dump to generatedfile\n";
	cerr << "'u' load from updatedfile\n";
	cerr << "'s' status\n";
	cerr << "-single to not make use of double buffering\n";
}

int main( int argc, char** argv)
{
	int i;
	bool single=false;

	glutInit( &argc, argv);
	if(argc<4)
//( argc !=4 && argc != 5 || ( argc == 5 && strcmp("-single", argv[4])))
	{
                ShowUsage(-1,argv);
		return -1;
	}
	generatedfile=argv[2];
	updatedfile=argv[3];
	if( config_info.ReadFile(argv[1]))
	{
		cerr << "Error reading config file aborting\n";
		return -1;
	}
        
        for(i=4;i<argc;i++)
           if(argv[i][0]!='-')
           {
              ShowUsage(i,argv);
              return -1;
           }
           else
              switch(argv[i][1])
              {
                 case 'n':
                    if(!strcmp(argv[i],"-nobg"))
                       nodrawbackground=true;
                    else
                    {
                       ShowUsage(i,argv);
                       return -1;
                    }
                    break;
                 case 's':
                    if(!strcmp(argv[i],"-single"))
                       single=true;
                    else
                    {
                       ShowUsage(i,argv);
                       return -1;
                    }
                    break;
                 default:
                    ShowUsage(i,argv);
                    return -1;
              }

	/* already check above that argument four is -single */

	if(single)
		glutInitDisplayMode( GLUT_SINGLE | GLUT_RGB);
	else
		glutInitDisplayMode( GLUT_DOUBLE | GLUT_RGB);
	glutInitWindowSize( 250, 250);
	glutCreateWindow("Power Grid simulation");
	init(); // once sent OpenGL commands
	// will be called when the window needs to be displayed
	glutDisplayFunc(display);
	glutReshapeFunc(reshape);
	glutMouseFunc(mouse);
	glutMotionFunc(mouse_motion);
	glutKeyboardFunc(file_ops);
	glutTimerFunc(100,timedRedrawFunc,0);
	glutMainLoop();
	return 0;
}
