#include <GL/glut.h>
#include <stdlib.h>
#include <math.h>
#ifndef WIN32
#include <sys/time.h>
#include <unistd.h>
#endif
#include <iostream>
#include <time.h>
#include <vector>
#include <fstream>
#include <stdio.h>
#include "config.h"
#include "display.h"
#include "drawstuff.h"

bool flag3d=false;

using namespace std;
int infomode = 0;
void (*print_string)(char *)=0;
char final_string[512]="";

// help to debug selections
//#define DEBUG_SELECT 1
static float scaler = .2;
static int width = 0; // screen width and height
static int height = 0;
static float offsetx = 30;
static float offsety = -30;
static int mouse1_pressed = 0;
static int mouse3_pressed = 0;
static int mouse_lastx;
static int mouse_lasty;
static int mouse_moved=0;

bool nodrawbackground=false;
void DrawBackground();

/* data is from a cookie, and there are num of them,
 * data 0 though num-3 are the pairs (flow rate, 0 disabled / 1 enabled)
 * for the power lines
 * the last two are the offsetx and offsety
 */
int read_config_info( int num, long * data)
{
	int stop = config_info.powerline_v.size();
	for(int i = 0, line=0; i < num-2 && line<stop; i+=2,line++)
	{
		config_info.powerline_v[line].flow = data[i];
		config_info.powerline_v[line].type = (powerline_enum)data[i+1];
		if( (powerline_enum)data[i+1] == disabled)
			config_info.sink_source.power_max--;
	}
	if( num >= 2)
	{
		/* want to preserve some of the decimal places */
		offsetx = (float)data[num-2]/1000;
		offsety = (float)data[num-1]/1000;
	}
	return 0;
}

/* write the data back to a form for the cookie,
 * data zero though num-3 are the flow rates for the power lines
 * followed by 1 for enabled, 0 for disabled
 * the last two are the offsetx and offsety
 * if *data is null it is deleted and replaced
 */
int write_config_info( int *num, long **data)
{
	if( !(*data ))
		delete [] (*data);
	*num=2*config_info.powerline_v.size()+2;
	(*data) = new long[*num];
	int stop = config_info.powerline_v.size();
	for(int i = 0, line=0; i < *num-2 && line < stop; i+=2,line++)
	{
		(*data)[i] = config_info.powerline_v[line].flow;
		(*data)[i+1] = (long)config_info.powerline_v[line].type;
	}
	/* want to preserve some of the decimal places */
	(*data)[*num-2] = (long)(offsetx*1000);
	(*data)[*num-1] = (long)(offsety*1000);
	return 0;
}

static void printstring( void *font, char *string, int length)
{
	int i;
	for( i =0; i < length; i++)
		glutBitmapCharacter(font, string[i]);
}

/* draw the graphic for a sink or a source object at the location (x,y)
 * it is also given how much power is going to or from the sink
 * and the capacity of the lines given by current_flow and max_capacity
 */
static void draw_sink_source( powerline_enum type, float x, float y,
	int current_flow, int max_capacity)
{
	glPushMatrix();
	glLoadIdentity();
	glTranslatef(x,y,0);
	glBegin( GL_POLYGON );
	if (type == power_sink )
	{
		if(current_flow)
		/* Good power*/
			glColor3f( 0,.6,0);
		else
		if(current_flow!=max_capacity)
		/* brown out */
			glColor3f(.545098, .270588,.074509);
		else
		/* black out */
			glColor3f( .5,0,0);
	}
	else
	// power_source
	{
		glColor3f(0,0,.8);
	}
	/* draw a octagon */
	const float size=0.85;
	const float size2=size/2;
	const float dx=size*.9238795325;
	const float dy=size*.3826834324;
	glVertex3f( -size2,-dy-size2,0);
	if( type == power_sink )
	{
		glVertex3f( size-size2,-dy-size2,0);
		glVertex3f( size+dx-size2,dy-dy-size2,0);
		glVertex3f( size+dx-size2,size+dy-dy-size2,0);
		glVertex3f( size-size2,2*dy+size-dy-size2,0);
	}
	else
	{
		glVertex3f( size-size2,2*dy+size-dy-size2,0);
		glVertex3f( -size2,2*dy+size-dy-size2,0);
		glVertex3f( -dx-size2,dy+size-dy-size2,0);
		glVertex3f( -dx-size2,dy-dy-size2,0);
	}
	glEnd();
	glPopMatrix();
}

/* write the string that coresponds to each node */
static void label_nodes()
{
	int stop = config_info.powernode_v.size();
	glColor3f( 1.0, 1.0, 0.0);
	for( int i=0; i < stop; i++)
	{
		powernode_s *pn = &config_info.powernode_v[i];
		// don't display when they are both == to zero
		if( pn->x || pn->y)
		if( pn->text_length )
		{
			glRasterPos2f( pn->x, -pn->y);
			printstring( GLUT_BITMAP_HELVETICA_12,
				pn->text, pn->text_length);
		}
	}
}

static int sequence=0; /* a sequencer for animations,
		updated each time the display is drawn */
static int seq_max=4; // how many states are in the sequnecer, 0 to (seq_max-1)

/* draw a line that represents a power grid line given the geometry and
 * current power characteristics of the line
 * the sign of ratio is the direction of current flow
 */
static void draw_power_line( float from_x, float from_y, float to_x, float to_y,
	float max, float flow, float ratio, powerline_enum type)
{
	unsigned short series_positive[4]={ 0xeeee,0x7777,0xbbbb,0xdddd};
	unsigned short series_negative[4]={ 0xdddd, 0xbbbb, 0x7777, 0xeeee};
	if( ratio<0 )
	{
		glLineStipple(3,series_negative[sequence]);
		ratio*=-1;
	}
	else
	{
		glLineStipple(3,series_positive[sequence]);
	}
	glBegin( GL_LINES );
	if( type == disabled )
		glColor3f( 0,0,1);
	else
	{
		if( ratio > .90)
			glColor3f( 0, (float)(ratio-.90)*10,0);
		else
			glColor3f( (float)(1-ratio)*1.1111111111,0,0);
	}
	glVertex3f( from_x, from_y, 0);
	glVertex3f( to_x, to_y, 0);
	glEnd();
}

void render_scene(GLenum mode,bool showlabels)
{
	/* clear all pixels */
	int stop = config_info.powerline_v.size();
	powerline_s *pl;
	powerline_s *pl_next;
	float ratio;
	float from_x, to_x, from_y, to_y;
	/* only draw power sinks or sources if we aren't in select mode
	 * draw these first so they are under everything else
	 */
	
	if(mode!=GL_SELECT)
	{
                DrawBackground();

		const int sink_node = config_info.sink_source.node[sink];
		const int source_node = config_info.sink_source.node[source];
	for( int i=0; i<stop; i++)
	{
		pl=&config_info.powerline_v[i];
		if(pl->type == power_sink || pl->type == power_source )
		{
		if(pl->node[0] == sink_node || pl->node[0] == source_node)
		{
			// node[1] is the real one
			from_x = config_info.powernode_v[pl->node[1]-1].x;
			from_y = -config_info.powernode_v[pl->node[1]-1].y;
		}
		else
		{
			// node[0] is the real one
			from_x = config_info.powernode_v[pl->node[0]-1].x;
			from_y = -config_info.powernode_v[pl->node[0]-1].y;
		}
      if((flag3d) && (showlabels))
           drawNode((int)from_x,(int)from_y,pl->type);
        else
           draw_sink_source(pl->type, from_x, from_y,pl->flow,pl->max);
		}
	}
	sequence=(sequence+1)%seq_max;
	if(showlabels)
		glEnable(GL_LINE_STIPPLE);
	}
	for( int i = 0; i < stop; i++)
	{
		pl=&config_info.powerline_v[i];
		// don't display sink or source lines
		if(pl->type == power_sink || pl->type == power_source)
		{
			/* power source/sink doesn't get selected */
			continue;
		}
		if(mode==GL_SELECT)
		{
			glLoadName(i);
		#ifdef SEQUENTIAL_MAXFLOW
			// we will draw one line, but we need to unselect
			// both lines to write to the backend
			glPushName(i+1);
		#endif
		}
		if( pl->max )
		{
		#ifdef SEQUENTIAL_MAXFLOW
			pl_next = &config_info.powerline_v[i+1];
			// sanity check, the next line should go in the other
			// direction
			if( pl_next->node[1] != pl->node[0] &&
				pl_next->node[0] != pl->node[1] )
			{
			cerr << "bad config file, lines aren't opposite, ["
				<< i << "] " << pl->node[0] << ','
				<< pl->node[1]
				<< endl;
				ratio=(float)pl->flow/pl->max;
			}
			else
			{
				i++; // skip over the next line
				ratio=(float)(pl_next->flow - pl->flow)
					/pl_next->max;
			}
		#else
			ratio=(float)pl->flow/pl->max;
		#endif
		}
		else
		{
			ratio = 1;
		}
		from_x = config_info.powernode_v[pl->node[0]-1].x;
		from_y = -config_info.powernode_v[pl->node[0]-1].y;
		to_x = config_info.powernode_v[pl->node[1]-1].x;
		to_y = -config_info.powernode_v[pl->node[1]-1].y;
        if((flag3d) && (showlabels))
           drawLink((int)from_x, (int)from_y, 
					      (int)to_x, (int)to_y, pl->flow, pl->max);
        else
           draw_power_line(from_x, from_y, to_x, to_y, pl->max, pl->flow,
                           ratio, pl->type);

		#ifdef SEQUENTIAL_MAXFLOW
		if(mode==GL_SELECT)
		{
			// we will draw one line, but we need to unselect
			// both lines to write to the backend
			glPopName();
		}
		#endif
	}
	if(mode!=GL_SELECT)
	{
		if(showlabels)
		{
		   label_nodes();
			glDisable(GL_LINE_STIPPLE);
		}
	}
}

// Hack to stop pre-windowshown activation
int threed_count=0;
void display()
{
#ifdef PHILCLIENT
if(threed_count==0)
   InitGL();
if(threed_count++==1)
   LoadGLTextures();

         glPushMatrix();
         reshape(width,height);
         render_scene(GL_RENDER);
         glPopMatrix();
#else
	glClear(GL_COLOR_BUFFER_BIT);
	render_scene(GL_RENDER);

	glFlush();
	glutSwapBuffers();
#endif
}

/* Executed if the window is resized or moved etc. */
void reshape(int w, int h)
{
	width=w;
	height=h;
	//cout << "w " << w << ", h " << h << endl << flush;
	glViewport(0, 0, (GLsizei) w, (GLsizei) h);
	glMatrixMode(GL_PROJECTION);
	glLoadIdentity();
/*	
	cerr << '(' << (float)offsetx << ',' << (float)offsetx+width*scaler
		<< ',' << (float)offsety+height*scaler << ',' << (float)offsety
		<< ")\n" << flush;
*/		
	/* scale from the center */
	float w2 = width*scaler/2;
	float y2 = height*scaler/2;
	glOrtho(offsetx-w2,offsetx+w2,
		offsety+y2,offsety-y2,-1,1);
	glMatrixMode(GL_MODELVIEW);
	glLoadIdentity();
	glFinish();
}

void enable_all_lines()
{
	int stop = config_info.powerline_v.size();
	for( int i = 0; i < stop; i++)
		if( config_info.powerline_v[i].type == disabled )
		{
			config_info.powerline_v[i].type = functional_power_line;
			config_info.sink_source.power_max++;
		}
}

void disable_all_lines()
{
	int stop = config_info.powerline_v.size();
	for( int i = 0; i < stop; i++)
      if ( config_info.powerline_v[i].type == functional_power_line )
		{
			config_info.powerline_v[i].type = disabled;
			config_info.sink_source.power_max--;
		}
}

void recalc_disabled()
{
        int stop = config_info.powerline_v.size();
        for( int i = 0; i < stop; i++)
                if ( config_info.powerline_v[i].type == disabled )
						config_info.powerline_v[i].flow = 0;
}

void disable_all()
{
        int stop = config_info.powerline_v.size();
        for( int i = 0; i < stop; i++)
                if ( config_info.powerline_v[i].type == functional_power_line )
			config_info.powerline_v[i].type = disabled;
}

void disable_lines(GLuint hits, GLuint buffer[])
{
		unsigned int i,j;
	GLuint ii,jj,names, *ptr;

	//printf("hits = %d\n", hits);
	ptr = (GLuint *) buffer;
	for( i = 0; i < hits; i++)
	{
		names=*ptr++;
		//printf(" number of names for hit = %d\n", names);
		//printf(" z1 is %g;", (float)*ptr++/0x7fffffff);
		//printf(" z2 is %g\n", (float)*ptr++/0x7fffffff);
		//printf("   the name(s) is(are) ");
		*ptr++;
		*ptr++;
		for(j=0; j< names; j++)
		{
			//printf("%d\t",*ptr);
			powerline_s &pl = config_info.powerline_v[*ptr];
			/* toggle between enabled and disabled */
			if( pl.type == disabled )
			{
				pl.type=functional_power_line;
				// one more active line
				config_info.sink_source.power_max++;
			}
			else
			{
				pl.type=disabled;
				// one less active line
				config_info.sink_source.power_max--;
			}
			if(j==0)
				ii=*ptr;
			else if (j==1)
				jj=*ptr;
			ptr++;
		}
		//printf("\n");
	}
}

void printinfo(GLuint index1, GLuint index2)
{
	powerline_s *pl, *pl_next;
	char str_type[15];
	float ratio;
	
	pl = &config_info.powerline_v[index1];
	final_string[0] = 0;
	int node_start, node_end;

	//printf("\nPrint info of this line: %u  %u\n", index1, index2);

	#ifdef SEQUENTIAL_MAXFLOW
		pl_next = &config_info.powerline_v[index2];
		// sanity check, the next line should go in the other
		// direction
		if( pl_next->node[1] != pl->node[0] &&
			pl_next->node[0] != pl->node[1] )
		{
			cerr << "printinfo:: bad config file, lines aren't opposite" << endl;
			ratio=(float)pl->flow/pl->max;
		}
		else
		{
			ratio=(float)(pl_next->flow - pl->flow)
				/pl_next->max;
		}
	#else
		ratio=(float)pl->flow/pl->max;
	#endif

	if (ratio < 0) {
		node_start = pl->node[1];
		node_end = pl->node[0];
	}
	else {
		node_end = pl->node[1];
		node_start = pl->node[0];
	}		
	switch (pl->type)
	{
		case functional_power_line:
			strcpy(str_type, "Functional");
			break;
		case disabled:
			strcpy(str_type, "Disabled");
			break;
		case power_sink:
			strcpy(str_type, "Power Sink");
			break;
		case power_source:
			strcpy(str_type, "Power Source");
			break;
		default:
			strcpy(str_type, "Unknown");
	}
	sprintf(final_string, "%s power line from node %i to node %i: Max = %i, Current = %i",
			  str_type, node_start, node_end, pl->max, pl->flow);
	
	if(print_string)
		print_string(final_string);
}


void processHits( GLuint hits, GLuint buffer[])
{
	if (infomode){
		if (hits>0) {
		  printinfo(buffer[3], buffer[4]);
	  }
	}
	else {
		disable_lines(hits, buffer);
	}
	
}

/* disable the line at location x,y (scene coordinates) */
void disable( float x, float y)
{
	const int BUFSIZE = 512;
	GLuint selectBuf[BUFSIZE];
	GLint hits;
	GLint viewport[4];

	glGetIntegerv(GL_VIEWPORT, viewport);

	glPushMatrix();
	glSelectBuffer(BUFSIZE, selectBuf);
	glRenderMode(GL_SELECT);

	glInitNames();
	glPushName(0);

	glMatrixMode(GL_PROJECTION);
	glLoadIdentity();
	gluPickMatrix((GLdouble)x, (GLdouble)(viewport[3]-y), 5.0, 5.0,
		viewport);
	float w2 = width*scaler/2;
	float y2 = height*scaler/2;
	glOrtho(offsetx-w2,offsetx+w2,
		offsety+y2,offsety-y2,-1,1);
	glMatrixMode(GL_MODELVIEW);
	glLoadIdentity();
	render_scene(GL_SELECT);
#ifndef DEBUG_SELECT
	glPopMatrix();
#endif /* DEBUG */
	glFlush();
	hits = glRenderMode(GL_RENDER);
	processHits(hits, selectBuf);
#ifndef PHILCLIENT
	// shouldn't be need, but seems to be, look into sometime
	reshape(width,height);
#endif
}

void pan_window( int oldx, int oldy, int curx, int cury)
{
	offsetx += (float)(oldx - curx)*scaler;
	offsety += (float)(oldy - cury)*scaler;
	reshape(width,height);
}

void zoom_window( int oldx, int oldy, int curx, int cury)
{
	int diff = cury-oldy;
	scaler*=pow(1.01,diff);
	reshape(width,height);
}

void mouse(int button, int state, int x, int y)
{
	// only interested in mouse presses
	mouse_lastx = x;
	mouse_lasty = y;
	if(state==GLUT_DOWN)
	switch (button)
	{
	case GLUT_LEFT_BUTTON:
		mouse_moved=0;
		mouse1_pressed =1;
		break;
	case GLUT_MIDDLE_BUTTON:
		break;
	case GLUT_RIGHT_BUTTON:
		mouse3_pressed =1;
		break;
	default:
		break;
	}
	if(state==GLUT_UP)
	switch (button)
	{
	case GLUT_LEFT_BUTTON:
		if(!mouse_moved)
		{
			disable(x, y);
         glFinish();
			glutPostRedisplay();
		}
		mouse1_pressed =0;
		break;
	case GLUT_MIDDLE_BUTTON:
		break;
	case GLUT_RIGHT_BUTTON:
		mouse3_pressed =0;
		zoom_window(mouse_lastx, mouse_lasty, x, y);
      glFinish();
		glutPostRedisplay();
		break;
	default:
		break;
	}
}

void mouse_motion( int x, int y)
{
	/* Return if we aren't interested in this movement */
	if(!mouse1_pressed&&!mouse3_pressed)
		return;

	// update so we can tell between move and select
	mouse_moved=1;
	if(mouse1_pressed)
	{
		pan_window(mouse_lastx, mouse_lasty, x, y);
		glutPostRedisplay();
	}
	if(mouse3_pressed)
	{
		zoom_window(mouse_lastx, mouse_lasty, x, y);
		glutPostRedisplay();
	}
	mouse_lastx = x;
	mouse_lasty = y;
}

int DB_count=1;
void DrawBackground()
{
   int i,wnd=glutGetWindow();
   vector<background_s>::iterator iter;

   if(nodrawbackground)
//      ||--DB_count>0)
      return;

   glEnable(GL_TEXTURE_2D);
   for(iter=config_info.background_v.begin();
       iter!=config_info.background_v.end();
       iter++                                )
   {
      // Hopefully, there are no more than 5 windows
      for(i=0;i<5&&iter->loadedin[i]!=0;i++)
         if(iter->loadedin[i]==wnd)
            break;

      if(iter->loadedin[i]==0)
      {
         iter->t[i].Load(iter->name);
         iter->loadedin[i]=wnd;
      }
      iter->t[i].Quad(iter->ul_x,-iter->ul_y,0.0f,0.0f,
                      iter->lr_x,-iter->ul_y,1.0f,0.0f,
                      iter->lr_x,-iter->lr_y,1.0f,1.0f,
                      iter->ul_x,-iter->lr_y,0.0f,1.0f );
   }

   glDisable(GL_TEXTURE_2D);
   return;
}

// ******************************************************
// * Minimap stuff
// ******************************************************
bool mm_moving=false;

void mm_display()
{
   int w=glutGet((GLenum)GLUT_WINDOW_WIDTH),
       h=glutGet((GLenum)GLUT_WINDOW_HEIGHT);
   float w2=width*scaler/2,
         h2=height*scaler/2;
   float wsmall=0.5f*(config_info.extent_right-config_info.extent_left)/w,
         hsmall=0.5f*(config_info.extent_bottom-config_info.extent_top)/h;

   mm_reshape(w,h);
   render_scene(GL_RENDER,false);

   // Draw zoom box
   glColor3f(1.0f,1.0f,1.0f);
   glBegin(GL_LINE_LOOP);
      glVertex2f(offsetx-w2,offsety-h2);
      glVertex2f(offsetx+w2,offsety-h2);
      glVertex2f(offsetx+w2,offsety+h2);
      glVertex2f(offsetx-w2,offsety+h2);
   glEnd();

   // Draw minimap border
   glColor3f(0.0f,0.0f,1.0f);
   glBegin(GL_LINE_LOOP);
      glVertex2f(config_info.extent_left+wsmall,-config_info.extent_top-hsmall);
      glVertex2f(config_info.extent_right-wsmall,-config_info.extent_top-hsmall);
      glVertex2f(config_info.extent_right-wsmall,-config_info.extent_bottom+hsmall);
      glVertex2f(config_info.extent_left+wsmall,-config_info.extent_bottom+hsmall);
   glEnd();
}

void mm_reshape(int w, int h)
{
   glViewport(0, 0, (GLsizei) w, (GLsizei) h);
   glMatrixMode(GL_PROJECTION);
   glLoadIdentity();

   glOrtho(config_info.extent_left,config_info.extent_right,
           -config_info.extent_top,-config_info.extent_bottom,
           -1,1);
   glMatrixMode(GL_MODELVIEW);
   glLoadIdentity();
}

void mm_mouse(int button, int state, int x, int y)
{
   if(state==GLUT_DOWN&&
      button==GLUT_LEFT_BUTTON)
      mm_moving=true;

   if(state==GLUT_UP&&
      button==GLUT_LEFT_BUTTON)
   {
      // vvvv Changes to mm_mouse_motion affect this
      mm_mouse_motion(x,y);
      mm_moving=false;
   }
}

// See mm_mouse before making changes
void mm_mouse_motion(int x, int y)
{
   int w,h;
   float xspan,yspan;

   if(mm_moving)
   {
      w=glutGet((GLenum)GLUT_WINDOW_WIDTH);
      h=glutGet((GLenum)GLUT_WINDOW_HEIGHT);
      
      xspan=(float)x/w;
      yspan=(float)y/h;

      offsetx=(1-xspan)*config_info.extent_left+
              xspan*config_info.extent_right;
      offsety=-(1-yspan)*config_info.extent_bottom-
              xspan*config_info.extent_top;

      // Should be here, but this will only affect the minimap,
      //  won't refresh main window. Need to find out how to do
      //  that.
      // glutPostRedisplay();
   }

   return;
}
