#ifndef _drawstuff_h_
#define _drawstuff_h_

#include <GL/glut.h>
#include <GL/gl.h>
#include <GL/glu.h>
#include <stdio.h>

#ifdef WIN32
#include <time.h>
#else
#include <unistd.h>
#include <sys/time.h>
#include <GL/glx.h>
#endif

#include <stdlib.h>
#include <math.h>
#include <iostream>
#include <time.h>
#include <vector>
#include <stdio.h>
#include <fstream>

#include "glTexture.h"

using namespace std;

GLuint	filter;			// Which Filter To Use 
glTexture texture[6];

GLUquadricObj *quadratic;	// Storage For Our Quadratic Objects

//Light stuff
GLfloat LightAmbient[] = { 1.75f, 1.75f, 1.75f, 1.0f };
GLfloat LightDiffuse[] = { 1.0f, 1.0f, 1.0f, 1.0f };
GLfloat LightPosition[] = { 0.0f, 0.0f, 2.0f, 1.0f };

// Load texture into memory
GLvoid LoadGLTextures() 
{
	texture[0].Load("r.png");
	texture[1].Load("y.png");
	texture[2].Load("g.png");
	texture[3].Load("gray.png");
	texture[4].Load("color.png");
	texture[5].Load("p.png");
}


GLvoid InitGL()	// We call this right after our OpenGL window is created.
{
    // set up light number 1.
    glLightfv(GL_LIGHT1, GL_AMBIENT, LightAmbient);  	// add lighting. (ambient)
    glLightfv(GL_LIGHT1, GL_DIFFUSE, LightDiffuse);  	// add lighting. (diffuse).
    glLightfv(GL_LIGHT1, GL_POSITION,LightPosition); 	// set light position.
    glEnable(GL_LIGHT1);                             	// turn light 1 on.

    /* setup blending */
    glBlendFunc(GL_SRC_ALPHA,GL_ONE);			// Set The Blending Function For Translucency
    glColor4f(1.0f, 1.0f, 1.0f, 0.5);

    quadratic=gluNewQuadric();                  // Create A Pointer To The Quadric Object ( NEW )

    // Can also use GLU_NONE, GLU_FLAT
    gluQuadricNormals(quadratic, GLU_SMOOTH);   // Create Smooth Normals
    gluQuadricTexture(quadratic, GL_TRUE);      // Create Texture Coords ( NEW )
   
} 

void drawNode(int x, int y, int type)
{
	//type 0 source
	//type 1 sink
	//type 2 both
    GLdouble radius = 1.0;
	
	if(type == 0)
		filter = 4;
	if(type == 1)
		filter = 5;

   glEnable(GL_TEXTURE_2D);
	glPushMatrix();
	glLoadIdentity();
	if(type != 2)
	{
		//glBindTexture(GL_TEXTURE_2D, texture[filter]);
		texture[filter].Select();
		glPushMatrix();
		glTranslatef(x,y,0);
		gluSphere(quadratic, radius, 32, 32);
		glPopMatrix();
	}
	else
	{
		//glBindTexture(GL_TEXTURE_2D, texture[4]);
		texture[4].Select();
		glPushMatrix();
		glTranslatef(x,y,0);
		gluSphere(quadratic, radius, 32, 32);
		glPopMatrix();

		//glBindTexture(GL_TEXTURE_2D, texture[5]);
		texture[5].Select();
		glPushMatrix();
		glTranslatef(x+1,y+1,0);
		gluSphere(quadratic, radius, 32, 32);
		glPopMatrix();
	}
        glPopMatrix();
	glDisable(GL_TEXTURE_2D);
}

void drawLink(int startx, int starty, int endx, int endy, float current, float max)
{
	float base, height, theta, dx, dy;
	int slices, stacks;

	//Lets get the texture information
	//0-50 is green
	//51-75 is yellow
	//76-100+ is red

	if(current == 0)
		filter = 3;
	else if( (current / max) <= .5 )
		filter = 2;
	else if( (current / max) <= .75)
		filter = 1;
	else
		filter = 0;

	glEnable(GL_TEXTURE_2D);
	  glPushMatrix();
  	    glLoadIdentity();
	    texture[filter].Select();

	    slices = 15;
	    stacks = 15;
	    base = .5;

	    height = sqrt( (startx - endx) * 
				          (startx - endx) + (starty-endy) * 
				          (starty - endy)) + 1;

	    dx = endx - startx;
	    dy = endy - starty;

	    //The +y direction is down		
	    //Fix that right now
	    dy = -dy;

  	    if( (dx == 0) && (dy > 0) )
		  theta = 270;
	    else if( (dx == 0) && (dy < 0) )
		  theta = 90;
	    else if( (dy == 0) && (dx > 0) )
		  theta = 0;
	    else if( (dy == 0) && (dx < 0) )
		  theta = 180;
	    else if( (dx > 0) && (dy > 0) )
		  theta = 360 - ( atan(dy/dx) * 180 / 3.1415 );
	    else if( (dx > 0) && (dy < 0) )
		  theta = -1 * (atan(dy/dx) * 180 / 3.1415);
	    else if( (dx < 0) && (dy > 0) )
		  theta = 180 - (atan(dy/dx) * 180 / 3.1415);
	    else if( (dx < 0) && (dy < 0) )
		  theta = 90 + ( atan(dx/dy) * 180 / 3.1415 );

	      glTranslatef(startx,starty,0.0f);
	      glRotatef(theta, 0.0, 0.0, 1.0);
	      glRotatef(90.0, 0.0, 1.0, 0.0);
	      glutSolidCone(base,height,slices,stacks);

	  glPopMatrix();
	glDisable(GL_TEXTURE_2D);
}

#endif
