#include "glTexture.h"

#define NOJPEG

#include <stdio.h>
#include <string.h>
#ifdef WIN32
#define strcasecmp(x,y) _strcmpi(x,y)
#endif

#include <png.h>
#ifndef NOJPEG
extern "C" {
#include <jpeglib.h>
}
#endif

#define MAX(X,Y) ( (X>Y)?X:Y )
#define MIN(X,Y) ( (X<Y)?X:Y )

GLuint glTexture::m_currenttexid=0;

glTexture::glTexture()
{
   m_texid=0;
   m_width=0;
   m_height=0;
   *(m_refs=new int)=1;
}

glTexture::glTexture(const glTexture &copy)
{
   m_texid=copy.m_texid;
   m_width=copy.m_width;
   m_height=copy.m_height;
   *(m_refs=copy.m_refs)++;
}

glTexture::~glTexture()
{
   (*m_refs)--;
   if(*m_refs==0)
   {
      glDeleteTextures(1,&m_texid);
      delete m_refs;
   }
}


/*void glImage2dTexture::Draw(const GLfloat x, const GLfloat y)
{
   glPushAttrib(GL_ENABLE_BIT|GL_TEXTURE_BIT|GL_CURRENT_BIT);
   glEnable(GL_TEXTURE_2D);
   glBindTexture(GL_TEXTURE_2D, m_texid);
   glColor3f(1.0,1.0,1.0);
   glBegin(GL_QUADS);
      // Upper-left
      glTexCoord2f(m_texleft,m_textop);
      glVertex3f(x,y,0.0);
      // Upper-right
      glTexCoord2f(m_texright,m_textop);
      glVertex3f(x+m_width,y,0.0);
      // Lower-right
      glTexCoord2f(m_texright,m_texbottom);
      glVertex3f(x+m_width,y+m_height,0.0);
      // Lower-left
      glTexCoord2f(m_texleft,m_texbottom);
      glVertex3f(x,y+m_height,0.0);
   glEnd();
   glPopAttrib();

   return;
}*/

bool glTexture::Load(const char *const fname)
{
   BYTE *image=0;
   const char *ext=0;
   WORD w,h;
   int i;

   // Make new(get rid of reference)
   if(*m_refs>1)
   {
      (*m_refs)--;
      *(m_refs=new int)=1;
      m_texid=0;
   }
   
   // Find extension
   for(i=strlen(fname),ext=fname+i;i>=0;i--,ext--)
      if(*ext=='.')
      {
         ext++;
         break;
      }

   if(i<0)
   {
#ifdef DEBUG
fprintf(stderr, "glTexture::Load - "
                "No extension for file: %s\n",fname);
#endif
      return false;
   }

   // ** Determine filetype here **
#ifndef NOJPEG
   if(!strcasecmp(ext,"jpg")||
      !strcasecmp(ext,"jpeg") )
   {
#ifdef DEBUG
fprintf(stderr, "glTexture::Load - "
                "Loading JPEG\n"    );
#endif
      image=readJPEGImage(fname,w,h);
   }
   else
#endif
   if(!strcasecmp(ext,"png"))
   {
#ifdef DEBUG
fprintf(stderr, "glTexture::Load - "
                "Loading PNG\n"     );
#endif
      image=readPNGImage(fname,w,h);
   }

   if(image)
      MakeTexture(image,w,h);

#ifdef DEBUG
fprintf(stderr, "glTexture::Load - "
                "Deleting image buffer\n");
#endif
   delete [] image;

   return (image!=0);
}

void glTexture::MakeTexture(BYTE *image, const WORD w, const WORD h)
{
   if(!m_texid)
      glGenTextures(1,&m_texid);

#ifdef DEBUG
fprintf(stderr, "glTexture::MakeTexture - "
                "Setting parameters\n"     );
#endif
   glBindTexture (GL_TEXTURE_2D, m_texid);
   m_currenttexid=m_texid;
   glPixelStorei (GL_UNPACK_ALIGNMENT, 1);
   //glTexParameteri (GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_REPEAT);
   glTexParameteri (GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_CLAMP);
   //glTexParameteri (GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_REPEAT);
   glTexParameteri (GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_CLAMP);
   glTexParameteri (GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR);
   //glTexParameteri (GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR);
   glTexParameteri (GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_NEAREST);
   //glTexEnvf (GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_MODULATE);
   glTexEnvf (GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_DECAL);

#ifdef DEBUG
fprintf(stderr, "glTexture::MakeTexture - "
                "Building Mipmaps\n"       );
#endif
   gluBuild2DMipmaps(GL_TEXTURE_2D,GL_RGB5_A1,w,h,GL_RGBA,GL_UNSIGNED_BYTE,image);
   //gluBuild2DMipmaps(GL_TEXTURE_2D,3,w,h,GL_RGB,GL_UNSIGNED_BYTE,image);

   m_width=w;
   m_height=h;

   return;
}

BYTE * glTexture::readJPEGImage(const char *const fname,
                                WORD &w,WORD &h         )
#ifdef NOJPEG
{ return 0; }
#else
{
   struct jpeg_decompress_struct cinfo;
   struct jpeg_error_mgr jerr;
   BYTE channels;
   JSAMPLE *row=0;
   BYTE *image=0,*curimage=0,
        *currow=0;
   FILE *infile=0;
   unsigned int i,i1;

   // Read JPEG in
   cinfo.err = jpeg_std_error(&jerr);
   jpeg_create_decompress(&cinfo);

   if ((infile = fopen(fname, "rb")) == NULL)
   {
#ifdef DEBUG
fprintf(stderr, "glTexture::readJPEGImage - "
                "can't open %s\n", fname);
#endif
      return 0;
   }

#ifdef DEBUG
fprintf(stderr, "glImage2d::readJPEGImage - "
                "Initial file read\n");
#endif
   jpeg_stdio_src(&cinfo, infile);
   jpeg_read_header(&cinfo, TRUE);
   w=cinfo.image_width;
   h=cinfo.image_height;
   channels=cinfo.num_components;

#ifdef DEBUG
fprintf(stderr, "glTexture::readJPEGImage - "
                "Width: %i  Height: %i  Channels: %i\n",w,h,channels);
#endif

	// Allocate memory
#ifdef DEBUG
fprintf(stderr, "glTexture::readJPEGImage - "
		        "Allocating memory\n");
#endif
	image=new BYTE[w*h*4];
	row=new JSAMPLE[w*channels]; // BYTE also

#ifdef DEBUG
fprintf(stderr, "glTexture::readJPEGImage - "
		        "Decompressing\n"    );
#endif
   jpeg_start_decompress(&cinfo);
   curimage=image;
   for(i=0;i<h;i++)
   {
      jpeg_read_scanlines(&cinfo,&row,1);

      // reformat to RGBA
      currow=(BYTE*)row;
      for(i1=0;i1<w;i1++)
         switch(channels)
         {
            case 3: // RGB
               *(curimage++)=*(currow++); // Red
               *(curimage++)=*(currow++); // Green
               *(curimage++)=*(currow++); // Blue
               *(curimage++)=0xFF;        // Alpha
               break;

            case 2: // Treat like greyscale
               currow++;
            case 1: // Greyscale
               *(curimage++)=*currow;     // Red
               *(curimage++)=*currow;     // Green
               *(curimage++)=*(currow++); // Blue
               *(curimage++)=0xFF;        // Alpha
               break;

            default:  // Make white image
               *(curimage++)=0xFF; // Red
               *(curimage++)=0xFF; // Blue
               *(curimage++)=0xFF; // Green
               *(curimage++)=0xFF; // Alpha
         }
      }

#ifdef DEBUG
fprintf(stderr, "glTexture::readJPEGImage - "
                "Destroying JPEG objects\n");
#endif
   jpeg_finish_decompress(&cinfo);
   jpeg_destroy_decompress(&cinfo);

#ifdef DEBUG
fprintf(stderr, "glTexture::readJPEGImage - "
                "Closing file handle\n");
#endif
   fclose(infile);

   delete [] row;

#ifdef DEBUG
fprintf(stderr, "glTexture::readJPEGImage - "
                "Returning\n");
#endif

   return image;
}
#endif

BYTE * glTexture::readPNGImage(const char *const fname,
                               WORD &w,WORD &h         )
{
   FILE *infile;
   BYTE header[8];
   BYTE *image=0;
   png_structp png_ptr;
   png_infop info_ptr;
   png_infop end_info;
   DWORD width, height;
   int bit_depth, color_type;
   WORD i;

   if ((infile = fopen(fname, "rb")) == NULL)
   {
#ifdef DEBUG
fprintf(stderr, "glTexture::readPNGImage - "
                "can't open %s\n", fname);
#endif
      return 0;
   }

   // *********************************
   // * Copied loosely from Libpng docs
   // *********************************
   fread(header, 1, 8, infile);
   if(png_sig_cmp(header, 0, 8))
      return 0;

   if(!(png_ptr=png_create_read_struct(PNG_LIBPNG_VER_STRING,
                                       (png_voidp)0,0,0      )))
   {
      fclose(infile);
      return 0;
   }

   if(!(info_ptr=png_create_info_struct(png_ptr)))
   {
      png_destroy_read_struct(&png_ptr,0,0);
      fclose(infile);
      return 0;
   }

   if(!(end_info=png_create_info_struct(png_ptr)))
   {
      png_destroy_read_struct(&png_ptr, &info_ptr,0);
      fclose(infile);
      return 0;
   }

   if (setjmp(png_jmpbuf(png_ptr)))
   {
      png_destroy_read_struct(&png_ptr, &info_ptr,&end_info);
      fclose(infile);
      return 0;
   }

   png_init_io(png_ptr,infile);
   png_set_sig_bytes(png_ptr,8);
   png_set_keep_unknown_chunks(png_ptr,0,0,0);

   png_read_info(png_ptr, info_ptr);
   png_get_IHDR(png_ptr, info_ptr, &width, &height,
                &bit_depth,&color_type,0,0,0       );
   w=width;
   h=height;

   // Format image to RGBA
   if (bit_depth<8)
      png_set_packing(png_ptr);

   if (color_type==PNG_COLOR_TYPE_PALETTE)
      png_set_palette_to_rgb(png_ptr);

   if ((color_type==PNG_COLOR_TYPE_GRAY)||
       (color_type == PNG_COLOR_TYPE_GRAY_ALPHA))
   {
      if(bit_depth<8)
         png_set_gray_1_2_4_to_8(png_ptr);

      png_set_gray_to_rgb(png_ptr);
   }

   if (png_get_valid(png_ptr,info_ptr,PNG_INFO_tRNS))
      png_set_tRNS_to_alpha(png_ptr);

   if (bit_depth == 16)
      png_set_strip_16(png_ptr);

   if ((bit_depth==8)&&(color_type==PNG_COLOR_TYPE_RGB))
       png_set_filler(png_ptr,0xFF,PNG_FILLER_AFTER);
   // Done formatting

   // Read image
#ifdef DEBUG
fprintf(stderr, "glTexture::readPNGImage - "
                "Width: %i  Height: %i\n",w,h);
#endif

   // Allocate memory
#ifdef DEBUG
fprintf(stderr, "glTexture::readPNGImage - "
                "Allocating memory\n");
#endif
   image=new BYTE[w*h*4];

#ifdef DEBUG
fprintf(stderr, "glTexture::readPNGImage - "
                "Decompressing\n");
#endif
   for(i=0;i<h;i++)
      png_read_row(png_ptr, &image[i*w*4], NULL);

   // Clean up
#ifdef DEBUG
fprintf(stderr, "glTexture::readPNGImage - "
                "Destroying PNG objects\n"  );
#endif
    png_destroy_read_struct(&png_ptr, &info_ptr,&end_info);
#ifdef DEBUG
fprintf(stderr, "glTexture::readPNGImage - "
                "Closing file handle\n"      );
#endif
    fclose(infile);

#ifdef DEBUG
fprintf(stderr, "glTexture::readPNGImage - "
                "Returning\n");
#endif
   return image;
}

void glTexture::Quad(GLfloat x1,GLfloat y1,GLfloat s1,GLfloat t1,
                     GLfloat x2,GLfloat y2,GLfloat s2,GLfloat t2,
                     GLfloat x3,GLfloat y3,GLfloat s3,GLfloat t3,
                     GLfloat x4,GLfloat y4,GLfloat s4,GLfloat t4 )
{
   Select();
   glBegin(GL_QUADS);
      glTexCoord2f(s1,t1);
      glVertex2f(x1,y1);
      glTexCoord2f(s2,t2);
      glVertex2f(x2,y2);
      glTexCoord2f(s3,t3);
      glVertex2f(x3,y3);
      glTexCoord2f(s4,t4);
      glVertex2f(x4,y4);
   glEnd();

   return;
}

