/********************************************************************
 ** master.h
 ********************************************************************/
#include "master.h"
#include "container.h"

guiContainer::guiContainer(){
  ControlCount = 0;
  AcceptNulls = false;
}

guiContainer::~guiContainer() {
  ControlList::const_iterator i;

  for(i=controls.begin(); i != controls.end(); i++) {
    delete i->binding;
  }

}


/* 
 ** guiContainer Packing Functions.
 ** Repack is a general function that allow me to 
 ** not rewriten the pack funciton our reshape functions
 ** so drastically.  Instead to arrange controls, just
 ** call RePack();
 */
void guiContainer::Pack(guiControl *target) {
  control_info new_control={0,0,0,0,0};

  if (!target && !AcceptNulls) return;

  new_control.binding=target;

  controls.push_back(new_control);

  if (target)
    glApplication.Request_SubWindow(this, target);

  Repack();
}

void guiContainer::Repack(){
  ControlList::iterator i;

  for(i=controls.begin(); i != controls.end(); i++) {
    if (i->binding) {
      i->x = i->binding->get_xHint(Width, Height);
      i->y = i->binding->get_yHint(Width, Height);
      i->w = i->binding->get_wHint(Width, Height);
      i->h = i->binding->get_hHint(Width, Height);
      i->binding->Place(i->x,i->y);
      i->binding->Reshape(i->w,i->h);
    }
  }
}

/***************************************************************
 ** Callback Functions
 **************************************************************/
void guiContainer::Display(void) {
  if(glApplication.isStarted())
  {
  Push();

  /*clear back ground*/
  glMatrixMode(GL_PROJECTION);
  glLoadIdentity();
  glViewport(0,0,Width,Height);
  glMatrixMode(GL_MODELVIEW);
  glLoadIdentity();
  glClearColor(bg_color.r, bg_color.g, bg_color.b, bg_color.a);
  glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);

  /*call background callback function*/
  if (bg_display) {
//    glPushMatrix();
    (*bg_display)();
//    glPopMatrix();
  }

  /*grey it out if needed*/
  if (!Enabled)
    GreyOut();

  /*commit drawing*/
  glutSwapBuffers();

  /*draw contained objects*/
  controls_Display();

  Pop();
  }
}

void guiContainer::Idle(void) {
  Push();

  /*
   **push and pop are inside since it is silly to use them
   **with a function that does nothing when there is no
   **background call back function
   */

  if (bg_idle){

    (*bg_idle)();

  }
  controls_Idle();
  Pop();
}

void guiContainer::Reshape(int w, int h) {
  Push();

  if (h<1) h = 1;
  if (w<1) w = 1;
  Height = h;
  Width = w;

  glutReshapeWindow(Width, Height);
  glMatrixMode(GL_PROJECTION);
  glLoadIdentity();
  glViewport(0,0,Width,Height);
  gluOrtho2D(0,Width,0,Height);
  glMatrixMode(GL_MODELVIEW);

  if (bg_reshape) {
    (*bg_reshape)(w,h);
  }

  controls_Reshape();

  glutPostRedisplay();

  Pop();
}

void guiContainer::Visibility(int state)
{
  Push();
  if (bg_visibility)
    (*bg_visibility)(state);
  controls_Visibility(state);
  Pop();
}

void guiContainer::setEnabled(bool value)
{
  Enabled = value;
  controls_Enabled(value);
}

/***************************************************************
 ** Callback the Children
 **************************************************************/
void guiContainer::controls_Display(void) {
  ControlList::const_iterator i;

  for(i=controls.begin(); i != controls.end(); i++) {
    if (i->binding) {
      i->binding->Touch();
    }
  }
} 

void guiContainer::controls_Idle(void) {
  ControlList::const_iterator i;

  for(i=controls.begin(); i != controls.end(); i++) {
    if (i->binding) {
      i->binding->Idle();
    }
  }
} 

void guiContainer::controls_Visibility(int state) {
  ControlList::const_iterator i;

  for(i=controls.begin(); i != controls.end(); i++) {
    if (i->binding) {
      i->binding->Visibility(state);
    }
  }
} 

void guiContainer::controls_Enabled(int state) {
  ControlList::const_iterator i;

  for(i=controls.begin(); i != controls.end(); i++) {
    if (i->binding) {
      i->binding->setEnabled(state);
    }
  }
} 

void guiContainer::controls_Reshape() {
  ControlList::const_iterator i;

  Repack();

  for(i=controls.begin(); i != controls.end(); i++) {
    if (i->binding) {
      i->binding->Reshape(i->w, i->h);
    }
  }
} 

