#include "master.h"
#include "control.h"
#include <iostream>
#include <GL/gl.h>

guiControl::guiControl() {
   isInit = 0;
   Parent_ID = -1;
   Window_ID = -1;
   Width = 1;
   Height = 1;
   Cursor = -1;
   State = 0;
   Visible = 1;
   Enabled = 1;
   bg_color.r = 0.0f;
   bg_color.g = 0.0f; 
   bg_color.b = 0.0f;
   bg_display = 0;
   bg_idle = 0;
   bg_mouse = 0;
   bg_motion = 0;
   bg_passivemotion = 0;
   bg_keyboard = 0;
   bg_special = 0;
   bg_visibility = 0;
   bg_reshape = 0;
   bg_entry = 0;
   xHint = 1;
   yHint = 1;
   wHint = 1;
   hHint = 1;
   resetIdle();
   xHintFunc = yHintFunc = hHintFunc = wHintFunc = 0;
}

guiControl::~guiControl(){
  if (Window_ID >= 0);
   glApplication.Destroy_Window(Window_ID);
}
void guiControl::setVisible(bool value)
{
  if (Visible != value) {
    Push();
    Visible = value;
    if(Visible) {
      glutShowWindow();
    }
    else {
      glutHideWindow();
    }
    Pop();
  }
}

void guiControl::resetIdle()
{
  isIdling = 0;
}

void guiControl::setEnabled(bool value)
{
   Enabled = value;
   /*
   **here see if there is a way to mix in grey with everything
   **drawn in this window
   */
}

void guiControl::Init(int ID, int Parent, int w, int h){
   isInit = 1;
   Window_ID = ID;
   Parent_ID = Parent;
   Width = w;
   Height = h;
   glApplication.SetupWindow(Window_ID);
}

/***************************************************************
** Call back functions
**************************************************************/
void guiControl::Display(void) {
if (glApplication.isStarted()) {
  Push();
  
  /*clear to background color*/
  glMatrixMode(GL_PROJECTION);
  glLoadIdentity();
  glViewport(0,0,Width,Height);
  glMatrixMode(GL_MODELVIEW);
  glLoadIdentity();
  glClearColor(bg_color.r, bg_color.g, bg_color.b, bg_color.a);
  glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
  
  /*call the background display function if it is there*/
  if (bg_display)
    (*bg_display)();
  
  /*grey it out if needed*/
  if (!Enabled)
    GreyOut();
  
  /*display drawing*/
  glutSwapBuffers();
  
  Pop();
}
}

void guiControl::Idle() {
  if (bg_idle) {
    Push();  
    /*
    **push and pop are inside since it is silly to use them
    **with a function that does nothing when there is no
    **background call back function
    */
    (*bg_idle)();
    Pop();
  }
}

void guiControl::Mouse(int key, int state, int x, int y) {
  if (!Enabled) return;
  if (bg_mouse) {
    Push();
    (*bg_mouse)(key,state,x,y);
    Pop();
  }
}

void guiControl::Motion(int x, int y) {
  if (!Enabled) return;
  if (bg_motion) {
    Push();
    (*bg_motion)(x,y);
    Pop();
  }
}

void guiControl::PassiveMotion(int x, int y) {
  if (!Enabled) return;
  if (bg_passivemotion) {
    Push();
    (*bg_passivemotion)(x,y);
    Pop();
  }
}

void guiControl::Keyboard(unsigned char key, int x, int y) {
  if (!Enabled) return;
  if (bg_keyboard) {
    Push();
    (*bg_keyboard)(key,x,y);
    Pop();
  }
}

void guiControl::Special(int key, int x, int y) {
  if (!Enabled) return;
  if (bg_special) {
    Push();
    (*bg_special)(key,x,y);
    Pop();
  }
}

void guiControl::Visibility(int state) {
  if (bg_visibility) {
    Push();
    (*bg_visibility)(state);
    Pop();
  }
}

void guiControl::Reshape(int w, int h) {
  Push();
  
  /*set height and width resonablly*/
  if (h<1) h = 1;
  if (w<1) w = 1;
  Height = h;
  Width = w;
  
  /*reshape everything*/
  glutReshapeWindow(Width, Height);
  glMatrixMode(GL_PROJECTION);
  glLoadIdentity();
  glViewport(0,0,Width,Height);
  gluOrtho2D(0,Width,0,Height);
  glMatrixMode(GL_MODELVIEW);

  /*call the background reshape function*/
  if (bg_reshape) {
    (*bg_reshape)(w,h);
  }
 
 /* 
  if(glApplication.isStarted()) {
    Touch();
  } 
  */

  Pop();
}

void guiControl::Entry(int state) {
  if (bg_entry) {
    Push();
    (*bg_entry)(state);
    Pop();
  }
}

void guiControl::Place(int x, int y) {
  Push();
  glutPositionWindow(x,y);
  Pop();
}

void guiControl::SetCursor(int value) {
	Push();
	glutSetCursor(value);
	Pop();
}
