/********************************************************************
** master.cpp
**-------------------------------------------------------------------
** This file contains most of the code for the member functions and
** other help functions related to the master object of the GUI
** opengl frame work.
********************************************************************/

#include "master.h"
#include "control.h"
#include "container.h"

#include <iostream>

#define SW_TO_CURRENT(x)   (Find_by_ID(glutGetWindow()))->x;

/*THE Application object for the Framework*/
guiMaster glApplication;
                          
/********************************************************************
 ** Wrapper Functions to C++ Callbacks
 *******************************************************************/
void glApplication_Display(void) {
   glApplication.Display();
}

void glApplication_Timer(int value) {
   glApplication.Idle();
  
   if (glApplication.IdleRate > 0) {
    glutTimerFunc(glApplication.IdleRate, glApplication_Timer,0);
 //   cout << " ResetTimerFunc  " << glApplication.IdleRate << endl;
  }    
}

void glApplication_Idle() {
   glApplication.Idle();
}

void glApplication_Mouse(int key, int state, int x, int y) {
   glApplication.Mouse(key,state,x,y);
}

void glApplication_Motion(int x, int y) {
   glApplication.Motion(x,y);
}

void glApplication_PassiveMotion(int x, int y) {
   glApplication.PassiveMotion(x,y);
}

void glApplication_Keyboard(unsigned char key, int x, int y) {
   glApplication.Keyboard(key,x,y);
}

void glApplication_Special(int key, int x, int y) {
   glApplication.Special(key,x,y);
}

void glApplication_Visibility(int state) {
   glApplication.Visibility(state);
}

void glApplication_Reshape(int w, int h) {
   glApplication.Reshape(w,h);
}

void glApplication_Entry(int state) {
   glApplication.Entry(state);
}

/********************************************************************
 ** guiMaster member Functions
 *******************************************************************/

/******************************************************
 ** guiMaster::guiMaster()
 **-----------------------------------------------------
 ** Pre: none
 ** Post: Mouse_Info Returns 0,
 **       CurrentControl returns NULL
 ******************************************************/

guiMaster::guiMaster(void) {
  MouseState = 0;
  Started = 0;
}

/******************************************************
 ** guiMaster::~guiMaster()
 ** ----------------------------------------------------
 **            ** Pre: none
 ** Post: recursivly destory each standalone window
 ******************************************************/
guiMaster::~guiMaster(void) {
  WindowMap::const_iterator i;

  /*iterate over the map and delete the top level controls**
   **do not have to worry about rmove from map because     **
   **it's next on garabage collection                      */
  for(i = windows.begin(); !windows.empty(); i = windows.begin()){
    delete i->second;
  }

}

/******************************************************
** void guiMaster::SetupWindow(int winID)
**-----------------------------------------------------
** pre: winID is a valid glut Window ID.
** post: the frameworks wrapper functions are set as
**       the callbacks for the winID
******************************************************/
void guiMaster::SetupWindow(int winID) {
  glutSetWindow(winID);
  glutDisplayFunc(glApplication_Display);
  glutMouseFunc(glApplication_Mouse);
  glutMotionFunc(glApplication_Motion);
  glutPassiveMotionFunc(glApplication_PassiveMotion);
  glutKeyboardFunc(glApplication_Keyboard);
  glutSpecialFunc(glApplication_Special);
  glutVisibilityFunc(glApplication_Visibility);
  glutReshapeFunc(glApplication_Reshape);
  glutEntryFunc(glApplication_Entry);
  /*note that this doesn't setup idleing*/
}

/******************************************************
** guiControl *guiMaster::Find_by_ID(int ID)
**-----------------------------------------------------
** pre: ID is a valid glut Window ID
** post: the corresponding control is returned.
******************************************************/
guiControl *guiMaster::Find_by_ID(int ID){
  guiControl *returnval = 0;
  ControlMap::const_iterator result;

  result = controls.find(ID);
  if (result != windows.end())
    returnval = result->second;

  return returnval;
}

/******************************************************
** void guiMaster::SetIdle(int rate)
**-----------------------------------------------------
** pre: Init should be called
** post: if int > 0 the IDLE rate is set to this
**       else if int == 0 the IDLE loops is continuous
**       else the idle loop is disabled.
**       NOTE: rate is in milliseconds
******************************************************/
void guiMaster::SetIdle(int rate)
{
  IdleRate = rate;
  
  if (IdleRate>0) {
    glutIdleFunc(0);
    glutTimerFunc(IdleRate, glApplication_Timer,0);
//    cout << "Time Function  " << IdleRate <<endl;
  }
  else if (IdleRate==0){
    glutIdleFunc(glApplication_Idle);
  }
  else{
    glutIdleFunc(0);
  }
    
}

/*
** Control use this to request and receive a Standalone window
*/
int guiMaster::Request_Window(char name[], guiControl *binding) {
  int ID = -1;
  int w,h;
  w=binding->get_wHint(-1,-1);
  h=binding->get_hHint(-1,-1);

  if (binding) {
    glutInitWindowSize(w,h);
    ID = glutCreateWindow(name);

    binding->Init(ID, -1, w, h);
    controls[ID] = binding;
    windows[ID] = binding;
  }

  return ID;
}

/*
** Contols use this to request and receive a subwindow of
** either a window or another subwindow
*/
int guiMaster::Request_SubWindow(guiContainer *parent, 
                                 guiControl *binding) {
  int ID = -1;
  int x,y,w,h,Parent_ID, Height, Width;


  Parent_ID=parent->get_WindowID();
  Height=parent->get_Height();
  Width=parent->get_Width();

  x=binding->get_xHint(Height,Width);
  y=binding->get_yHint(Height,Width);
  w=binding->get_wHint(Height,Width);
  h=binding->get_hHint(Height,Width);

  if (binding) {
    ID = glutCreateSubWindow(Parent_ID,x,y,w,h);
    binding->Init(ID, Parent_ID,w,h);
    controls[ID] = binding;
  }

  return ID;
}

/*
** Use this to remove the alloted window of a control
** from "memory"
*/
void guiMaster::Destroy_Window(int ID) {
  //cout << "Destroy Window:  " << ID << endl;
  if(controls.find(ID) == windows.end())
    return;

  glutDestroyWindow(ID);
  windows.erase(ID);

  if(windows.find(ID) == windows.end())
    windows.erase(ID);
}

/*
** Control that access their own window should call this
** first in order to let the master object know what is
** going on
*/
void guiMaster::NewContext(guiControl *thiscontrol, int ID) {
  control_context new_context;

  /*create the new context*/
  new_context.WindowID = ID;
  new_context.Current = thiscontrol;

  /*push it on the stack*/
  controlstack.push(new_context);

  /*put it into action*/
  glutSetWindow(ID);
}

/*
** If a control used NewContext this MUST be called when done
** with the master object
*/
void guiMaster::DeleteContext() {

  /*remove the current context*/
  controlstack.pop();

  /*if there is a context, set it back in action*/
  if (!controlstack.empty()) {
   glutSetWindow((controlstack.top()).WindowID);
  }
                 
}

/*
** Idle Request/Relinquish functions
*/
void guiMaster::Request_Idletime(guiControl *to_insert)
{
  if (!to_insert) return;

  IdleControls.push_back(to_insert);
}

void guiMaster::Relenquish_Idletime(guiControl *to_remove)
{
  if (!to_remove) return;

  IdleControls.remove(to_remove);

}

/*
** The functions simply do a lookup and call the right function
*/
void guiMaster::Display(void) {
  if(Started) {
    SW_TO_CURRENT(Display());
  }
}

/* 
** If idling is turn on then this is called a rate milliseconds
** However instead of blinding idling everyone, just idle those who
** have registered Idle() functions.
*/
void guiMaster::Idle() {
  IdleList::iterator i;
  if (app_idle) 
    (*app_idle)();

  if (IdleControls.empty()) return;
  
  for(i=IdleControls.begin(); i != IdleControls.end(); i++) {
      (*i)->Idle();
  }

}

void guiMaster::Mouse(int key, int state, int x, int y) {
  int bitmap = 0;

  /*translates the key press to a bitmap*/
  switch(key) {
     case GLUT_LEFT_BUTTON:
        bitmap = MOUSESTATE_LEFT;
        break;
     case GLUT_RIGHT_BUTTON:
        bitmap = MOUSESTATE_RIGHT;
        break;
     case GLUT_MIDDLE_BUTTON:
        bitmap = MOUSESTATE_MIDDLE;
        break;
  }

  /*turn on or off the bit accordingly*/
  if (state == GLUT_DOWN)
    MouseState = MouseState | bitmap;
  else
    MouseState = MouseState & (~bitmap);

  SW_TO_CURRENT(Mouse(key,state,x,y));
}

void guiMaster::Motion(int x, int y) {
  SW_TO_CURRENT(Motion(x,y));
}

void guiMaster::PassiveMotion(int x, int y) {
  SW_TO_CURRENT(PassiveMotion(x,y));
}

void guiMaster::Keyboard(unsigned char key, int x, int y) {
  SW_TO_CURRENT(Keyboard(key,x,y));
}

void guiMaster::Special(int key, int x, int y) {
  SW_TO_CURRENT(Special(key,x,y));
}

void guiMaster::Visibility(int state) {
  SW_TO_CURRENT(Visibility(state));
}

void guiMaster::Reshape(int w, int h) {
  SW_TO_CURRENT(Reshape(w,h));
}

void guiMaster::Entry(int state) {
  SW_TO_CURRENT(Entry(state));
}


