/*******************************************************************
 ** container.h
 **-----------------------------------------------------------------
 ** Some controls have to have the capability to put group controls
 ** inside themselve.  These are call container and should inherit 
 ** from this class
 ******************************************************************/
#ifndef _CONTAINER_H_
#define _CONTAINER_H_

#include "master.h"
#include "control.h"
#include <list>

using namespace std;

/**********************************************************
 ** The Control Entry for a "Contained" Control
 *********************************************************/
typedef struct _control_info
{
   guiControl *binding;
   int x;
   int y;
   int w;
   int h;
} control_info;

/**********************************************************
** Typedefs
*********************************************************/
typedef list<control_info> ControlList;

class guiContainer:public guiControl
{

 public:
    /******************************************************
    ** guiContainer::guiContainer()
    **----------------------------------------------------
    ** post: in addition to the posts for guiControl,
    **       these truths override guiControl on 
    **       conflicts:
    **       - Control count is 0;
    **       - does not accept nulls as packable objects
    *****************************************************/
    guiContainer();
    /******************************************************
    ** guiContainer::~guiContainer()
    **----------------------------------------------------
    ** post: before the guiControl part is destroyed, 
    **       all contained objects are destroyed
    *****************************************************/
    virtual ~guiContainer();

    /*
    ** Callbacks (they must have Push() and Pop()) that
    ** must be overridden from guiControl
    */
    virtual void Display(void);
    virtual void Idle(void);
    virtual void Reshape(int w, int h);
    virtual void Visibility(int state);
   
    /* 
    ** Enabled must be overidden
    */ 
    virtual void setEnabled(bool value);

    /*
    ** New functions for placing control into the container
    ** NOTE: there are NO x,y,w,h stuff... You want to 
    **       have any thing but the control defaults you
    **       MUST set the [xywh]Hints in that control
    */
    virtual void Pack(guiControl *target);
    virtual void Repack();

 protected:
    
    /*
    ** Functions designed to loop through "children"
    ** and execute proper functions.  No Push/Pop here
    */
    void controls_Display(void);
    void controls_Idle(void);
    void controls_Visibility(int state);
    void controls_Enabled(int state);
    void controls_Reshape();

    bool AcceptNulls;
    int ControlCount;
    ControlList controls;



};

#endif
