/********************************************************************
** control.h
**------------------------------------------------------------------
** This file container the declaration for the abstract data type:
** guiControl.  From guiControl All opengl framework widgets 
** inherit.  
********************************************************************/
#ifndef _CONTROL_H_
#define _CONTROL_H_

class guiControl
{
  /*
  ** guiMaster need access to the hidden Init function so resources
  ** and be assigned to it
  */
  friend class guiMaster;

 public:
    /******************************************************
    ** guiControl::guiControl()
    **----------------------------------------------------
    ** post Conditions: the following are true:
    **       - the Control is not recognized as initalized
    **       - the window and parent ID's are set to 
    **         invalid
    **       - width and height are 1 pixel
    **       - the general state is zero
    **       - the control is visable and enabled
    **       - the background color is black
    **       - there are not "background" callback 
    **         functions
    **       - the x,y,w,h hint constants are 1
    **       - there is no define corresponding hint 
    **         functions
    **       - the control is not idling
    *****************************************************/
    guiControl();
    /******************************************************
    ** guiControl::~guiControl()
    **----------------------------------------------------
    ** post conditions: if the control has a valid glut
    **   window assigned to it, it is destroied.
    *****************************************************/
    virtual ~guiControl();

    
   
    /*
    ** "Callback functions"
    **
    ** Each one must obtain its own control context from
    ** the master object. so control to control called 
    ** work with out strange problems.
    **
    ** Also any each own must assume that the its 
    ** "Background" callback function screwed with the
    ** window ID
    */
    virtual void Display(void);
    virtual void Idle(void);
    virtual void Mouse(int key, int state, int x, int y);
    virtual void Motion(int x, int y);
    virtual void PassiveMotion(int x, int y);
    virtual void Keyboard(unsigned char key, int x, int y);
    virtual void Special(int key, int x, int y);
    virtual void Visibility(int state);
    virtual void Reshape(int w, int h);
    virtual void Entry(int state);
    virtual void Place(int x, int y);
    
    /*
    ** These are class information retrieval functions
    ** Push and Pop are not needed
    */
    int get_WindowID();
    int get_ParentID();
    int get_Width();
    int get_Height();
    int get_Cursor();
    int get_hHint(int w, int h);
    int get_wHint(int w, int h);
    int get_xHint(int w, int h);
    int get_yHint(int w, int h);
    int get_Idling();
    color_t get_bgColor();
    bool isVisible();
    bool isEnabled();
    
    /*
    ** These are attibute setting functions
    ** the only one that has to have push and pop 
    ** is set_Visible(bool)
    */
    void setIdle(bool value);
    void setVisible(bool value);
    virtual void setEnabled(bool value);
    void setbg_Display(void (*func)(void));
    void setbg_Idle(void (*func)(void));
    void setbg_Mouse(void (*func)(int,int,int,int));
    void setbg_Motion(void (*func)(int,int));
    void setbg_Passivemotion(void (*func)(int,int));
    void setbg_Keyboard(void (*func)(unsigned char,int,int));
    void setbg_Special(void (*func)(int,int,int));
    void setbg_Visibility(void (*func)(int));
    void setbg_Reshape(void (*func)(int,int));
    void setbg_Entry(void (*func)(int));
    void setbg_Color(color_t value);
    void set_hHint(int val);
    void set_wHint(int val);
    void set_xHint(int val);
    void set_yHint(int val);
    void set_hHint(int (*val)(int,int));
    void set_wHint(int (*val)(int,int));
    void set_xHint(int (*val)(int,int));
    void set_yHint(int (*val)(int,int));
	 void SetCursor(int value);         
    void Touch();
     
 protected:
           
    /*
    ** Push/Pop add and remove itself from the current
    ** control stack, and Push must be used first at 
    ** the begining of a function, and pop is paired 
    ** with push at the time when the window resoruces
    ** are no longer needed
    */
    void Push();
    void Pop();
    
    /*use this ONLY is existing functions that set the window*/
    void GreyOut();
    
    /*
    ** This gets called to grant window resources
    ** during mapping
    */
    virtual void Init(int ID, int Parent, int w, int h);
    
    /* 
    ** This resets the Idle default for a given control
    */
    virtual void resetIdle();

    int isInit;
    int isIdling;
    int Parent_ID;
    int Window_ID;
    int Width;
    int Height;
    int Cursor;
    int State;
    int Visible;
    int Enabled;
    int xHint;
    int yHint;
    int hHint;
    int wHint;


    int (*xHintFunc)(int,int);
    int (*yHintFunc)(int,int);
    int (*hHintFunc)(int,int);
    int (*wHintFunc)(int,int);
    
    color_t bg_color;
    void (*bg_display)(void);
    void (*bg_idle)(void);
    void (*bg_mouse)(int,int,int,int);
    void (*bg_motion)(int,int);
    void (*bg_passivemotion)(int,int);
    void (*bg_keyboard)(unsigned char,int,int);
    void (*bg_special)(int,int,int);
    void (*bg_visibility)(int);
    void (*bg_reshape)(int,int);
    void (*bg_entry)(int);
   
};

inline void guiControl::Touch() {
  Push();
  glutPostRedisplay();
  Pop();
}

inline int guiControl::get_Idling() {
  return isIdling;
};

inline void guiControl::setIdle(bool value) {
  /*If were are going to idle request to*/
  if (value) {
    glApplication.Request_Idletime(this);
  }
  /*but if not and we currently are relenquish*/
  else if (isIdling) {
    glApplication.Relenquish_Idletime(this);
  }
  
  /*by now all the needed stuff is done to a assignment is ok*/
  isIdling = value;
      
};

inline void guiControl::GreyOut()
{
  /*basic draw a grey rectangle with 25% opacity*/
  glBegin(GL_POLYGON);
  glColor4f(0.5,0.5,0.5,0.25);
  glVertex2f(0,0);
  glVertex2f(0,Height);
  glVertex2f(Width,Height);
  glVertex2f(Width,0);
  glColor4f(1.0,1.0,1.0,1.0);
  glEnd();
};

inline void guiControl::Push() {
  glApplication.NewContext(this, Window_ID);
};

inline void guiControl::Pop() {
  glApplication.DeleteContext();
}

inline int guiControl::get_WindowID() {
  return Window_ID;
};

inline int guiControl::get_Width() {
  return Width;
};

inline int guiControl::get_Height() {
  return Height;
};

inline int guiControl::get_Cursor() {
  return Cursor;
};

inline color_t guiControl::get_bgColor() {
  return bg_color;
};

inline bool guiControl::isVisible() {
  return Visible;
};

inline bool guiControl::isEnabled() {
  return Enabled;
};

inline void guiControl::setbg_Display(void (*func)(void)){
  bg_display = func;
};

inline void guiControl::setbg_Idle(void (*func)(void)) {
  /*
  ** set someone is setting the Idle function chances are
  ** that they want to enable idling, and if the are
  ** removing the idle function then chances are they 
  ** we need to reset to the control's default
  */
  if (func)
    setIdle(1);
  else
    resetIdle();
    
  bg_idle = func;
};

inline void guiControl::setbg_Mouse(void (*func)(int,int,int,int)){
  bg_mouse = func;
};

inline void guiControl::setbg_Motion(void (*func)(int,int)){
  bg_motion = func;
};

inline void guiControl::setbg_Passivemotion(void (*func)(int,int)){
  bg_passivemotion = func;
};

inline void guiControl::setbg_Keyboard(
                              void (*func)(unsigned char,int,int)){
  bg_keyboard = func;
};

inline void guiControl::setbg_Special(void (*func)(int,int,int)){
  bg_special = func;
};

inline void guiControl::setbg_Visibility(void (*func)(int)){
  bg_visibility = func;
};

inline void guiControl::setbg_Reshape(void (*func)(int,int)){
  bg_reshape = func;
};

inline void guiControl::setbg_Entry(void (*func)(int)){
  bg_entry = func;
};

inline void guiControl::setbg_Color(color_t value){
  bg_color.r = value.r;
  bg_color.g = value.g;
  bg_color.b = value.b;
  bg_color.a = value.a;
};

inline void guiControl::set_hHint(int val) {
  hHint = val;
  hHintFunc = 0;
}
inline void guiControl::set_wHint(int val){
  wHint = val;
  wHintFunc = 0;
}
inline void guiControl::set_xHint(int val){
  xHint = val;
  xHintFunc = 0;
}

inline void guiControl::set_yHint(int val){
  yHint = val;
  yHintFunc = 0;
}

inline void guiControl::set_hHint(int (*val)(int,int)) {
  hHintFunc = val;
  hHint = -1;
}
inline void guiControl::set_wHint(int (*val)(int,int)) {
  wHintFunc = val;
  wHint = -1;
}
inline void guiControl::set_xHint(int (*val)(int,int)) {
  xHint = -1;
  xHintFunc = val;
}

inline void guiControl::set_yHint(int (*val)(int,int)) {
  yHint = -1;
  yHintFunc = val;
}

inline int guiControl::get_hHint(int w, int h)
{
  if (hHintFunc) 
    return (*hHintFunc)(w,h);
  else
    return hHint;
};

inline int guiControl::get_wHint(int w, int h)
{
  if (wHintFunc) 
    return (*wHintFunc)(w,h);
  else
    return wHint;
};

inline int guiControl::get_xHint(int w, int h)
{
  if (xHintFunc) 
    return (*xHintFunc)(w,h);
  else
    return xHint;
};

inline int guiControl::get_yHint(int w, int h)
{
  if (yHintFunc) 
    return (*yHintFunc)(w,h);
  else
    return yHint;
};
#endif
