/*Client implemented as a function. data is the pointer to the data to 
transfer to the server. size is the size of that data. server name is the name 
of the computer acting as the server (ex: ultra14.cs.umr.edu) and portno is 
the port number to connect to that server. if this function is successful it 
returns 1 and the data pointer will point to the data returned by the server 
and size will be the size of that data.
******************************************************************************/  
#include "iclient.h"

#include <sys/types.h>
#include <stdio.h>
#include<stdlib.h>

#ifdef WIN32
#define WIN32_LEAN_AND_MEAN
#include <windows.h>
#include <winsock2.h>
#include <string.h>
#define bcopy(s1,s2,n) memcpy(s2,s1,n)
#define bzero(s,n) memset(s,0,n)
#define bcmp(s1,s2,n) memcmp(s1,s2,n)
typedef SOCKET socktype_t;
typedef char *socketdata_t;
#define close(fd) closesocket(fd)
#define write(fd,buf,size) send(fd,buf,size,0)
#define read(fd,buf,size) recv(fd,buf,size,0)
#else
#include <unistd.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <netdb.h>
#include <strings.h>
typedef int socktype_t;
typedef void *socketdata_t;
#endif

int client(const char* data, long & size, char** rcvdata, char* server_name, 
int portno) {

    socktype_t sockfd;
    int n;
    struct sockaddr_in serv_addr;
    struct hostent *server;
    long size_in,sizenl, size_inl;
    //long* psize = new long;
    //long* psize_inl = new long;

    sockfd = socket(AF_INET, SOCK_STREAM, 0);
    if (sockfd < 0 ) 
        return -1;

    server = gethostbyname(server_name);
    if (server == NULL) {
        close(sockfd);
        return -1;
    }

    bzero((char *) &serv_addr, sizeof(serv_addr));
    serv_addr.sin_family = AF_INET;
    bcopy((char *)server->h_addr, 
         (char *)&serv_addr.sin_addr.s_addr,
         server->h_length);
    serv_addr.sin_port = htons(portno);
    if (connect(sockfd,(struct sockaddr *)&serv_addr,sizeof(serv_addr)) < 0) 
    {   close(sockfd); 
        return -1;
    }

    //tell the server how many bytes to expect
    sizenl = htonl(size);
    n = write(sockfd,(socketdata_t)&sizenl,sizeof(size));
    if (n < 0)
    {    close(sockfd);
         return -1;
    }

    //send the data to the server
    n = write(sockfd,data,size);
    
    //how big is the size of the file coming back? get that from the server
    n = read(sockfd,(socketdata_t)&size_inl,sizeof(size_in));
    if (n < 0)
    {    close(sockfd);
         return -1;
    }
	 
    //size_inl = (*psize_inl);
    size_in = ntohl(size_inl);
    if(sizeof(size_in) != n)
    {
       close(sockfd);
       return -1;
    }
    *rcvdata = new char[size_in+1];
    long received = 0;
    long result = 0;
    while(received != size_in)
    {
       received += result;
       result = read(sockfd, (*rcvdata)+received, (size_in-received));
       if (result < 0)
           {
	      close(sockfd);
              delete[] rcvdata;
	      //delete psize;
              return -1;
            }
    }
    received += result;
    (*rcvdata)[received] = NULL;
    size = size_in;

    //delete some pointers
    //delete psize;
    //delete psize_inl;
    //delete[] rcvdata;

    close(sockfd);
    return 1; //success
}

