#include <GL/glut.h>
#include <GL/gl.h>
#include "master.h"
#include "control.h"
#include "window.h"
#include "status.h"
#include "philxpm.h"
#include "quit.Xpm"
#include "on_off.Xpm"
#include "off_on.Xpm"
#include "recalc.Xpm"
#include "revert.Xpm"
#include "info.Xpm"
#include "flag3d.Xpm"
#include <iostream>
#include <stdlib.h>
#ifndef WIN32
#include <unistd.h>
#endif
#include <stdio.h>
#include <iostream>
#include "display/display.h"
#include "display/config.h"
#include "substuff.h"
#include "network/iclient.h"

extern guiWindow *Main;
extern guiStatusBar *sBar1;


philxpm Quit_PhilXpm, 
        On_Off_PhilXpm,
        Recalc_PhilXpm,
        Revert_PhilXpm,
        Off_On_PhilXpm,
        Info_PhilXpm,
		  Flag3d_PhilXpm;

void idle()
{
	static int wait=10;
	
	if (wait) {
		wait--;
		return;
	}
	//Yeah this is a hack to prevent drawinf before a window appears on slow
	//machines, need to check opengl docs
        if(glApplication.isStarted()) {
                glFinish(); //wait till everything has completed
		          glutPostRedisplay();
        }
}

void 
draw_icon(philxpm *icon, int centerx, int centery);

void
init_icons() {
  int ret;
  
  if (!(ret = Quit_PhilXpm.parse(quit_Xpm))) {
    if (!(ret = On_Off_PhilXpm.parse(on_off_Xpm))) {
      if (!(ret = Recalc_PhilXpm.parse(recalc_Xpm))) {
        if (!(ret = Revert_PhilXpm.parse(revert_Xpm))) {
          if (!(ret = Off_On_PhilXpm.parse(off_on_Xpm))) {
            if(!(ret = Info_PhilXpm.parse(info_Xpm))) {
					ret = Flag3d_PhilXpm.parse(flag3d_Xpm);
				}
          }
        }
      }
    }
  }
  

  if (ret)
  {
    switch (ret) {
       case 1:
          cout << "First Line of Xpm Invalid" << endl;
          break;
       case 2:
          cout << "XPM is not a 1 cpp image." << endl;
          break;
       case 3:
          cout << "Error in Color Table." << endl;
          break;
       case 4:
          cout << "Error in Color Value." << endl;
    }
    exit(ret);
  }
}

void
draw_Off_On_Icon(int w, int h, int border) {
  int cx,cy;
  cx = (int)(w/2.00f);
  cy = (int)(h/2.00f);
  draw_icon(&Off_On_PhilXpm, cx, cy);
}

void
draw_On_Off_Icon(int w, int h, int border) {
  int cx,cy;
   cx = (int)(w/2.00f);
   cy = (int)(h/2.00f);
   draw_icon(&On_Off_PhilXpm, cx, cy);
}

void
draw_Revert_Icon(int w, int h, int border) {
  int cx,cy;
   cx = (int)(w/2.00f);
   cy = (int)(h/2.00f);
   draw_icon(&Revert_PhilXpm, cx, cy);
}

void
draw_Recalc_Icon(int w, int h, int border) {
  int cx,cy;
   cx = (int)(w/2.00f);
   cy = (int)(h/2.00f);
   draw_icon(&Recalc_PhilXpm, cx, cy);
}

void
draw_Quit_Icon(int w, int h, int border) {
  int cx,cy;
   cx = (int)(w/2.00f);
   cy = (int)(h/2.00f);
   draw_icon(&Quit_PhilXpm, cx, cy);
}

void
draw_Info_Icon(int w, int h, int border) {
  int cx,cy;
   cx = (int)(w/2.00f);
   cy = (int)(h/2.00f);
   draw_icon(&Info_PhilXpm, cx, cy);
}

void
draw_Flag3d_Icon(int w, int h, int border) {
  int cx,cy;
   cx = (int)(w/2.00f);
   cy = (int)(h/2.00f);
   draw_icon(&Flag3d_PhilXpm, cx, cy);	
}

void 
draw_icon(philxpm *icon, int centerx, int centery)
{
   int icon_x, icon_y,
      xadj, yadj,x,y;
   color_t pixel_color = {0.0f,0.0f,0.0f,0.0f};
   GLubyte pix_todraw[1][1][4];
            
   char curpixel=0, newpixel;

   if (!icon) return;
      
   icon_x = icon->get_width();
   icon_y = icon->get_height();
   xadj = centerx - (int)(icon_x/2.00);
   yadj = centery - (int)(icon_y/2.00);
   

   for(y=0; y<icon_y; y++) {
     for(x=0; x<icon_x; x++) {
       newpixel = icon->get_pixel(x,y);
       if ( !curpixel || (newpixel!=curpixel)) {
         curpixel = newpixel;
         pixel_color = icon->get_color(curpixel);
         pix_todraw[0][0][0] = (GLubyte)(255 * pixel_color.r);
         pix_todraw[0][0][1] = (GLubyte)(255 * pixel_color.g);
         pix_todraw[0][0][2] = (GLubyte)(255 * pixel_color.b);
         pix_todraw[0][0][3] = (GLubyte)(255 * pixel_color.a);
       }
       if ( pixel_color.a > 0.00001) {
         glRasterPos2i(x+xadj, y+yadj);
         glDrawPixels(1,1,GL_RGBA,GL_UNSIGNED_BYTE,pix_todraw);
       }
     }
   }
}

void action_Off_On(void)
{
   enable_all_lines();
   if (glApplication.isStarted()) {
     glutPostRedisplay();
   }
}

void action_On_Off(void)
{ 
   disable_all_lines();
   glFinish(); //wait till everything has completed
   glutPostRedisplay();
}

void action_Recalc(void)
{
	Main->SetCursor(GLUT_CURSOR_WAIT);
	if(networked_backend()) {
		sBar1->setString(31, "Last Backend Calculation Failed");
	}
	else
	{
		sBar1->setString(34 ,"Last Backend Calculation Succeeded");
		recalc_disabled();
	}

        if(glApplication.isStarted()) {
   	  glutPostRedisplay();
        }

	if (infomode) {
#ifdef WIN32
      Main->SetCursor(GLUT_CURSOR_HELP);
#else
		Main->SetCursor(GLUT_CURSOR_INFO);
#endif
	}
	else {
      Main->SetCursor(GLUT_CURSOR_LEFT_ARROW);
   }
}

void action_Revert(void)
{
	action_Off_On();
	action_Recalc();
}

void action_Quit(void)
{
#ifdef WIN32
   _exit(0);
#else
   exit(0);
#endif
}

void action_Info(int state)
{
	if (state)
	{
#ifdef WIN32
      Main->SetCursor(GLUT_CURSOR_HELP);
#else
		Main->SetCursor(GLUT_CURSOR_INFO);
#endif
		infomode = 1;
	}
	else
	{
		Main->SetCursor(GLUT_CURSOR_LEFT_ARROW);
		infomode = 0;
	}
   return;
}

void action_3d(int state)
{
	if (state)	{
		flag3d = 1;
		glShadeModel(GL_SMOOTH);
	}
	else {
		flag3d = 0;
		glShadeModel(GL_FLAT);
	}
	
}
